@@ -1,5 +1,5 @@
 /*
- * $Id: events.c,v 1.8 1998/10/17 06:23:09 gritton Exp $
+ * $Id: events.c,v 1.9 2000/01/10 22:59:03 amai Exp $
  */
 /*****************************************************************************/
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
@@ -1175,8 +1175,7 @@ enter_notify(ScreenInfo *scr, MwmWindow *win, XEvent *event)
 
     if (event->xany.window == scr->root_win)
     {
-	if ((!(Mwm.keyboard_focus_policy == XmEXPLICIT)) &&
-	    (!(Mwm.keyboard_focus_policy == XmPOINTER)))
+	if (!(Mwm.keyboard_focus_policy == XmEXPLICIT))
 	{
 	    WIN_SetFocus(scr, scr->no_focus_win, NULL);
 	    MISC_SetFocusSequence(scr);
