@@ -87,7 +87,7 @@ draw_separator(Window w, GC TopGC, GC BottomGC, int x1, int y1, int x2, int y2,
 /*
  *  Draws a little Triangle pattern within a window
  */
-void
+static void
 draw_arrow(Window w, GC GC1, GC GC2, GC GC3, int l, int u, int r, int b)
 {
     int m;
@@ -149,8 +149,20 @@ function_allowed(MwmWindow *tmp, MenuItem *mi)
     if (mi->func == F_RESIZE && tmp && !(tmp->functions & MWM_FUNC_RESIZE))
 	return False;
 
-    if (mi->func == F_ICONIFY && tmp && !(tmp->flags & ICONIFIED) &&
-	!(tmp->functions & MWM_FUNC_MINIMIZE))
+    /* Cannot iconify if the window is already iconified or the window
+       does not allow minimize */
+    if ((mi->func == F_ICONIFY && tmp) &&
+          ((tmp->flags & ICONIFIED) ||
+           !(tmp->functions & MWM_FUNC_MINIMIZE)))
+	return False;
+    
+    /* Cannot resize if iconified */
+    if (mi->func == F_RESIZE && tmp && (tmp->flags & ICONIFIED))
+        return False;
+
+    /* Can only normalize if iconified or maximized */
+    if (mi->func == F_NORMALIZE && tmp &&
+        !(tmp->flags & ICONIFIED) && !(tmp->flags & MAXIMIZED))
 	return False;
 
     if (mi->func == F_MAXIMIZE && tmp && !(tmp->functions & MWM_FUNC_MAXIMIZE))
@@ -221,9 +233,10 @@ paint_entry(ScreenInfo *scr, MwmWindow *tmp, MenuRoot * mr, MenuItem *mi)
 
     if (function_allowed(tmp, mi))
 	currentGC = scr->components[MWM_MENU].normal_GC;
+    else {
     /* MLM: FIXME: SHOULD GRAY OUT ITEM */
-    else
-	currentGC = scr->components[MWM_MENU].normal_GC;
+	currentGC = scr->components[MWM_MENU].grayed_GC;
+    }
 
     if (*mi->item)
 	XDrawString(dpy, mr->w, currentGC, mi->x, text_y,
@@ -313,7 +326,7 @@ find_entry(ScreenInfo *scr, MwmWindow *tmp)
 
     /* if we weren't on the active item, change the active item and turn it
      * on */
-    if ((mi != ActiveItem) && (mi != NULL))
+    if ((mi != ActiveItem) && (mi != NULL) && function_allowed(tmp, mi))
     {
 	mi->state = 1;
 	paint_entry(scr, tmp, ActiveMenu, mi);
