@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DebugUtil.h,v 1.17 1998/06/14 12:50:34 rwscott Exp $
+ * $Id: DebugUtil.h,v 1.20 2000/02/13 12:43:22 rwscott Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -28,13 +28,8 @@
 #include <LTconfig.h>
 #include <Xm/Xm.h>
 
-#ifdef STDC_HEADERS
 #include <stdarg.h>
 #define Va_start(a,b) va_start(a,b)
-#else /* ! STDC_HEADERS */
-#include <varargs.h>
-#define Va_start(a,b) va_start(a)
-#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -59,6 +54,8 @@ char *XdbFocusMode2String(int type);
 char *XdbEventType2String(int type);
 char *XdbGeoAction2String(int action);
 char *XdbGeometryResult2String(XtGeometryResult r);
+char *XdbDragAndDropMessageType2String(unsigned char r);
+char *XdbDragType2String(unsigned char r);
 char *XdbWidgetGeometry2String(XtWidgetGeometry *g);
 
 char *XdbAttachment2String(int i);
@@ -114,11 +111,6 @@ Boolean XdbInDebug(char *fn, Widget w);
 
 #include <dmalloc.h>
 
-extern XtPointer	XdbMalloc(char *f, int l, int size);
-extern XtPointer	XdbCalloc(char *f, int l, int count, int size);
-extern XtPointer	XdbRealloc(char *f, int l, XtPointer p, int size);
-extern void		XdbFree(char *f, int l, XtPointer p);
-
 #ifdef	XtMalloc
 #undef	XtMalloc
 #endif
@@ -138,6 +130,11 @@ extern void		XdbFree(char *f, int l, XtPointer p);
 
 #endif	/* USE_DMALLOC */
 
+extern XtPointer	XdbMalloc(char *f, int l, int size);
+extern XtPointer	XdbCalloc(char *f, int l, int count, int size);
+extern XtPointer	XdbRealloc(char *f, int l, XtPointer p, int size);
+extern void		XdbFree(char *f, int l, XtPointer p);
+
 #ifdef __cplusplus
 }
 #endif
