@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Mrm.c,v 1.8 1998/06/19 17:05:41 jon Exp $
+ * $Id: Mrm.c,v 1.20 2000/01/26 12:43:37 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -25,9 +25,13 @@
  *                    codesmit@southwind.net
  *
 */
+
+#include <LTconfig.h>
+
 #include <stdarg.h>
 #include <stdlib.h>
 #include <Mrm/MrmPublic.h>
+#include <Xm/XmP.h>
 #include <Xm/Text.h>
 #include <Xm/RowColumn.h>
 #include <Xm/MainW.h>
@@ -37,7 +41,6 @@
 #include <Xm/CascadeB.h>
 #include <Xm/Separator.h>
 #include <Xm/PanedW.h>
-#include <Xm/Text.h>
 #include <Xm/FileSB.h>
 #include <Xm/ToggleB.h>
 #include <Xm/Form.h>
@@ -209,8 +212,7 @@ void PrintWidgetTree(UilModuleType *Module);
 void PrintWidgets(UilModuleType *Module);
 void MakePixmap(Display *d, Pixmap *pixmap, PixmapType * data, Pixel fg, Pixel bg);
 void FreeSymbolTable(SymbolTableType *t);
-void GetCallbacks(int fileno, int wid_id, DummyWidget *DW);
-void GetCallbacks(int fileno, int wid_id, DummyWidget *DW);
+void GetCallbacks(int filenr, int wid_id, DummyWidget *DW);
 void AddGeometryAttributes(GeoType **GeometryList, int module_id, int this_id,
 			   DummyWidget *DW);
 void SetGeometryAttributes(GeoType **GeometryList);
@@ -223,7 +225,7 @@ void PrintSymbolTable(SymbolTableType *SymbolTable);
 SymbolTableType *LookUpAddr(char *temp, int module_id);
 char *CreateFontListFromFontName(char *FontName);
 
-SymbolTableType *ReadAttributes();
+SymbolTableType *ReadAttributes(void);
 void ReadCallbacks(callbackType **last);
 char *ReadType(int type);
 
@@ -236,7 +238,7 @@ void GetAttributes(Display *d, Window w, int module_id,
 		   *FormSymbolTable, int OnGeometry);
 
 void 
-MrmInitialize()
+MrmInitialize(void)
 {
 }
 
@@ -348,10 +350,10 @@ MakePixmap(Display *d, Pixmap *pixmap, PixmapType * data, Pixel fg,
 {
     char *temp, *value;
     char buffer[256];
-    int fileno;
+    int filenr;
     ColorTableHeaderType *ColorTableHeader;
     ColorTableType *ColorTable;
-    XmXpmAttributes attributes;
+    _LtXpmAttributes attributes;
     char Header[256];
     char **andrew = NULL;
     int NumColors;
@@ -366,10 +368,10 @@ MakePixmap(Display *d, Pixmap *pixmap, PixmapType * data, Pixel fg,
     temp = data->ColorTable;
     if (NULL == (symbol = LookupValue(GlobalSymbolTable, temp)))
     {
-	for (fileno = 0; fileno < NumberFiles; fileno++)
+	for (filenr = 0; filenr < NumberFiles; filenr++)
 	{
 	    if (NULL !=
-		(symbol = LookupValue(UilModule[fileno].SymbolTable, temp)))
+		(symbol = LookupValue(UilModule[filenr].SymbolTable, temp)))
 		break;
 	}
 	if (NULL == symbol)
@@ -390,10 +392,10 @@ MakePixmap(Display *d, Pixmap *pixmap, PixmapType * data, Pixel fg,
 	    {
 		if (NULL == (symbol = LookupValue(GlobalSymbolTable, value)))
 		{
-		    for (fileno = 0; fileno < MaxHier; fileno++)
+		    for (filenr = 0; filenr < MaxHier; filenr++)
 		    {
 			if (NULL !=
-			    (symbol = LookupValue(UilModule[fileno].SymbolTable, value)))
+			    (symbol = LookupValue(UilModule[filenr].SymbolTable, value)))
 			    break;
 		    }
 		    if (NULL == symbol)
@@ -468,8 +470,8 @@ MakePixmap(Display *d, Pixmap *pixmap, PixmapType * data, Pixel fg,
 	}
     }
 #if 0
-    if (_XmXpmCreateImageFromData(d, andrew, &im, &imshape, &attributes) ==
-	XmXpmSuccess)
+    if (_LtXmXpmCreateImageFromData(d, andrew, &im, &imshape, &attributes) ==
+	_LtXpmSuccess)
     {
 	GC gc;
 
@@ -488,7 +490,7 @@ MakePixmap(Display *d, Pixmap *pixmap, PixmapType * data, Pixel fg,
 	    XDestroyImage(imshape);
 #endif
 
-	_XmXpmFreeAttributes(&attributes);
+	_LtXmXpmFreeAttributes(&attributes);
     }
 #endif
 
@@ -550,6 +552,32 @@ MrmFetchWidgetOverride(MrmHierarchy hierarchy_id, String index, Widget parent_wi
     return returnValue;
 }
 
+Cardinal
+MrmFetchSetValues(MrmHierarchy hierarchy_id,
+				  Widget w,
+				  ArgList args,
+				  Cardinal num_args)
+{
+	/* Fix Me ! */
+	_XmWarning(NULL, "MrmFetchSetValues() is not yet implemented!");
+	return MrmFAILURE;
+}
+
+Cardinal
+MrmFetchBitmapLiteral(MrmHierarchy hierarchy_id,
+				      String index,
+				      Screen *screen,
+				      Display *display,
+				      Pixmap *pixmap_return,
+				      Dimension *width,
+				      Dimension *height)
+{
+	/* Fix Me ! */
+	_XmWarning(NULL, "MrmFetchBitmapLiteral() is not yet implemented!");
+	return MrmFAILURE;
+}
+	
+
 static int FetchCount = 0;
 SymbolTableType *InstantationSymbolTable = NULL;
 
@@ -733,6 +761,17 @@ MrmFetchWidget(MrmHierarchy hierarchy_id, String index, Widget parent_widget, Wi
     return MrmSUCCESS;
 }
 
+
+Cardinal
+MrmFetchInterfaceModule(MrmHierarchy hierarchy_id ,
+					char *module_name,
+					Widget parent,
+					Widget *w_return)
+{
+	/* Fix Me ! */
+	_XmWarning(NULL, "MrmFetchInterfaceModule() is not yet implemented!");
+	return MrmFAILURE;
+}
 void 
 AddGeometryAttributes(GeoType **GeometryList, int module_id, int this_id,
 		      DummyWidget *DW)
@@ -770,12 +809,12 @@ SetGeometryAttributes(GeoType **GeometryList)
 }
 
 void 
-GetCallbacks(int fileno, int wid_id, DummyWidget *DW)
+GetCallbacks(int filenr, int wid_id, DummyWidget *DW)
 {
     callbackType *Next = DW->callbacks;
     XtPointer ParameterValue;
     SymbolTableType *SymbolTable = NULL;
-    Widget w = UilModule[fileno].WidgetArray[wid_id];
+    Widget w = UilModule[filenr].WidgetArray[wid_id];
 
     while (Next)
     {
@@ -788,7 +827,7 @@ GetCallbacks(int fileno, int wid_id, DummyWidget *DW)
 		temp = Next->Parameters->value;
 		if (1 != LookUpPredefines(temp, (long *)&ParameterValue))
 		    if (NULL ==
-			(SymbolTable = LookupValue(UilModule[fileno].SymbolTable, temp)))
+			(SymbolTable = LookupValue(UilModule[filenr].SymbolTable, temp)))
 		    {
 			if (NULL == (SymbolTable = LookupValue(GlobalSymbolTable, temp)))
 			{
@@ -1249,15 +1288,19 @@ PrintArgs(SymbolTableType *Next)
 	{
 	case MrmRtypeInteger:
 	case MrmRtypeBoolean:
-	    fprintf(stderr, "\t\t%s = %d;\n", Next->Name, (int)Next->value);
+	    fprintf(stderr, "\t\t%s = %ld;\n", Next->Name, (long)Next->value);
 	    break;
 	case MrmRtypeKeysym:
-	case MrmRtypeChar8:
 	case MrmRtypeAddrName:
 	    fprintf(stderr, "\t\t%s = \"%s\";\n", Next->Name, Next->value);
 	    break;
+	case MrmRtypeChar8:
+	    fprintf(stderr, "\t\t%s = \"%s\";\n", Next->Name,
+		((StringType *)Next->value)->theString);
+	    break;
 	case MrmRtypeCString:
-	    fprintf(stderr, "COMPLEX STRING\n");
+	    fprintf(stderr, "\t\t%s = \"%s\";\n", Next->Name,
+		((StringType *)Next->value)->theString);
 	    break;
 	default:
 	    fprintf(stderr, "!  ????  %s \n", Next->Name);
@@ -1298,15 +1341,19 @@ PrintCallbacks(callbackType *Next)
 	    {
 	    case MrmRtypeInteger:
 	    case MrmRtypeBoolean:
-		fprintf(stderr, "%d", (int)NextParam->value);
+		fprintf(stderr, "%ld", (long)NextParam->value);
 		break;
 	    case MrmRtypeKeysym:
 	    case MrmRtypeAddrName:
-	    case MrmRtypeChar8:
 		fprintf(stderr, "%s", NextParam->value);
 		break;
+	    case MrmRtypeChar8:
+		fprintf(stderr, "%s",
+			((StringType *)NextParam->value)->theString);
+		break;
 	    case MrmRtypeCString:
-		fprintf(stderr, "COMPLEX STRING\n");
+		fprintf(stderr, "%s",
+			((StringType *)NextParam->value)->theString);
 		break;
 	    default:
 		break;
@@ -1464,7 +1511,7 @@ ReadCallbacks(callbackType **first)
    //                    | NULL
  */
 SymbolTableType *
-ReadAttributes()
+ReadAttributes(void)
 {
     char c;
     SymbolTableType *Return = NULL, *Next, *Last = NULL;
@@ -1550,6 +1597,7 @@ ReadType(int type)
 	    break;
 	}
     case MrmRtypeKeysym:
+    case MrmRtypeAddrName:
 	temp = &buffer[0];
 	while ((*temp++ = getchar()));
 	ReturnValue = (char *)Store(buffer);
@@ -1659,10 +1707,7 @@ ReadType(int type)
 		fread(&type, sizeof(type), 1, stdin);
 		ColorTable[j].IsAddress = (MrmRtypeAddrName == type);
 		ColorTable[j].StringXPM = NULL;
-		for (i = 0; (i < MAX_BUFFER) && (buffer[i] = getchar()); i++);
-		if (MAX_BUFFER == i)
-		    Exit(LOC, "Static storage area exhausted\n");
-		ColorTable[j].Color = Store(buffer);
+		ColorTable[j].Color=ReadType(type);
 	    }
 	    ReturnValue = (char *)ColorTableHeader;
 	    break;
@@ -1712,12 +1757,6 @@ ReadType(int type)
 	    ReturnValue = (char *)pixmap;
 	    break;
 	}
-    case MrmRtypeAddrName:
-	for (i = 0; (i < MAX_BUFFER) && (buffer[i] = getchar()); i++);
-	if (MAX_BUFFER == i)
-	    Exit(LOC, "Static storage area exhausted\n");
-	ReturnValue = Store(buffer);
-	break;
     default:
 	Exit(LOC, "UNKNOWN ATTRIBUTE: code = %d\n", type);
     }
@@ -2062,5 +2101,6 @@ MrmRegisterNamesInHierarchy(MrmHierarchy hierarchy_id,
 			    MrmCount num_reg)
 {
     /* FIX ME */
+    _XmWarning(NULL, "MrmRegisterNamesInHierarchy() is not yet implemented!");
     return MrmFAILURE;
 }
