@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Sound.c,v 1.2 1998/06/13 02:17:21 rwscott Exp $
+ * $Id: Sound.c,v 1.4 2000/01/25 13:17:18 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,6 +22,7 @@
  *
  **/
 
+#include "../config.h"
 #include <stdio.h>
 #include <signal.h>
 #include <unistd.h>
@@ -39,8 +40,8 @@
 #include "BubbleButton.h"
 
 static void PlaySound(Widget W, XEvent *event, String *SoundFiles, Cardinal *NumSounds);
-static void EnableSound();
-static void DisableSound();
+static void EnableSound(void);
+static void DisableSound(void);
 
 
 typedef struct
@@ -331,7 +332,7 @@ XltSoundEnable(Boolean State)
 /* ***** */
 
 static void
-EnableSound()
+EnableSound(void)
 {
     AppResources.Sounds = True;
 }
@@ -339,7 +340,7 @@ EnableSound()
 /* ***** */
 
 static void
-DisableSound()
+DisableSound(void)
 {
     AppResources.Sounds = False;
 }
