@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.7 1998/09/24 15:37:38 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.12 2000/02/15 23:32:53 danny Exp $
  *
  * Copyright (C) 1995-1998 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PanedW.c,v 1.7 1998/09/24 15:37:38 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.12 2000/02/15 23:32:53 danny Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -251,7 +251,7 @@ static Boolean constraint_set_values(Widget current,
 				     ArgList args,
 				     Cardinal *num_args);
 
-Cardinal panedw_insert_position(Widget w);
+static Cardinal panedw_insert_position(Widget w);
 
 /* helpers */
 static void _XmPanedWPreferredSize(Widget w, Widget child,
@@ -321,7 +321,7 @@ static XtResource resources[] =
     {
 	XmNsashShadowThickness, XmCShadowThickness, XmRHorizontalDimension,
 	sizeof(Dimension), Offset(sash_shadow_thickness),
-	XmRCallProc, (XtPointer)_XmPrimitiveShadowThicknessDefault
+	XmRImmediate, (XtPointer)2
     },
     {
 	XmNinsertPosition, XmCInsertPosition, XmRFunction,
@@ -628,7 +628,7 @@ AllPanesRealized(Widget w)
 
 
 static void
-class_initialize()
+class_initialize(void)
 {
     _XmPanedWCoreClassExtRec.record_type = XmQmotif;
 }
@@ -999,7 +999,7 @@ realize(Widget w,
 /*
  * Insert panes before sashes and separators.
  */
-Cardinal
+static Cardinal
 panedw_insert_position(Widget w)
 {
     XmPanedWindowWidget pw = (XmPanedWindowWidget)XtParent(w);
