@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.22 1998/09/24 15:37:40 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.26 2000/01/26 12:43:59 amai Exp $
  *
  * Copyright (C) 1997-1998 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.22 1998/09/24 15:37:40 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.26 2000/01/26 12:43:59 amai Exp $";
 
 #include <stdio.h>
 #include <LTconfig.h>
@@ -42,7 +42,7 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox
 
 /* Forward Declarations */
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -81,7 +81,7 @@ static void Layout(Widget w, Widget instig);
 /* Trait functions */
 static void _XmSPBTraitChangeMoveCB(Widget, XtCallbackProc, XtPointer, Boolean);
 static void _XmSPBTraitSetValue(Widget, XmNavigatorData, Boolean);
-static void _XmSPBTraitGetValue(Widget, XmNavigatorData);
+static int _XmSPBTraitGetValue(Widget, XmNavigatorData);
 
 static void _XmSPBTraitMoveCB(Widget nav, XtPointer client, XtPointer call);
 
@@ -409,7 +409,7 @@ XmNavigatorTraitRec _XmSPBNavigatorTraitRec = {
 };
 
 static void
-class_initialize()
+class_initialize(void)
 {
     _XmSpinBoxCoreClassExtRec.record_type = XmQmotif;
 
@@ -1281,15 +1281,28 @@ _XmSPBTraitSetValue(Widget nav, XmNavigatorData navigatorData, Boolean notify)
 	if (navigatorData == NULL)
 		return;
 
+	if (navigatorData->valueMask & NavMinimum) {
+		SPBC_Minimum(tw) = navigatorData->minimum->x;
+	}
+
+	if (navigatorData->valueMask & NavMaximum) {
+		SPBC_Maximum(tw) = navigatorData->maximum->x;
+	}
+
+	if (navigatorData->valueMask & NavValue) {
+		/* FIX ME */
+	}
+
 	SPBC_Position(tw) = navigatorData->value->x;
 
 	_XmSpinBoxShowValue(nav, tw);
 }
 
-static void
+static int
 _XmSPBTraitGetValue(Widget nav, XmNavigatorData navigatorData)
 {
 	DEBUGOUT(XdbDebug(__FILE__, nav, "_XmSPBTraitGetValue\n"));
+	return 0;
 }
 
 /*
