@@ -2,7 +2,7 @@
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/TranslNew.c,v 1.4 1998/09/24 15:37:41 pgw Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/TranslNew.c,v 1.6 2000/01/19 20:34:34 amai Exp $
  * This file is part of the GNU LessTif Library.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,7 +21,9 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/TranslNew.c,v 1.4 1998/09/24 15:37:41 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/TranslNew.c,v 1.6 2000/01/19 20:34:34 amai Exp $";
+
+#include <LTconfig.h>
 
 #include <Xm/XmP.h>
 #include <Xm/TransltnsP.h>
@@ -333,3 +335,36 @@ _XmConst char _XmSpinB_defaultAccelerators[] =
     "<Key> osfBeginLine:SpinBFirst()\n"
     "<Key> osfEndLine:SpinBLast()";
 
+char _XmClipWindowTranslationTable[] = "\
+    :c <Key>osfBeginLine:     ActionGrab(SWTopLine)\n\
+    :<Key>osfBeginLine:       ActionGrab(SWBeginLine)\n\
+    :c <Key>osfEndLine:       ActionGrab(SWBottomLine)\n\
+    :<Key>osfEndLine:         ActionGrab(SWEndLine)\n\
+    :<Key>osfPageLeft:        ActionGrab(SWLeftPage)\n\
+    :c <Key>osfPageUp:        ActionGrab(SWLeftPage)\n\
+    :<Key>osfPageUp:          ActionGrab(SWUpPage)\n\
+    :<Key>osfPageRight:       ActionGrab(SWRightPage)\n\
+    :c <Key>osfPageDown:      ActionGrab(SWRightPage)\n\
+    :<Key>osfPageDown:        ActionGrab(SWDownPage)";
+
+char _XmDragC_defaultTranslations[] = "\
+    Button1<Enter>:           DragMotion()\n\
+    Button1<Leave>:           DragMotion()\n\
+    Button1<Motion>:          DragMotion()\n\
+    Button2<Enter>:           DragMotion()\n\
+    Button2<Leave>:           DragMotion()\n\
+    Button2<Motion>:          DragMotion()\n\
+    <Btn2Up>:                 FinishDrag()\n\
+    <Btn1Up>:                 FinishDrag()\n\
+    <BtnDown>:                IgnoreButtons()\n\
+    <BtnUp>:                  IgnoreButtons()\n\
+    <Key>Return:              FinishDrag()\n\
+    :<Key>osfActivate:        FinishDrag()\n\
+    :<Key>osfCancel:          CancelDrag()\n\
+    :<Key>osfHelp:            HelpDrag()\n\
+    :<Key>osfUp:              DragKey(Up)\n\
+    :<Key>osfDown:            DragKey(Down)\n\
+    :<Key>osfLeft:            DragKey(Left)\n\
+    :<Key>osfRight:           DragKey(Right)\n\
+    :<KeyUp>:                 DragKey(Update)\n\
+    :<KeyDown>:               DragKey(Update)";
