@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v 1.11 1999/11/11 13:43:31 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v 1.12 2000/01/16 17:12:28 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v 1.11 1999/11/11 13:43:31 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v 1.12 2000/01/16 17:12:28 amai Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,11 +38,7 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/Xme.c,v
 #include <Xm/VendorSEP.h>
 #include <XmI/MessagesI.h>
 
-#if STDC_HEADERS
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
 /* DragIcon */
 Widget
@@ -92,23 +88,12 @@ XmeGetPixmapData(Screen *screen, Pixmap pixmap,
 
 /* ResConvert */
 void
-#if STDC_HEADERS
 XmeWarning(Widget w, char *fmt,...)
 {
     va_list arg_list;
     char buf[256];
 
     va_start(arg_list, fmt);
-#else
-XmeWarning(Widget w, fmt, va_list)
-     Widget w;
-     char *fmt;
-     va_dcl
-{
-    char buf[256];
-
-    va_start(arg_list);
-#endif
     if (w)
     {
 	sprintf(buf, "\n    Name: %s\n    Class: %s\n    ",
