@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ArrowBG.c,v 1.21 1998/10/10 07:40:44 gritton Exp $
+ * $Id: ArrowBG.c,v 1.28 2000/03/17 17:48:46 danny Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,13 +22,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ArrowBG.c,v 1.21 1998/10/10 07:40:44 gritton Exp $";
+static const char rcsid[] = "$Id: ArrowBG.c,v 1.28 2000/03/17 17:48:46 danny Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
 #include <Xm/ArrowBGP.h>
+#include <Xm/RepType.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -44,16 +45,26 @@ static const char rcsid[] = "$Id: ArrowBG.c,v 1.21 1998/10/10 07:40:44 gritton E
 void _XmArrowBG_TraitAddCallback(Widget, XtCallbackProc, XtPointer, Boolean);
 
 static XmActivatableTraitRec _XmArrowBGTraitRec = {
-        /* version      */      0,
-        /* cb           */      _XmArrowBG_TraitAddCallback
+	/* version      */      0,
+	/* cb           */      _XmArrowBG_TraitAddCallback
 };
 #endif
 
+#if XmVERSION > 1
+#define PUSH_GC(w)			(XtSensitive(w)		\
+					 ? ABG_ArrowGC(w)	\
+					 : ABG_InsensitiveGC(w))
+#define DETAIL_SHADOW_THICKNESS(w)	ABG_DetailShadowThickness(w)
+#else
+#define PUSH_GC(w)			NULL
+#define DETAIL_SHADOW_THICKNESS(w)	Xm3D_ENHANCE_PIXEL
+#endif
+
 /* Forward Declarations */
 
 
 #if XmVERSION > 1
-static void class_initialize();
+static void class_initialize(void);
 #endif
 
 static void class_part_initialize(WidgetClass w_class);
@@ -106,8 +117,26 @@ static XtResource resources[] =
 	XmNdisarmCallback, XmCCallback, XmRCallback,
 	sizeof(XtCallbackList), Offset(disarm_callback),
 	XmRPointer, (XtPointer)NULL
+    },
+#if XmVERSION > 1
+    {
+	XmNdetailShadowThickness, XmCShadowThickness, XmRHorizontalDimension,
+	sizeof(Dimension), Offset(detail_shadow_thickness),
+	XmRImmediate, (XtPointer)2 /* FIX ME */
+    }
+#endif
+};
+
+#if XmVERSION > 1
+static XmSyntheticResource syn_resources[] =
+{
+    {
+	XmNdetailShadowThickness,
+	sizeof(Dimension), Offset(detail_shadow_thickness),
+	_XmFromHorizontalPixels, _XmToHorizontalPixels
     }
 };
+#endif
 
 static void Arm(Widget w, XEvent *event,
 		String *params, Cardinal *num_params);
@@ -135,7 +164,7 @@ static void LeaveWindow(Widget w, XEvent *event,
 #if 0
 static XmBaseClassExtRec _XmArrowBGRectClassExtRec = {
     /* next_extension            */ NULL,
-    /* record_type               */ NULLQUARK,                             
+    /* record_type               */ NULLQUARK,
     /* version                   */ XmBaseClassExtVersion,
     /* size                      */ sizeof(XmBaseClassExtRec),
     /* initialize_prehook        */ NULL,
@@ -202,7 +231,7 @@ XmArrowButtonGadgetClassRec xmArrowButtonGadgetClassRec = {
 	/* set_values_almost     */ XtInheritSetValuesAlmost,
 	/* get_values_hook       */ NULL,
 	/* accept_focus          */ NULL,
-	/* version               */ XtVersion,
+	/* version               */ XtVersionDontCheck,
 	/* callback offsets      */ NULL,
 	/* tm_table              */ NULL,
 	/* query_geometry        */ NULL /* query_geometry */,
@@ -211,12 +240,16 @@ XmArrowButtonGadgetClassRec xmArrowButtonGadgetClassRec = {
     },
     /* XmGadget part */
     {
-	/* border_highlight   */ XmInheritBorderHighlight, 
+	/* border_highlight   */ XmInheritBorderHighlight,
 	/* border_unhighlight */ XmInheritBorderUnhighlight,
 	/* arm_and_activate   */ ArmAndActivate,
 	/* input_dispatch     */ input_dispatch,
 	/* visual_change      */ NULL, /* FIX ME */
+#if XmVERSION > 1
+	/* syn_resources      */ syn_resources,
+#else
 	/* syn_resources      */ NULL,
+#endif
 	/* num_syn_resources  */ 0,
 	/* cache_part         */ NULL,
 	/* extension          */ (XtPointer)NULL /*&_XmArrowBGadgetClassExtRec*/
@@ -234,7 +267,7 @@ WidgetClass xmArrowButtonGadgetClass =
 
 #if XmVERSION > 1
 static void
-class_initialize()
+class_initialize(void)
 {
 #if 0
     _XmArrowBGRectClassExtRec.record_type = XmQmotif;
@@ -293,11 +326,8 @@ CreateInsensitiveGC(Widget w)
     values.fill_style = FillStippled;
     values.ts_x_origin = values.ts_y_origin = 0;
 
-    values.stipple = XmGetPixmapByDepth(XtScreen(w),
-					XmEVEN_STIPPLE_IMAGE,
-					_XmWhitePixelOfObject(w),
-					_XmBlackPixelOfObject(w),
-					1);
+    values.stipple =
+	XmGetPixmapByDepth(XtScreen(w), "50_foreground", 1, 0, 1);
 
     ABG_InsensitiveGC(w) = XtGetGC(w, mask, &values);
 }
@@ -310,6 +340,14 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 	_XmError(new_w, "parent should be manager.");
     }
 
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRArrowDirection),
+			     ABG_Direction(new_w), new_w))
+	ABG_Direction(new_w) = XmARROW_UP;
+
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRMultiClick),
+			     ABG_MultiClick(new_w), new_w))
+	ABG_MultiClick(new_w) = XmMULTICLICK_KEEP;
+
     if (XtWidth(request) == 0)
     {
 	XtWidth(new_w) += 15;
@@ -342,6 +380,7 @@ destroy(Widget w)
     if (ABG_Timer(w) != 0)
     {
 	XtRemoveTimeOut(ABG_Timer(w));
+	ABG_Timer(w) = 0;
     }
 
     XtReleaseGC(w, ABG_ArrowGC(w));
@@ -352,12 +391,17 @@ static Boolean
 set_values(Widget old, Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-
-    XmArrowButtonGadget abg = (XmArrowButtonGadget)new_w;
-    XmArrowButtonGadget og = (XmArrowButtonGadget)old;
     Boolean refresh_needed = False;
 
-    if (XmParentBackground(new_w) != XmParentBackground(old))
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRArrowDirection),
+			     ABG_Direction(new_w), new_w))
+	ABG_Direction(new_w) = ABG_Direction(old);
+
+    if (!XmRepTypeValidValue(XmRepTypeGetId(XmRMultiClick),
+			     ABG_MultiClick(new_w), new_w))
+	ABG_MultiClick(new_w) = ABG_MultiClick(old);
+
+    if (XmParentForeground(new_w) != XmParentForeground(old))
     {
 	XtReleaseGC(new_w, ABG_ArrowGC(new_w));
 	XtReleaseGC(new_w, ABG_InsensitiveGC(new_w));
@@ -368,7 +412,11 @@ set_values(Widget old, Widget request, Widget new_w,
 	refresh_needed = True;
     }
 
-    if (ABG_Direction(abg) != ABG_Direction(og))
+    if (ABG_Direction(new_w) != ABG_Direction(old) ||
+#if XmVERSION > 1
+	ABG_DetailShadowThickness(old) != ABG_DetailShadowThickness(new_w) ||
+#endif
+	XtSensitive(new_w) != XtSensitive(old))
     {
 	refresh_needed = True;
     }
@@ -379,17 +427,9 @@ set_values(Widget old, Widget request, Widget new_w,
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-    Dimension x, y, xsize, ysize;
-    Dimension margin = G_ShadowThickness(w) + G_HighlightThickness(w) + 1;
-    Boolean State;
+    Dimension margin = G_ShadowThickness(w) + G_HighlightThickness(w);
     GC myGC;
 
-    x = XtX(w) + margin;
-    y = XtY(w) + margin;
-
-    xsize = XtWidth(w) - margin * 2;
-    ysize = XtHeight(w) - margin * 2;
-
     /* use the right GC */
     if (XtIsSensitive(w))
     {
@@ -402,7 +442,7 @@ expose(Widget w, XEvent *event, Region region)
 
     if (G_Highlighted(w))
     {
-	    (*GC_BorderHighlight(XtClass(w))) (w);
+	(*GC_BorderHighlight(XtClass(w))) (w);
     }
     else
     {
@@ -421,38 +461,28 @@ expose(Widget w, XEvent *event, Region region)
 		   G_ShadowThickness(w),
 		   XmSHADOW_OUT);
 
-    State = ABG_Armed(w);
-
-    if (State)
+    if (ABG_Armed(w))
     {
 	_XmDrawArrow(XtDisplayOfObject(w),
 		     XtWindowOfObject(w),
-		     XmParentTopShadowGC(w),
 		     XmParentBottomShadowGC(w),
+		     XmParentTopShadowGC(w),
 		     myGC,
-		     x, y,
-		     xsize, ysize,
-	/* Per tegla@katalin.csoma.elte.hu, M*tif ignores
-	   this
-	   G_ShadowThickness(w),
-	 */
-		     2,
+		     XtX(w) + margin, XtY(w) + margin,
+		     XtWidth(w) - (margin << 1), XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
 		     ABG_Direction(w));
     }
     else
     {
 	_XmDrawArrow(XtDisplayOfObject(w),
 		     XtWindowOfObject(w),
-		     XmParentBottomShadowGC(w),
 		     XmParentTopShadowGC(w),
+		     XmParentBottomShadowGC(w),
 		     myGC,
-		     x, y,
-		     xsize, ysize,
-	/* Per tegla@katalin.csoma.elte.hu, M*tif ignores
-	   this
-	   G_ShadowThickness(w),
-	 */
-		     2,
+		     XtX(w) + margin, XtY(w) + margin,
+		     XtWidth(w) - (margin << 1), XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
 		     ABG_Direction(w));
     }
 }
@@ -489,14 +519,27 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 static void
 Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
+    int margin;
     XmArrowButtonCallbackStruct cbs;
-    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     if (!ABG_Armed(w))
     {
 	ABG_Armed(w) = True;
 
-	(*exp)(w, NULL, (Region)NULL);
+	if (XtIsRealized(w))
+	{
+	    margin = G_ShadowThickness(w) + G_HighlightThickness(w);
+	    _XmDrawArrow(XtDisplayOfObject(w),
+			 XtWindowOfObject(w),
+			 XmParentBottomShadowGC(w),
+			 XmParentTopShadowGC(w),
+			 PUSH_GC(w),
+			 XtX(w) + margin, XtY(w) + margin,
+			 XtWidth(w) - (margin << 1),
+			 XtHeight(w) - (margin << 1),
+			 DETAIL_SHADOW_THICKNESS(w),
+			 ABG_Direction(w));
+	}
 
 	if (ABG_ArmCallback(w))
 	{
@@ -516,8 +559,23 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 Disarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
+    int margin;
     XmArrowButtonCallbackStruct cbs;
-    XtExposeProc exp = XtClass(w)->core_class.expose;
+
+    if (XtIsRealized(w))
+    {
+	margin = G_ShadowThickness(w) + G_HighlightThickness(w);
+	_XmDrawArrow(XtDisplayOfObject(w),
+		     XtWindowOfObject(w),
+		     XmParentTopShadowGC(w),
+		     XmParentBottomShadowGC(w),
+		     PUSH_GC(w),
+		     XtX(w) + margin, XtY(w) + margin,
+		     XtWidth(w) - (margin << 1),
+		     XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
+		     ABG_Direction(w));
+    }
 
     if (ABG_DisarmCallback(w))
     {
@@ -532,7 +590,6 @@ Disarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 			   (XtPointer)&cbs);
     }
     ABG_Armed(w) = False;
-    (*exp)(w, NULL, (Region)NULL);
 }
 
 static void
@@ -541,6 +598,7 @@ Activate(Widget w,
 	 String *params,
 	 Cardinal *num_params)
 {
+    int margin;
     XButtonEvent *ev = (XButtonEvent *)event;
     XmArrowButtonCallbackStruct cbs;
 
@@ -553,8 +611,21 @@ Activate(Widget w,
 
 	if (ABG_ActivateCallback(w))
 	{
-
-	    XFlush(XtDisplay(w));
+	    if (XtIsRealized(w))
+	    {
+		margin = G_ShadowThickness(w) + G_HighlightThickness(w);
+		_XmDrawArrow(XtDisplayOfObject(w),
+			     XtWindowOfObject(w),
+			     XmParentBottomShadowGC(w),
+			     XmParentTopShadowGC(w),
+			     PUSH_GC(w),
+			     XtX(w) + margin, XtY(w) + margin,
+			     XtWidth(w) - (margin << 1),
+			     XtHeight(w) - (margin << 1),
+			     DETAIL_SHADOW_THICKNESS(w),
+			     ABG_Direction(w));
+		XFlush(XtDisplay(w));
+	    }
 
 	    XtCallCallbackList(w,
 			       ABG_ActivateCallback(w),
@@ -568,16 +639,42 @@ Activate(Widget w,
 static void
 ArmTimeout(XtPointer data, XtIntervalId *id)
 {
+    int margin;
     Widget w = (Widget)data;
-    XtExposeProc exp = XtClass(w)->core_class.expose;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
- 
+
     ABG_Timer(w) = 0;
 
     if (XtIsRealized(w))
     {
-	(*exp) (w, NULL, NULL);
+	margin = G_ShadowThickness(w) + G_HighlightThickness(w);
+	if (ABG_Armed(w))
+	{
+	    _XmDrawArrow(XtDisplayOfObject(w),
+			 XtWindowOfObject(w),
+			 XmParentBottomShadowGC(w),
+			 XmParentTopShadowGC(w),
+			 PUSH_GC(w),
+			 XtX(w) + margin, XtY(w) + margin,
+			 XtWidth(w) - (margin << 1),
+			 XtHeight(w) - (margin << 1),
+			 DETAIL_SHADOW_THICKNESS(w),
+			 ABG_Direction(w));
+	}
+	else
+	{
+	    _XmDrawArrow(XtDisplayOfObject(w),
+			 XtWindowOfObject(w),
+			 XmParentTopShadowGC(w),
+			 XmParentBottomShadowGC(w),
+			 PUSH_GC(w),
+			 XtX(w) + margin, XtY(w) + margin,
+			 XtWidth(w) - (margin << 1),
+			 XtHeight(w) - (margin << 1),
+			 DETAIL_SHADOW_THICKNESS(w),
+			 ABG_Direction(w));
+	}
 	XFlush(XtDisplayOfObject(w));
     }
 }
@@ -586,7 +683,7 @@ static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmArrowButtonCallbackStruct cbs;
- 
+
     DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
 
     /* Arm, Activate, and Disarm now, but draw the disarmed state later */
@@ -617,6 +714,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     if (ABG_Timer(w) != 0)
     {
 	XtRemoveTimeOut(ABG_Timer(w));
+	ABG_Timer(w) = 0;
     }
 
     ABG_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
@@ -626,24 +724,18 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    int margin = G_ShadowThickness(w) + G_HighlightThickness(w) + 1;
-    int xsize = XtWidth(w) - margin * 2;
-    int ysize = XtHeight(w) - margin * 2;
+    int margin = G_ShadowThickness(w) + G_HighlightThickness(w);
 
     if (ABG_Armed(w))
     {
 	_XmDrawArrow(XtDisplayOfObject(w),
 		     XtWindowOfObject(w),
-		     XmParentTopShadowGC(w),
 		     XmParentBottomShadowGC(w),
-		     ABG_ArrowGC(w),
+		     XmParentTopShadowGC(w),
+		     PUSH_GC(w),
 		     XtX(w) + margin, XtY(w) + margin,
-		     xsize, ysize,
-	/* Per tegla@katalin.csoma.elte.hu, M*tif ignores
-	   this
-	   Prim_ShadowThickness(w),
-	 */
-		     2,
+		     XtWidth(w) - (margin << 1), XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
 		     ABG_Direction(w));
     }
 }
@@ -651,24 +743,18 @@ EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    int margin = G_ShadowThickness(w) + G_HighlightThickness(w) + 1;
-    int xsize = XtWidth(w) - margin * 2;
-    int ysize = XtHeight(w) - margin * 2;
+    int margin = G_ShadowThickness(w) + G_HighlightThickness(w);
 
     if (ABG_Armed(w))
     {
 	_XmDrawArrow(XtDisplayOfObject(w),
 		     XtWindowOfObject(w),
-		     XmParentBottomShadowGC(w),
 		     XmParentTopShadowGC(w),
-		     ABG_ArrowGC(w),
+		     XmParentBottomShadowGC(w),
+		     PUSH_GC(w),
 		     XtX(w) + margin, XtY(w) + margin,
-		     xsize, ysize,
-	/* Per tegla@katalin.csoma.elte.hu, M*tif ignores
-	   this
-	   Prim_ShadowThickness(w),
-	 */
-		     2,
+		     XtWidth(w) - (margin << 1), XtHeight(w) - (margin << 1),
+		     DETAIL_SHADOW_THICKNESS(w),
 		     ABG_Direction(w));
     }
 }
@@ -773,13 +859,13 @@ XmCreateArrowButtonGadget(Widget parent, char *name,
 
 #if XmVERSION > 1
 void _XmArrowBG_TraitAddCallback(Widget w,
-                        XtCallbackProc cb,
-                        XtPointer cbp,
-                        Boolean set)
-{   
-        if (set)
-                XtAddCallback(w, XmNactivateCallback, cb, cbp);
-        else
-                XtRemoveCallback(w, XmNactivateCallback, cb, cbp);
+				 XtCallbackProc cb,
+				 XtPointer cbp,
+				 Boolean set)
+{
+	if (set)
+	    XtAddCallback(w, XmNactivateCallback, cb, cbp);
+	else
+	    XtRemoveCallback(w, XmNactivateCallback, cb, cbp);
 }
 #endif
