@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Command.c,v 1.14 1998/10/23 01:09:49 rwscott Exp $
+ * $Id: Command.c,v 1.19 2000/03/05 13:12:24 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Command.c,v 1.14 1998/10/23 01:09:49 rwscott Exp $";
+static const char rcsid[] = "$Id: Command.c,v 1.19 2000/03/05 13:12:24 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -44,7 +44,9 @@ static void class_part_initialize(WidgetClass w_class);
 static void initialize(Widget request, Widget new_w,
 		       ArgList args, Cardinal *num_args);
 
+/*
 static void destroy(Widget w);
+*/
 
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
@@ -138,7 +140,7 @@ static XmSyntheticResource syn_resources[] =
     {
 	XmNcommand,
 	sizeof(XmString), SBOffset(text_string),
-	_XmExportXmString, NULL
+	_XmSelectionBoxGetTextString, NULL
     },
     {
 	XmNhistoryItems,
@@ -215,7 +217,7 @@ XmCommandClassRec xmCommandClassRec = {
 	/* compress_exposure     */ XtExposeCompressSeries /*XtExposeCompressMultiple*/,
 	/* compress_enterleave   */ True,
 	/* visible_interest      */ False,
-	/* destroy               */ destroy,
+	/* destroy               */ NULL /*destroy*/,
 	/* resize                */ XtInheritResize,
 	/* expose                */ XtInheritExpose,
 	/* set_values            */ set_values,
@@ -268,11 +270,12 @@ XmCommandClassRec xmCommandClassRec = {
     },
     /* XmSelectionBox part */
     {
-	/* extension */ NULL,
+	/* list_callback                */ NULL,
+	/* extension                    */ NULL,
     },
     /* XmCommand Class Part */
     {
-	/* extension */	NULL,
+	/* extension                    */ NULL,
     }
 };
 /* *INDENT-ON* */
@@ -294,25 +297,24 @@ class_part_initialize(WidgetClass widget_class)
     _XmFastSubclassInit(widget_class, XmCOMMAND_BOX_BIT);
 }
 
-#define	C_ACT		1	/* ActivateCallback on textfield */
-#define	C_LIST_SELECT	2	/* Selection on list */
-#define	C_LIST_DOUBLE	3	/* DoubleClick on list */
+#define	C_ACT		((XtPointer)1)	/* ActivateCallback on textfield */
+#define	C_LIST_SELECT	((XtPointer)2)	/* Selection on list */
+#define	C_LIST_DOUBLE	((XtPointer)3)	/* DoubleClick on list */
 
-void
+static void
 _XmCommandCallback(Widget w, XtPointer client, XtPointer call)
 {
-    int x = (int)client;
     XmCommandCallbackStruct cbs;
     XmAnyCallbackStruct *p = (XmAnyCallbackStruct *)call;
     XmListCallbackStruct *l = (XmListCallbackStruct *) call;
     XmCommandWidget cw = (XmCommandWidget)XtParent(w);
     char *t;
 
-    if (x != C_ACT)
+    if (client != C_ACT)
 	cw = (XmCommandWidget)XtParent(XtParent(w));
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)cw, "_XmCommandCallback [%s]\n",
-		      (x == C_ACT) ? "TextField Activate" : "List Selection"));
+	(client == C_ACT) ? "TextField Activate" : "List Selection"));
 
     /*
      * In the cases of TextField Activate and List DefaultAction, we need to
@@ -320,7 +322,7 @@ _XmCommandCallback(Widget w, XtPointer client, XtPointer call)
      * In the List SingleSelect case, just make the list item show up in
      * TextField.
      */
-    if (x == C_ACT)
+    if (client == C_ACT)
     {
 	cbs.reason = XmCR_COMMAND_ENTERED;
 	cbs.event = p->event;
@@ -335,7 +337,7 @@ _XmCommandCallback(Widget w, XtPointer client, XtPointer call)
 	XtCallCallbackList((Widget)cw, cw->command.callback, &cbs);
     }
 
-    if (x == C_LIST_SELECT)
+    if (client == C_LIST_SELECT)
     {
 	if (!XmStringGetLtoR(l->item, XmFONTLIST_DEFAULT_TAG, &t))
 	{
@@ -351,7 +353,7 @@ _XmCommandCallback(Widget w, XtPointer client, XtPointer call)
 	return;
     }
 
-    if (x == C_LIST_DOUBLE)
+    if (client == C_LIST_DOUBLE)
     {
 	cbs.reason = XmCR_COMMAND_ENTERED;
 	cbs.event = l->event;
@@ -389,7 +391,7 @@ _XmCommandCallback(Widget w, XtPointer client, XtPointer call)
 			     List_ItemCount(SB_List(cw)) -
 				Com_HistoryMaxItems(cw), 1);
     }
-    if (x != C_LIST_DOUBLE)	/* FIX ME - I think this is right ! */
+    if (client != C_LIST_DOUBLE)	/* FIX ME - I think this is right ! */
     {
 	XmStringFree(cbs.value);
     }
@@ -404,6 +406,15 @@ initialize(Widget request, Widget new_w,
 {
     DEBUGOUT(XdbDebug(__FILE__, new_w, "Initialize\n"));
 
+    if (Com_PromptString(new_w) == (XmString)XmUNSPECIFIED)
+    {
+	XmString xms;
+
+	xms = XmStringCreateLocalized(">");
+	XtVaSetValues( SB_SelectionLabel(new_w), XmNlabelString, xms, NULL);
+	XmStringFree(xms);
+	Com_PromptString(new_w) = NULL;
+    }
 #if 0
     /* rws 2 Oct 1998
        These are just re-named SelectionBox resources so SelectionBox
@@ -429,11 +440,11 @@ initialize(Widget request, Widget new_w,
 #endif
 
     XtAddCallback(SB_Text(new_w), XmNactivateCallback,
-		  _XmCommandCallback, (XtPointer)C_ACT);
+		  _XmCommandCallback, C_ACT);
     XtAddCallback(SB_List(new_w), XmNsingleSelectionCallback,
-		  _XmCommandCallback, (XtPointer)C_LIST_SELECT);
+		  _XmCommandCallback, C_LIST_SELECT);
     XtAddCallback(SB_List(new_w), XmNdefaultActionCallback,
-		  _XmCommandCallback, (XtPointer)C_LIST_DOUBLE);
+		  _XmCommandCallback, C_LIST_DOUBLE);
 
     Com_Error(new_w) = False;
 }
@@ -625,10 +636,10 @@ int count = 0;
     *value = (XtArgVal)count;
 }
 
+#if 0
 static void 
 destroy(Widget w)
 {
-#if 0
   XmCommandWidget cw = (XmCommandWidget)w;
     /* rws 2 Oct 1998
        These are just re-named SelectionBox resources so SelectionBox
@@ -640,5 +651,5 @@ destroy(Widget w)
   if (Com_Command(cw)) {
     XmStringFree(Com_Command(cw));
   }
-#endif
 }
+#endif
