@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DebugUtil.c,v 1.39 1998/10/16 23:41:37 rwscott Exp $
+ * $Id: DebugUtil.c,v 1.45 2000/02/13 12:43:24 rwscott Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DebugUtil.c,v 1.39 1998/10/16 23:41:37 rwscott Exp $";
+static const char rcsid[] = "$Id: DebugUtil.c,v 1.45 2000/02/13 12:43:24 rwscott Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,6 +38,7 @@ static const char rcsid[] = "$Id: DebugUtil.c,v 1.39 1998/10/16 23:41:37 rwscott
 #include <X11/Xresource.h>
 #include <Xm/VendorSEP.h>
 #include <Xm/DrawingAP.h>
+#include <Xm/Display.h>
 
 /*
  * Functionality in this file is influenced at run time by these
@@ -62,7 +63,7 @@ static const char rcsid[] = "$Id: DebugUtil.c,v 1.39 1998/10/16 23:41:37 rwscott
 
 static FILE *XdbFile = NULL;
 
-void 
+static void 
 XdbOpenFile(void)
 {
     char *s, *p, *fn;
@@ -264,6 +265,74 @@ XdbGeometryResult2String(XtGeometryResult r)
 }
 
 char *
+XdbDragAndDropMessageType2String(unsigned char r)
+{
+    switch (r)
+    {
+    case XmTOP_LEVEL_ENTER:
+	return "TOP_LEVEL_ENTER";
+
+    case XmTOP_LEVEL_LEAVE:
+	return "TOP_LEVEL_LEAVE";
+
+    case XmDRAG_MOTION:
+	return "DRAG_MOTION";
+
+    case XmDROP_SITE_ENTER:
+	return "DROP_SITE_ENTER";
+
+    case XmDROP_SITE_LEAVE:
+	return "DROP_SITE_LEAVE";
+
+    case XmDROP_START:
+	return "DROP_START";
+
+    case XmDROP_FINISH:
+	return "DROP_FINISH";
+
+    case XmDRAG_DROP_FINISH:
+	return "DRAG_DROP_FINISH";
+
+    case XmOPERATION_CHANGED:
+	return "OPERATION_CHANGED";
+
+    default:
+	return "UNKNOWN";
+    }
+}
+
+char *
+XdbDragType2String(unsigned char r)
+{
+    switch (r)
+    {
+    case XmDRAG_NONE:
+	return "XmDRAG_NONE";
+
+    case XmDRAG_DROP_ONLY:
+	return "XmDRAG_DROP_ONLY";
+
+    case XmDRAG_PREFER_PREREGISTER:
+	return "XmDRAG_PREFER_PREREGISTER";
+
+    case XmDRAG_PREREGISTER:
+	return "XmDRAG_PREREGISTER";
+
+    case XmDRAG_PREFER_DYNAMIC:
+	return "XmDRAG_PREFER_DYNAMIC";
+
+    case XmDRAG_PREFER_RECEIVER:
+	return "XmDRAG_PREFER_RECEIVER";
+
+    case XmDRAG_DYNAMIC:
+	return "XmDRAG_DYNAMIC";
+
+    default:
+	return "UNKNOWN";
+    }
+}
+
+static char *
 XdbScrollingPolicy2String(unsigned char r)
 {
     switch (r)
@@ -279,7 +348,7 @@ XdbScrollingPolicy2String(unsigned char r)
     }
 }
 
-char *
+static char *
 XdbDialogStyle2String(int a)
 {
     switch (a)
@@ -399,7 +468,7 @@ ValidateSource(char *fn, char *l)
  * It allows us to choose at run time which sources should spit out
  * their guts.
  */
-char *
+static char *
 _XdbFindDebugSources(Widget w)
 {
     static char *env = NULL;
@@ -416,7 +485,7 @@ _XdbFindDebugSources(Widget w)
 }
 
 static char *
-_XdbPrintWidgetID()
+_XdbPrintWidgetID(void)
 {
     static char *env = NULL;
     static int checked=False;
@@ -656,15 +725,17 @@ XdbBoolean2String(int b)
 char *
 XdbXmString2String(XmString xms)
 {
-    char *s = NULL;
+  static char *s = NULL;
 
+    if (s) {
+      free(s);
+      s=NULL;
+    }
     if (xms == (XmString)XmUNSPECIFIED)
     {
 	return "XmUNSPECIFIED";
     }
-    XmStringGetLtoR(xms, XmFONTLIST_DEFAULT_TAG, &s);
-
-    if (s == NULL)
+    if ( ! XmStringGetLtoR(xms, XmFONTLIST_DEFAULT_TAG, &s) || s == NULL )
     {
 	return "(null)";
     }
@@ -807,6 +878,7 @@ XdbWidgetGeometry2String(XtWidgetGeometry *g)
 #define XdbRESIZE_POLICY	17
 #define XdbCOMBOBOX_TYPE	18
 #define XdbSCROLLING_POLICY	19
+#define XdbDRAG_TYPE		20
 
 static struct
 {
@@ -816,6 +888,14 @@ static struct
 }
 XdbTypes[] =
 {
+    {
+	XmNdragInitiatorProtocolStyle, XdbDRAG_TYPE, NULL
+    }
+    ,
+    {
+	XmNdragReceiverProtocolStyle, XdbDRAG_TYPE, NULL
+    }
+    ,
     {
 	XmNscrollingPolicy, XdbSCROLLING_POLICY, NULL
     }
@@ -1004,6 +1084,10 @@ XdbTypes[] =
 	XmNrightWidget, XdbWIDGET, NULL
     }
     ,
+    {
+	XmNsensitive, XdbBOOLEAN, NULL
+    }
+    ,
     {
 	XmNresizable, XdbBOOLEAN, NULL
     }
@@ -1277,6 +1361,12 @@ XdbPrintArgList(char *fn, Widget w, ArgList al, int n, Boolean Get)
 			: XdbSBDisplayPolicy2String((unsigned char)al[i].value));
 		break;
 
+	    case XdbDRAG_TYPE:
+		fprintf(XdbFile, "Arg[%d] : %s %s\n", i, al[i].name,
+		    (Get) ? XdbDragType2String(*(unsigned char *)al[i].value)
+			: XdbDragType2String((unsigned char)al[i].value));
+		break;
+
 	    case XdbSCROLLING_POLICY:
 		fprintf(XdbFile, "Arg[%d] : %s %s\n", i, al[i].name,
 		    (Get) ? XdbScrollingPolicy2String(*(unsigned char *)al[i].value)
