@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DragBS.c,v 1.18 2000/01/05 23:44:44 rwscott Exp $
+ * $Id: DragBS.c,v 1.21 2000/02/13 12:43:25 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DragBS.c,v 1.18 2000/01/05 23:44:44 rwscott Exp $";
+static const char rcsid[] = "$Id: DragBS.c,v 1.21 2000/02/13 12:43:25 rwscott Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -372,7 +372,7 @@ _XmReadDragBuffer(XmDndBufMgr bmgr, Boolean read_string,
 
 	if (rsize <= eaten)
 	{
-	    size = rsize - eaten;
+	    size = eaten - rsize;
 	}
 
 	bcopy(bmgr->name_start, retbuf, size);
@@ -389,7 +389,7 @@ _XmReadDragBuffer(XmDndBufMgr bmgr, Boolean read_string,
 
 	if (rsize <= eaten)
 	{
-	    size = rsize - eaten;
+	    size = eaten - rsize;
 	}
 
 	bcopy(bmgr->atom_start, retbuf, size);
@@ -407,6 +407,8 @@ short
 _XmWriteDragBuffer(XmDndBufMgr bmgr, Boolean write_string,
 		   char *data, short size)
 {
+    int increase_needed;
+
     DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmWriteDragBuffer(%d) write_string %s size %i\n",
 		      __FILE__, __LINE__,
 		      write_string ? "True" : "False",
@@ -414,9 +416,13 @@ _XmWriteDragBuffer(XmDndBufMgr bmgr, Boolean write_string,
 
     if (write_string)
     {
-	if ((bmgr->name_size + size) > bmgr->name_avail)
+	increase_needed = (bmgr->name_size + size) > bmgr->name_avail;
+	if (increase_needed > 0)
 	{
-	    bmgr->name_avail += 1000;
+	    increase_needed /= 1000;
+            increase_needed++;
+            increase_needed *= 1000;
+	    bmgr->name_avail += increase_needed;
 
 	    if (bmgr->names == bmgr->name_ptr)
 	    {
@@ -438,10 +444,13 @@ _XmWriteDragBuffer(XmDndBufMgr bmgr, Boolean write_string,
     }
     else
     {
-	if ((bmgr->atom_size + size) > bmgr->atom_avail)
+	increase_needed = (bmgr->atom_size + size) > bmgr->atom_avail;
+	if (increase_needed)
 	{
-	    bmgr->atom_avail += 1000;
-
+	    increase_needed /= 1000;
+            increase_needed++;
+            increase_needed *= 1000;		
+	    bmgr->atom_avail += increase_needed;
 	    if (bmgr->atoms == bmgr->atom_ptr)
 	    {
 		bmgr->atoms = XtMalloc(bmgr->atom_avail);
@@ -1538,7 +1547,9 @@ _XmAllocMotifAtom(Widget shell, Time time)
     int i;
     char buf[32];
 
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmAllocMotifAtom(%d)\n",
+    DEBUGOUT(XdbDebug(__FILE__, shell, "%s:_XmAllocMotifAtom(%d)\n",
+		      __FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSOURCE", shell, "%s:_XmAllocMotifAtom(%d)\n",
 		      __FILE__, __LINE__));
 
     if ((atoms = get_atoms_table(dpy)) == NULL)
@@ -1585,7 +1596,7 @@ _XmAllocMotifAtom(Widget shell, Time time)
 	    (XmDndAtomsTableEntry) XtRealloc((char *)atoms->entries,
 			 atoms->num_entries * sizeof(XmDndAtomsTableEntryRec));
 
-	sprintf(buf, _XA_MOTIF_ATOM_FORMAT, atoms->num_entries);
+	sprintf(buf, _XA_MOTIF_ATOM_FORMAT, i);
 	alloc = XmInternAtom(dpy, buf, False);
 
 	atoms->entries[i].atom = alloc;
@@ -1598,6 +1609,14 @@ _XmAllocMotifAtom(Widget shell, Time time)
 
     XFlush(dpy);
 
+    if (XdbInDebug("DRAGSOURCE", shell))
+    {
+    char *AtomName = XGetAtomName(dpy, alloc);
+
+	DEBUGOUT(XdbDebug0("DRAGSOURCE", shell, "\t%s - 0x%x\n", AtomName, time));
+    	XFree(AtomName);
+    }
+
     return alloc;
 }
 
@@ -1746,6 +1765,8 @@ _XmGetDragReceiverInfo(Display *display, Window win, XmDragReceiverInfo ri)
 
     DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmGetDragReceiverInfo(%d)\n",
 		      __FILE__, __LINE__));
+    DEBUGOUT(XdbDebug0("DRAGSOURCE", NULL, "%s:_XmGetDragReceiverInfo(%d) - %p\n",
+		      __FILE__, __LINE__, win));
 
     dri = XmInternAtom(display, _XA_MOTIF_DRAG_RECEIVER_INFO, False);
 
@@ -1753,6 +1774,8 @@ _XmGetDragReceiverInfo(Display *display, Window win, XmDragReceiverInfo ri)
 			   dri, &type, &format, &length, &bafter,
 			   (unsigned char **)&receiver) != Success)
     {
+	DEBUGOUT(XdbDebug0("DRAGSOURCE", NULL, "%s:_XmGetDragReceiverInfo(%d) - getting prop failed\n",
+		      __FILE__, __LINE__));
 	return False;
     }
 
@@ -1760,6 +1783,8 @@ _XmGetDragReceiverInfo(Display *display, Window win, XmDragReceiverInfo ri)
     {
 	ri->dragProtocolStyle = XmDRAG_NONE;
 	XFree((char *)receiver);
+	DEBUGOUT(XdbDebug0("DRAGSOURCE", NULL, "%s:_XmGetDragReceiverInfo(%d) - None available\n",
+		      __FILE__, __LINE__));
 	return False;
     }
 
