@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DragC.c,v 1.25 2000/01/05 23:44:44 rwscott Exp $
+ * $Id: DragC.c,v 1.26 2000/02/13 12:43:25 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -40,7 +40,7 @@
  * provided "as is" without express or implied warranty.
  */
 
-static const char rcsid[] = "$Id: DragC.c,v 1.25 2000/01/05 23:44:44 rwscott Exp $";
+static const char rcsid[] = "$Id: DragC.c,v 1.26 2000/02/13 12:43:25 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -508,7 +508,6 @@ set_values(Widget old, Widget request, Widget new_w,
 
     if (DC_ExportTargets(old) != DC_ExportTargets(new_w))
     {
-
 	if (DC_ExportTargets(old))
 	{
 	    XtFree((char *)DC_ExportTargets(old));
@@ -535,6 +534,9 @@ send_drag_message(Widget dc, Window win, XmDragDropCallbackStruct * cbs,
 
     DEBUGOUT(XdbDebug(__FILE__, dc, "%s:send_drag_message(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSOURCE", dc, "%s:send_drag_message(%d) - %s to %p\n",
+    	__FILE__, __LINE__,
+    	XdbDragAndDropMessageType2String(cbs->any.reason), win));
 
     msg = XmInternAtom(XtDisplay(dc), _XA_MOTIF_DRAG_AND_DROP_MESSAGE, False);
 
@@ -605,6 +607,11 @@ receive_drag_message(Widget dc, XEvent *event, XmDragDropCallbackStruct * cbs)
 
     msg = XmInternAtom(XtDisplay(dc), _XA_MOTIF_DRAG_AND_DROP_MESSAGE, False);
 
+    DEBUGOUT(XdbDebug("DRAGSOURCE", dc, "%s:receive_drag_message(%d) - %s %i %s\n",
+    	__FILE__, __LINE__,
+    	XdbDragAndDropMessageType2String(dnd_message->reason & DND_CLEAR_EVENT_TYPE), dnd_message->reason & DND_CLEAR_EVENT_TYPE,
+    	cm->message_type == msg ? "Valid DnD message" : "Invalid DnD message"));
+
     if (cm->message_type != msg)
     {
 	_XmWarning(dc, "Invalid Drag/Drop message.\n");
@@ -853,6 +860,9 @@ invoke_initiator_callback(Widget dc, int cr_reason)
 
     DEBUGOUT(XdbDebug(__FILE__, dc, "%s:invoke_initiator_callback(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSOURCE", dc, "%s:invoke_initiator_callback(%d) %s\n",
+    	__FILE__, __LINE__,
+    	XdbDragAndDropMessageType2String(cr_reason)));
 
     switch (cr_reason)
     {
@@ -1014,6 +1024,9 @@ invoke_receiver_callback(Widget dc, XmDragDropCallbackStruct * cbs)
 
     DEBUGOUT(XdbDebug(__FILE__, dc, "%s:invoke_receiver_callback(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSOURCE", dc, "%s:invoke_receiver_callback(%d) - %s\n",
+    	__FILE__, __LINE__,
+    	XdbDragAndDropMessageType2String(cbs->any.reason)));
 
     switch (cbs->any.reason)
     {
@@ -1176,6 +1189,8 @@ external_msg_handler(Widget widget, XtPointer client_data, XEvent *event,
 
     DEBUGOUT(XdbDebug(__FILE__, widget, "%s:external_msg_handler(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAG_SOURCE", widget, "%s:external_msg_handler(%d)\n",
+    	__FILE__, __LINE__));
 
     if (dc != NULL && event->type != ClientMessage)
     {
@@ -1441,6 +1456,8 @@ create_root_info(Widget w)
 
     DEBUGOUT(XdbDebug(__FILE__, w, "%s:create_root_info(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSOURCE", w, "%s:create_root_info(%d)\n",
+    	__FILE__, __LINE__));
 
     if (DC_NumReceiverInfos(w) == 0)
     {
@@ -1464,9 +1481,15 @@ create_root_info(Widget w)
 
     if (!_XmGetDragReceiverInfo(disp, ri->window, ri))
     {
+	DEBUGOUT(XdbDebug("DRAGSOURCE", w, "%s:create_root_info(%d) - couldn't get receiver info\n",
+	    __FILE__, __LINE__));
 	return;
     }
 
+    DEBUGOUT(XdbDebug("DRAGSOURCE", w, "%s:create_root_info(%d) - protocol %s\n",
+    	__FILE__, __LINE__,
+    	XdbDragType2String(ri->dragProtocolStyle)));
+
     if (ri->dragProtocolStyle == XmDRAG_NONE ||
 	ri->dragProtocolStyle == XmDRAG_DROP_ONLY ||
 	ri->dragProtocolStyle == XmDRAG_DYNAMIC)
@@ -1706,6 +1729,8 @@ get_current_receiver(Widget dc, Window root, Window win)
 
     DEBUGOUT(XdbDebug(__FILE__, dc, "%s:get_current_receiver(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSOURCE", dc, "%s:get_current_receiver(%d)\n",
+    	__FILE__, __LINE__));
 
     DC_CrossingTime(dc) = DC_LastChangeTime(dc);
 
@@ -1820,27 +1845,27 @@ drag_main_loop(XtPointer data, XtIntervalId *id)
 
     fw = XmGetFocusWidget(DC_SrcShell(dc));
 
-	/* T. Straumann: avoid dereferencing NULL fw */
-	if (fw)
-	{
-    if (_XmGetFocusPolicy(DC_SrcShell(dc)) != XmEXPLICIT)
+    /* T. Straumann: avoid dereferencing NULL fw */
+    if (fw)
     {
-	WidgetClass wc = XtClass(fw);
+	if (_XmGetFocusPolicy(DC_SrcShell(dc)) != XmEXPLICIT)
+	{
+	    WidgetClass wc = XtClass(fw);
 
-	if (XmIsPrimitive(fw) && PrimC_BorderUnhighlight(wc) != NULL)
-	{
-	    PrimC_BorderUnhighlight(wc) (fw);
+	    if (XmIsPrimitive(fw) && PrimC_BorderUnhighlight(wc) != NULL)
+	    {
+		PrimC_BorderUnhighlight(wc) (fw);
+	    }
+	    else if (XmIsGadget(fw) && GC_BorderUnhighlight(wc) != NULL)
+	    {
+		GC_BorderUnhighlight(wc) (fw);
+	    }
 	}
-	else if (XmIsGadget(fw) && GC_BorderUnhighlight(wc) != NULL)
+	else
 	{
-	    GC_BorderUnhighlight(wc) (fw);
+	    XtSetKeyboardFocus(DC_SrcShell(dc), NULL);
 	}
     }
-    else
-    {
-	XtSetKeyboardFocus(DC_SrcShell(dc), NULL);
-    }
-	}
 
     write_initiator(dc);
 
@@ -2313,6 +2338,7 @@ _XmDCStartDrag(XmDragContext dc,
     Window win;
 
     DEBUGOUT(XdbDebug(__FILE__, srcW, "_XmDCStartDrag\n"));
+    DEBUGOUT(XdbDebug("DRAGSOURCE", srcW, "_XmDCStartDrag\n"));
 
     disp = _XmDCtoDD((Widget)dc);
 
@@ -2358,6 +2384,9 @@ _XmDCStartDrag(XmDragContext dc,
 
     DC_ActiveProtocolStyle(dc) = _XmGetActiveProtocolStyle((Widget)dc);
 
+    DEBUGOUT(XdbDebug("DRAGSOURCE", srcW, "_XmDCStartDrag - Protocol %s (%d)\n",
+    	XdbDragType2String(DC_ActiveProtocolStyle(dc)), DC_ActiveProtocolStyle(dc)));
+
     /* the protocol gotten from _XmGetActiveProtocolStyle() needs to be
      * filtered for reasonable values for the initiator. */
     switch (DC_ActiveProtocolStyle(dc))
@@ -2775,6 +2804,9 @@ XmDragStart(Widget widget,
     ArgList merged;
 
     DEBUGOUT(XdbDebug(__FILE__, widget, "XmDragStart()\n"));
+    DEBUGOUT(XdbDebug("DRAGSOURCE", widget, "XmDragStart() - %s\n",
+    	XdbDragType2String(Display_DragInitiatorProtocolStyle(disp))));
+    DEBUGOUT(XdbPrintArgList("DRAGSOURCE", widget, arglist, argcount, False));
 
     if (Display_DragInitiatorProtocolStyle(disp) != XmDRAG_NONE)
     {
