@@ -1,6 +1,6 @@
 /** -*- Mode: C; tab-width: 8; indent-tabs-mode: nil -*-
  *
- * $Id: DropSMgr.c,v 1.25 2000/01/05 23:44:46 rwscott Exp $
+ * $Id: DropSMgr.c,v 1.27 2000/02/13 12:43:26 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DropSMgr.c,v 1.25 2000/01/05 23:44:46 rwscott Exp $";
+static const char rcsid[] = "$Id: DropSMgr.c,v 1.27 2000/02/13 12:43:26 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -33,6 +33,7 @@ static const char rcsid[] = "$Id: DropSMgr.c,v 1.25 2000/01/05 23:44:46 rwscott
 #include <Xm/DragCP.h>
 #include <Xm/DropTrans.h>
 #include <Xm/XmP.h>
+#include <Xm/AtomMgr.h>
 #include <X11/Xfuncs.h>
 
 #include <XmI/DebugUtil.h>
@@ -469,12 +470,43 @@ updateReceiverInfo(Widget w, XtPointer cd, XEvent *ev, Boolean *ctd)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "%s:updateReceiverInfo(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSINK", w, "%s:updateReceiverInfo(%d)\n",
+    	__FILE__, __LINE__));
+#if 1
+    /* rws 8 Feb 2000
+       I have no idea if this belongs here or not, but it has to be
+       created at some point.....
+     */
+    /* rws 12 Feb 2000
+       This should probably be DragBS.c:_XmSetDragReceiverInfo()
+     */
+    {
+    XmDndReceiverProp ri;
+    Atom pa;
+    Widget shell = w;
+
+	DEBUGOUT(XdbDebug("DND", shell, "%s:createShellInfoRec(%d)\n",
+	    __FILE__, __LINE__));
+    	ri.byte_order = _XmByteOrder();
+    	ri.protocol_version = DND_PROTOCOL_VERSION;
+    	ri.protocol_style = XmDRAG_DYNAMIC;
+    	ri.proxy_window = (CARD32)None;
+    	ri.num_drop_sites = (CARD32)0;
+    	ri.total_size = (CARD32)sizeof(XmDndReceiverProp);
+	pa = XmInternAtom(XtDisplay(shell), _XA_MOTIF_DRAG_RECEIVER_INFO, False);
+	XChangeProperty(XtDisplay(shell), XtWindow(shell), 
+		pa, pa, 8, PropModeReplace,
+		(unsigned char *)&ri, sizeof(ri));
+    }
+#endif
 }
 
 static void
 externalSourceHandler(Widget w, XtPointer cd, XEvent *ev, Boolean *ctd)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:externalSourceHandler(%d)\n",
+    DEBUGOUT(XdbDebug2(__FILE__, w, (Widget)cd, "%s:externalSourceHandler(%d)\n",
+    	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug2("DRAGSINK", w, (Widget)cd, "%s:externalSourceHandler(%d)\n",
     	__FILE__, __LINE__));
 #if 0
     Widget dc, disp;
@@ -507,6 +539,9 @@ updateTree(Widget w, XtPointer cd, XtPointer cbs)
 
     DEBUGOUT(XdbDebug(__FILE__, w, "%s:updateTree(%d)\n",
     	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSINK", w, "%s:updateTree(%d)\n",
+    	__FILE__, __LINE__));
+
     if (Display_DragReceiverProtocolStyle(disp) == XmDRAG_NONE)
     {
 	return;
@@ -736,12 +771,18 @@ __XmDSMCreateInfo(XmDropSiteManagerObject object,
     Widget shell;
     XmDropSiteInfo info;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)object,
+    DEBUGOUT(XdbDebug2(__FILE__, (Widget)object, widget,
 		      "%s:__XmDSMCreateInfo(%d) - %i args\n", 
 		      __FILE__, __LINE__,
 		      num_args
 		      ));
     DEBUGOUT(XdbPrintArgList(__FILE__, (Widget)object, args, num_args, False));
+    DEBUGOUT(XdbDebug2("DRAGSINK", (Widget)object, widget,
+		      "%s:__XmDSMCreateInfo(%d) - %i args\n", 
+		      __FILE__, __LINE__,
+		      num_args
+		      ));
+    DEBUGOUT(XdbPrintArgList("DRAGSINK", (Widget)object, args, num_args, False));
 
     DSMStartUpdate(object, widget);
 
@@ -760,7 +801,9 @@ __XmDSMCreateInfo(XmDropSiteManagerObject object,
     }
 
     if (hr->animationStyle == XmDRAG_UNDER_PIXMAP &&
-	hr->animationPixmap != None && hr->animationPixmapDepth == 0)
+	hr->animationPixmap != None &&
+        hr->animationPixmap != XmUNSPECIFIED_PIXMAP &&
+	hr->animationPixmapDepth == 0)
     {
 	XGetGeometry(XtDisplay(widget), hr->animationPixmap, &root,
 		     &x, &y, &width, &height, &border,
@@ -994,7 +1037,9 @@ __XmDSMUpdateInfo(XmDropSiteManagerObject dsm,
     }
 
     if (ndsi->animationStyle == XmDRAG_UNDER_PIXMAP &&
-	ndsi->animationPixmap != None && ndsi->animationPixmapDepth == 0)
+	ndsi->animationPixmap != None &&
+        ndsi->animationPixmap != XmUNSPECIFIED_PIXMAP &&
+        ndsi->animationPixmapDepth == 0)
     {
 	XGetGeometry(XtDisplay(widget), ndsi->animationPixmap, &root,
 		     &x, &y, &width, &height, &border,
@@ -1098,8 +1143,10 @@ loc_in_info(XmDropSiteManagerObject dsm, XmDropSiteInfo dsi,
 {
     Position tx, ty;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:loc_in_info(%d)\n",
-    	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug2(__FILE__, (Widget)dsm, dsi->dropSite, "%s:loc_in_info(%d) - %+i%+i %+i%+i\n",
+    	__FILE__, __LINE__,
+    	x, y,
+    	DS_RootX(dsm), DS_RootY(dsm)));
 
     XtTranslateCoords(dsi->dropSite, 0, 0, &tx, &ty);
 
@@ -1108,6 +1155,8 @@ loc_in_info(XmDropSiteManagerObject dsm, XmDropSiteInfo dsi,
 
     x -= tx;
     y -= ty;
+    DEBUGOUT(XdbDebug0(__FILE__, (Widget)dsm, "\t%+i%+i\n",
+    	x, y));
 
     if (_XmRegionPointInRegion(dsi->region, x, y))
     {
@@ -1126,12 +1175,14 @@ loc_to_info(XmDropSiteManagerObject dsm, XmDropSiteInfo dsi,
     int i;
     XmDropSiteInfo child;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:loc_to_info(%d)\n",
-    	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:loc_to_info(%d) - %+i%+i %s\n",
+    	__FILE__, __LINE__,
+    	x, y,
+    	dsi->leaf ? "Leaf" : "Not leaf"));
 
     if (!dsi->leaf)
     {
-
+	DEBUGOUT(XdbDebug0(__FILE__, (Widget)dsm, "\t%i children\n", dsi->numChildren));
 	for (i = 0; i < dsi->numChildren; i++)
 	{
 	    child = dsi->children[i];
@@ -1462,7 +1513,9 @@ __XmDSMProcessMotion(XmDropSiteManagerObject dsm,
     XmDropSiteInfo ndsi;
     unsigned char ps;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM ProcessMotion\n"));
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "__XmDSMProcessMotion - %s %+i%+i %+i%+i\n",
+    	XdbDragAndDropMessageType2String(cbs->reason),
+    	cbs->x, cbs->y, DS_RootX(dsm), DS_RootY(dsm)));
 
     if (DS_CurDragContext(dsm) == NULL)
     {
@@ -1493,6 +1546,9 @@ __XmDSMProcessMotion(XmDropSiteManagerObject dsm,
 
             DEBUGOUT(XdbDebug(__FILE__, 
                               (Widget)dsm, "Setting dsi to %p\n", ndsi ));
+            DEBUGOUT(XdbDebug("DRAGSINK", 
+                              (Widget)dsm, "Setting dsi to %s\n", 
+                              ndsi ? XtName(ndsi->dropSite) : "NULL"));
 	    DS_CurInfo(dsm) = (XtPointer)ndsi;
 
 	    if (ndsi)
@@ -1503,7 +1559,8 @@ __XmDSMProcessMotion(XmDropSiteManagerObject dsm,
     }
 
     DEBUGOUT(XdbDebug(__FILE__, 
-                      (Widget)dsm, "Dragging on %p\n", dsi ));
+                      (Widget)dsm, "Dragging on %s\n", 
+                      dsi ? XtName(dsi->dropSite) : "NULL"));
     drag_motion(dsm, cd, cbs, dsi, ps);
 }
 
@@ -1528,7 +1585,7 @@ __XmDSMProcessDrop(XmDropSiteManagerObject dsm,
     Cardinal nimps;
     Atom *itargs;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM ProcessDrop\n"));
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "__XmDSMProcessDrop\n"));
 
     dc = XmGetDragContext((Widget)dsm, cbs->timeStamp);
 
@@ -1803,8 +1860,8 @@ __XmDSMChangeRoot(XmDropSiteManagerObject dsm,
     (XmDragTopLevelClientDataStruct *) clientData;
     XmDragDropCallbackStruct *cbs = (XmDragDropCallbackStruct *) callData;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM ChangeRoot: %d\n",
-		      cbs->any.reason));
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "__XmDSMChangeRoot: %s\n",
+		      XdbDragAndDropMessageType2String(cbs->any.reason)));
 
     DS_CurTime(dsm) = cbs->any.timeStamp;
 
@@ -1869,7 +1926,10 @@ __XmDSMInsertInfo(XmDropSiteManagerObject dsm,
     Widget w, parent;
     XmTreeUpdateCallbackStruct tucbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM InsertInfo\n"));
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "%s:__XmDSMInsertInfo(%d)\n",
+    	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug("DRAGSINK", (Widget)dsm, "%s:__XmDSMInsertInfo(%d)\n",
+    	__FILE__, __LINE__));
 
     w = dsi->dropSite;
 
@@ -1985,8 +2045,8 @@ static void
 __XmDSMSyncTree(XmDropSiteManagerObject dsm,
 		Widget shell)
 {
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM SyncTree\n"));
-
+    DEBUGOUT(XdbDebug2(__FILE__, (Widget)dsm, shell, "%s:__XmDSMSyncTree(%d)\n",
+    	__FILE__, __LINE__));
 }
 
 /*
@@ -2051,7 +2111,10 @@ __XmDSMRegisterInfo(XmDropSiteManagerObject dsm,
     DSInfoTable *dst;
     long hash;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)dsm, "DSM RegisterInfo\n"));
+    DEBUGOUT(XdbDebug2(__FILE__, (Widget)dsm, widget, "%s:__XmDSMRegisterInfo(%d)\n",
+    	__FILE__, __LINE__));
+    DEBUGOUT(XdbDebug2("DRAGSINK", (Widget)dsm, widget, "%s:__XmDSMRegisterInfo(%d)\n",
+    	__FILE__, __LINE__));
 
     if (dsi->registered)
     {
@@ -2342,6 +2405,11 @@ XmDropSiteRegister(Widget widget,
 		      __FILE__, __LINE__,
 		      argcount));
     DEBUGOUT(XdbPrintArgList(__FILE__, widget, arglist, argcount, False));
+    DEBUGOUT(XdbDebug("DRAGSINK", widget,
+		      "%s:XmDropSiteRegister(%d) - %i args\n",
+		      __FILE__, __LINE__,
+		      argcount));
+    DEBUGOUT(XdbPrintArgList("DRAGSINK", widget, arglist, argcount, False));
 
     DSMCreateInfo(_XmGetDropSiteManagerObject((XmDisplay)disp),
 		  widget, arglist, argcount);
