@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DropTrans.c,v 1.9 1998/09/12 15:05:26 rwscott Exp $
+ * $Id: DropTrans.c,v 1.11 2000/02/13 12:43:27 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DropTrans.c,v 1.9 1998/09/12 15:05:26 rwscott Exp $";
+static const char rcsid[] = "$Id: DropTrans.c,v 1.11 2000/02/13 12:43:27 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -214,7 +214,17 @@ initialize(Widget request, Widget new_w,
 {
     XmDropTransferEntry entries;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "DropTransfer initialize\n"));
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:initialize(%d) - %i args\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (DT_NumDropTransfers(new_w) != 0)
     {
@@ -281,7 +291,20 @@ static Boolean
 set_values(Widget current, Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "DropTransfer set_values\n"));
+    DEBUGOUT(XdbDebug(__FILE__, new_w,
+		      "%s:set_values(%d) - %i args\n"
+		      "\t    old X %5i Y %5i W %5i H %5i\n"
+		      "\trequest X %5i Y %5i W %5i H %5i\n"
+		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
+		      *num_args,
+		      XtX(current), XtY(current),
+		      XtWidth(current), XtHeight(current),
+		      XtX(request), XtY(request),
+		      XtWidth(request), XtHeight(request),
+		      XtX(new_w), XtY(new_w),
+		      XtWidth(new_w), XtHeight(new_w)));
+    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     return True;
 }
@@ -293,6 +316,9 @@ entry_transfer(Widget dt, int which)
     Widget dc = DT_DragContext(dt);
     int i;
 
+    DEBUGOUT(XdbDebug(__FILE__, dt, "%s:entry_transfer(%d)",
+    	__FILE__, __LINE__));
+
     DT_CurDropTransferList(dt) = which;
     DT_CurTargets(dt) = (Atom *)XtMalloc(lst->num_transfers * sizeof(Atom));
     DT_CurClientData(dt) = (XtPointer *)XtMalloc(lst->num_transfers *
@@ -330,6 +356,9 @@ static void
 notified_callback(Widget w, XtPointer cd, Atom *select, Atom *type,
 		  XtPointer value, unsigned long *len, int *fmt)
 {
+    DEBUGOUT(XdbDebug(__FILE__, w, "%s:notified_callback(%d)",
+    	__FILE__, __LINE__));
+
     if (value != NULL)
     {
 	XtFree((char *)value);
@@ -343,6 +372,12 @@ terminate_transfer(Widget dt, Atom *select)
 {
     Atom how;
 
+    DEBUGOUT(XdbDebug(__FILE__, dt, "%s:terminate_transfer(%d) - drag context %s receiver info %s\n",
+    	__FILE__, __LINE__,
+    	DT_DragContext(dt) ? "Yes" : "No",
+    	DC_CurrReceiverInfo(DT_DragContext(dt)) ?  "Yes" : "No"
+    	));
+
     if (DT_TransferStatus(dt) == XmTRANSFER_SUCCESS)
     {
 	how = XmInternAtom(XtDisplay(dt), _XA_XmTRANSFER_SUCCESS, False);
@@ -364,6 +399,9 @@ select_callback(Widget w, XtPointer cd, Atom *select, Atom *type,
     Widget dt = (Widget)cd;
     XmDropTransferList lst;
 
+    DEBUGOUT(XdbDebug(__FILE__, dt, "%s:select_callback(%d)",
+    	__FILE__, __LINE__));
+
     lst = &DT_DropTransferLists(dt)[DT_CurDropTransferList(dt)];
 
     if (DT_TransferCallback(dt))
@@ -407,6 +445,9 @@ drop_timer(XtPointer cd, XtIntervalId *id)
     Widget dt = (Widget)cd;
     Atom select;
 
+    DEBUGOUT(XdbDebug(__FILE__, dt, "%s:drop_timer(%d)\n",
+    	__FILE__, __LINE__));
+
     if (DT_NumDropTransferLists(dt))
     {
 	entry_transfer(dt, 0);
@@ -425,6 +466,9 @@ start_drop_transfer(Widget refWidget,
 {
     Widget disp = XmGetXmDisplay(XtDisplay(refWidget)), dt;
 
+    DEBUGOUT(XdbDebug(__FILE__, refWidget, "%s:start_drop_transfer(%d)\n",
+    	__FILE__, __LINE__));
+
     dt = XtCreateWidget("drop_transfer",
 			Display_DropTransferClass(disp),
 			disp, args, argCount);
@@ -446,6 +490,9 @@ add_drop_transfer(Widget widget,
     XmDropTransferEntry entries;
     int idx;
 
+    DEBUGOUT(XdbDebug(__FILE__, widget, "%s:add_drop_transfer(%d)",
+    	__FILE__, __LINE__));
+
     idx = DT_NumDropTransferLists(widget);
     DT_NumDropTransferLists(widget)++;
 
@@ -461,3 +508,33 @@ add_drop_transfer(Widget widget,
     DT_DropTransferLists(widget)[idx].transfer_list = entries;
     DT_DropTransferLists(widget)[idx].num_transfers = num_transfers;
 }
+
+void
+XmDropTransferAdd(Widget drop_transfer,
+		  XmDropTransferEntryRec *transfers,
+		  Cardinal num_transfers)
+{
+    DEBUGOUT(XdbDebug(__FILE__, drop_transfer, "%s:XmDropTransferAdd(%d)",
+    	__FILE__, __LINE__));
+
+    DTC_AddTransferProc(XtClass(drop_transfer)) (drop_transfer,
+						 transfers,
+						 num_transfers);
+}
+
+Widget
+XmDropTransferStart(Widget widget,
+		    ArgList arglist,
+		    Cardinal argcount)
+{
+    Widget disp = XmGetXmDisplay(XtDisplay(widget));
+
+    DEBUGOUT(XdbDebug(__FILE__, widget, "%s:XmDropTransferAdd(%d)\n",
+    	__FILE__, __LINE__));
+    DEBUGOUT(XdbPrintArgList(__FILE__, widget, arglist, argcount, False));
+
+
+    return DTC_StartTransferProc(Display_DropTransferClass(disp)) (widget,
+								   arglist,
+								   argcount);
+}
