@@ -1,7 +1,7 @@
 #define MOZILLA
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.83 1999/12/08 20:57:51 danny Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.87 2000/03/05 13:12:25 rwscott Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.83 1999/12/08 20:57:51 danny Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.87 2000/03/05 13:12:25 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -200,12 +200,14 @@ static XtResource resources[] =
 	sizeof(int), SBOffset(list_item_count),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED
     },
-#if XmVERSION > 2
+#if 0
+#if XmVERSION > 1
     {
 	XmNpathMode, XmCPathMode, XtEnum,
 	sizeof(int), SBOffset(path_mode),
 	XmRImmediate, XmPATH_MODE_FULL
-    }
+    },
+#endif
 #endif
 };
 
@@ -407,11 +409,12 @@ XmFileSelectionBoxClassRec xmFileSelectionBoxClassRec = {
     /* XmSelectionBox part */
     {
 	/* Motif has something called a list_callback here */
-	/* extension */ NULL,
+	/* list_callback                */ NULL,
+	/* extension                    */ NULL,
     },
     /* XmFileSelectionBox part */
     {
-	/* extension */ NULL,
+	/* extension                    */ NULL,
     }
 };
 /* *INDENT-ON* */
