@@ -1,6 +1,6 @@
 /**
  * 
- * $Id: MenuShell.c,v 1.70 1998/10/16 14:43:24 rwscott Exp $
+ * $Id: MenuShell.c,v 1.81 1999/09/29 16:38:50 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: MenuShell.c,v 1.70 1998/10/16 14:43:24 rwscott Exp $";
+static const char rcsid[] = "$Id: MenuShell.c,v 1.81 1999/09/29 16:38:50 rwscott Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -41,7 +41,7 @@ static const char rcsid[] = "$Id: MenuShell.c,v 1.70 1998/10/16 14:43:24 rwscott
 #undef USE_FOCUS
 
 /* Forward Declarations */
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -210,7 +210,7 @@ XmMenuShellClassRec xmMenuShellClassRec = {
 WidgetClass xmMenuShellWidgetClass = (WidgetClass)&xmMenuShellClassRec;
 
 static void
-class_initialize()
+class_initialize(void)
 {
     _XmInitializeExtensions();
 
@@ -497,6 +497,27 @@ change_managed(Widget w)
 		}
 	    }
 	}
+#if 0
+	/* rws 26 Sep 1999
+	   This is for http://www.bartels.de/baedform.htm. It seems that it
+	   un-manages the menus somewhwhere along the line. We manage them
+	   in insert_child, and expect them to stay managed.
+	 */
+	 /* rws 29 Sep 1999
+	    However this messes up Mozilla's Go menu, sigh. So instead just
+	    make sure the menu is managed in MenuShellPopupSharedMenuPane
+	  */
+	else
+	{
+	    if (MS_PrivateShell(w))
+	    {
+		if (!XtIsManaged(MGR_Children(w)[i]))
+		{
+		    XtManageChild(MGR_Children(w)[i]);
+		}
+	    }
+	}
+#endif
     }
 
     if (!child)
@@ -519,7 +540,9 @@ change_managed(Widget w)
 		_XmRemoveGrab(w);
 
 		XtUnmapWidget(w);
+		XtCallCallbacks(w, XmNpopdownCallback, NULL);
 		Shell_PoppedUp(w) = False;
+
 		if (_XmIsActiveTearOff(MGR_Children(w)[0]))
 		{
 		Widget rc = MGR_Children(w)[0];
@@ -588,6 +611,12 @@ change_managed(Widget w)
 			DEBUGOUT(XdbDebug(__FILE__, w, "Popping up public shell\n"));
 		    	XtManageChild(child);
 			XMapRaised(XtDisplay(w), XtWindow(w));
+			XtCallCallbacks(w, XmNpopupCallback, NULL);
+#ifdef	NEW_MAPCALLBACK
+    /*
+    _XmCallRowColumnMapCallback(w2, event);
+    */
+#endif
 			Shell_PoppedUp(w) = True;
 		    }
 		}
@@ -609,6 +638,7 @@ change_managed(Widget w)
 			}
 
 			XtUnmapWidget(w);
+			XtCallCallbacks(w, XmNpopdownCallback, NULL);
 			XtUnmanageChild(child);
 		    }
 		}
@@ -642,6 +672,11 @@ insert_child(Widget w)
     }
     else
     {
+	/* T. Straumann: M*TIF enforces 0 borderWidth on MenuShell's child */
+	if ( 0 != XtBorderWidth(w) )
+	{
+		XtVaSetValues(w,XmNborderWidth,0,NULL);
+	}
 #define superclass (&overrideShellClassRec)
 	(*superclass->composite_class.insert_child) (w);
 #undef superclass
@@ -911,8 +946,10 @@ MenuShellPopdownOne(Widget w, XEvent *event,
 	}
 
 	XtUnmapWidget(w);
+	XtCallCallbacks(w, XmNpopdownCallback, NULL);
 	XtUnmanageChild(rc);
     }
+	_XmCallRowColumnUnmapCallback(rc, event);
 }
 
 static void
@@ -963,6 +1000,9 @@ MenuShellPopdownEveryone(Widget w, XEvent *event,
 	DEBUGOUT(XdbDebug(__FILE__, w, "  calling popdownOne.\n"));
 
 	MenuShellPopdownOne(XtParent(rc), event, params, num_params);
+#if 0
+	_XmCallRowColumnUnmapCallback(rc, event);
+#endif
 	DEBUGOUT(XdbDebug2(__FILE__, w, rc,"child %i parent %s\n", 
 			i,
 			XtName(XtParent(rc))));
@@ -986,11 +1026,18 @@ MenuShellPopupSharedMenuPane(Widget w, Widget w2, XEvent *event)
     /* must be a menu shell here. */
     assert(XmIsMenuShell(w));
 
+    XtManageChild(w2);
     XRaiseWindow(XtDisplay(w2), XtWindow(w2));
     _XmResizeObject(w, XtWidth(w2) == 0 ? 1 : XtWidth(w2), XtHeight(w2) == 0 ? 1 : XtHeight(w2), 0);
  
     XtRealizeWidget(w);
     XMapRaised(XtDisplay(w), XtWindow(w));
+    XtCallCallbacks(w, XmNpopupCallback, NULL);
+#ifdef	NEW_MAPCALLBACK
+    /*
+    _XmCallRowColumnMapCallback(w2, event);
+    */
+#endif
     Shell_PoppedUp(w) = True;
  
 #ifdef USE_FOCUS
@@ -1012,6 +1059,12 @@ _XmXtMenuPopup(Widget widget, XEvent *event,
     /* for popup menus */
     XtRealizeWidget(widget);
     XMapRaised(XtDisplay(widget), XtWindow(widget));
+    XtCallCallbacks(widget, XmNpopupCallback, NULL);
+#ifdef	NEW_MAPCALLBACK
+    /*
+    _XmCallRowColumnMapCallback(w2, event);
+    */
+#endif
     Shell_PoppedUp(widget) = True;
  
     XAllowEvents(XtDisplay(widget), SyncBoth, event->xbutton.time);
@@ -1036,6 +1089,7 @@ _XmXtMenuPopdown(Widget widget, XEvent *event,
     if (MS_PrivateShell(widget))
     {
         XtUnmapWidget(widget);
+	XtCallCallbacks(widget, XmNpopdownCallback, NULL);
 	Shell_PoppedUp(widget) = False;
 
 #ifdef USE_FOCUS
