@@ -1,6 +1,6 @@
 /**
  *
- * $Id: PanedW.c,v 1.42 1998/10/18 22:01:01 rwscott Exp $
+ * $Id: PanedW.c,v 1.45 1999/11/19 13:13:14 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PanedW.c,v 1.42 1998/10/18 22:01:01 rwscott Exp $";
+static const char rcsid[] = "$Id: PanedW.c,v 1.45 1999/11/19 13:13:14 amai Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -223,9 +223,11 @@ static void realize(Widget w,
 		    XtValueMask *value_mask,
 		    XSetWindowAttributes *attributes);
 
+/*
 static XtGeometryResult query_geometry(Widget w,
 				       XtWidgetGeometry *proposed,
 				       XtWidgetGeometry *desired);
+*/
 
 /* composite */
 static void insert_child(Widget w);
@@ -250,7 +252,7 @@ static Boolean constraint_set_values(Widget current,
 				     ArgList args,
 				     Cardinal *num_args);
 
-Cardinal panedw_insert_position(Widget w);
+static Cardinal panedw_insert_position(Widget w);
 
 /* helpers */
 static void _XmPanedWPreferredSize(Widget w, Widget child,
@@ -316,7 +318,7 @@ static XtResource resources[] =
     {
 	XmNsashShadowThickness, XmCShadowThickness, XmRHorizontalDimension,
 	sizeof(Dimension), Offset(sash_shadow_thickness),
-	XmRCallProc, (XtPointer)_XmPrimitiveShadowThicknessDefault
+	XmRImmediate, (XtPointer)2
     },
     {
 	XmNinsertPosition, XmCInsertPosition, XmRFunction,
@@ -491,7 +493,7 @@ XmPanedWindowClassRec xmPanedWindowClassRec = {
 	/* version               */ XtVersion,
 	/* callback offsets      */ NULL,
 	/* tm_table              */ XtInheritTranslations,
-	/* query_geometry        */ query_geometry,
+	/* query_geometry        */ NULL /*query_geometry*/,
 	/* display_accelerator   */ NULL,
 	/* extension             */ (XtPointer)NULL /*&_XmPanedWCoreClassExtRec*/
     },
@@ -982,7 +984,7 @@ realize(Widget w,
 /*
  * Insert panes before sashes and separators.
  */
-Cardinal
+static Cardinal
 panedw_insert_position(Widget w)
 {
     XmPanedWindowWidget pw = (XmPanedWindowWidget)XtParent(w);
@@ -1087,6 +1089,7 @@ delete_child(Widget w)
 #undef superclass
 }
 
+#if 0
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
 {
@@ -1124,6 +1127,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
     return _XmGMReplyToQueryGeometry(w, proposed, desired);
 #endif
 }
+#endif
 
 static XtGeometryResult
 geometry_manager(Widget w,
