@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ResConvert.c,v 1.39 1998/10/20 20:44:55 danny Exp $
+ * $Id: ResConvert.c,v 1.57 2000/03/02 21:01:08 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,17 +22,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ResConvert.c,v 1.39 1998/10/20 20:44:55 danny Exp $";
+static const char rcsid[] = "$Id: ResConvert.c,v 1.57 2000/03/02 21:01:08 amai Exp $";
 
 #include <LTconfig.h>
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef __STDC__
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 #include <ctype.h>
 
 #ifdef HAVE_STRING_H
@@ -41,6 +37,7 @@ static const char rcsid[] = "$Id: ResConvert.c,v 1.39 1998/10/20 20:44:55 danny
 
 #include <XmI/XmI.h>
 #include <Xm/XmP.h>
+#include <Xm/AtomMgr.h>
 #include <Xm/RepType.h>
 #include <Xm/BulletinBP.h>
 #include <Xm/MenuShellP.h>
@@ -106,6 +103,15 @@ static void destroy_xm_string_table(XtAppContext app, XrmValue *to,
 				    Cardinal *num_args);
 static void get_unit_type(Widget w, Cardinal *size, XrmValue *val);
 
+#if XmVERSION > 1
+static Boolean _XmCvtStringToTabList(Display *display, XrmValue *args,
+		       Cardinal *num_args, XrmValue *from, XrmValue *to,
+		       XtPointer *converter_data);
+static Boolean _XmCvtStringToRenderTable(Display *display, XrmValue *args,
+		       Cardinal *num_args, XrmValue *from, XrmValue *to,
+		       XtPointer *converter_data);
+#endif
+
 int	xmUseVersion = 0;
 
 /* please be careful if you're editting these arrays, as
@@ -149,24 +155,19 @@ static unsigned char label_type_values[] =
 static char *vertical_alignments[] =
 {
     "alignment_baseline_top",
+    "alignment_center",
+    "alignment_baseline_bottom",
+    "alignment_contents_top",
+    "alignment_contents_bottom"
+};
+
+static char *child_vertical_alignments[] =
+{
+    "alignment_baseline_top",
+    "alignment_center",
     "alignment_baseline_bottom",
     "alignment_widget_top",
-    "alignment_widget_bottom",
-    "alignment_contents_top",
-    "alignment_contents_bottom",
-    "alignment_left",
-    "alignment_right",
-};
-static unsigned char vertical_alignment_values[] =
-{
-    XmALIGNMENT_BASELINE_TOP,
-    XmALIGNMENT_BASELINE_BOTTOM,
-    XmALIGNMENT_WIDGET_TOP,
-    XmALIGNMENT_WIDGET_BOTTOM,
-    XmALIGNMENT_CONTENTS_TOP,
-    XmALIGNMENT_CONTENTS_BOTTOM,
-    0,				/* I can't find these two anywhere in Motif */
-    0				/* I can't find these two anywhere in Motif */
+    "alignment_widget_bottom"
 };
 
 static char *horizontal_alignments[] =
@@ -175,12 +176,6 @@ static char *horizontal_alignments[] =
     "alignment_center",
     "alignment_end"
 };
-static unsigned char horizontal_alignment_values[] =
-{
-    XmALIGNMENT_BEGINNING,
-    XmALIGNMENT_CENTER,
-    XmALIGNMENT_END
-};
 
 static char *arrow_directions[] =
 {
@@ -330,8 +325,7 @@ static char *separator_types[] =
     "shadow_etched_in",
     "shadow_etched_out",
     "shadow_etched_in_dash",
-    "shadow_etched_out_dash",
-    "invalid_separator_type"
+    "shadow_etched_out_dash"
 };
 
 static char *row_column_types[] =
@@ -355,13 +349,21 @@ static unsigned char row_column_type_values[] =
 static char *indicator_types[] =
 {
     "n_of_many",
-    "one_of_many"
+    "one_of_many",
+#if XmVERSION > 1
+    "one_of_many_round",
+    "one_of_many_diamond"
+#endif
 };
 
 static unsigned char indicator_type_values[] =
 {
     XmN_OF_MANY,
-    XmONE_OF_MANY
+    XmONE_OF_MANY,
+#if XmVERSION > 1
+    XmONE_OF_MANY_ROUND,
+    XmONE_OF_MANY_DIAMOND
+#endif
 };
 
 static char *resize_policies[] =
@@ -498,6 +500,13 @@ static XtConvertArgRec parentArgs[] = {
 };
 
 #if XmVERSION > 1
+static char *toggle_mode[] =
+{
+    "toggle_boolean",
+    "toggle_indeterminate"
+};
+
+/* Spinbox */
 static char *spb_child_types[] =
 {
     "numeric",
@@ -524,6 +533,29 @@ static unsigned char spb_arrow_layout_values[] =
     XmARROWS_SPLIT
 };
 
+#if XmVERSION > 1
+/* Notebook */
+static char *nb_child_types[] =
+{
+    "none",
+    "page",
+    "major_tab",
+    "minor_tab",
+    "status_area",
+    "page_scroller"
+};
+
+static unsigned char nb_child_type_values[] =
+{
+    XmNONE,
+    XmPAGE,
+    XmMAJOR_TAB,
+    XmMINOR_TAB,
+    XmSTATUS_AREA, 
+    XmPAGE_SCROLLER
+};
+#endif
+
 static char *indicator_on[] =
 {
     "indicator_none",
@@ -534,6 +566,12 @@ static char *indicator_on[] =
     "indicator_cross_box",
     "false",			/* Support 1.2 compatible defaults */
     "true",			/* Support 1.2 compatible defaults */
+    "off",			/* Boolean compatible values */
+    "on",
+    "0",
+    "1",
+    "no",
+    "yes",
 };
 
 static unsigned char indicator_on_values[] =
@@ -546,12 +584,18 @@ static unsigned char indicator_on_values[] =
     XmINDICATOR_CROSS_BOX,
     XmINDICATOR_NONE,		/* False */
     XmINDICATOR_FILL,		/* True */
+    XmINDICATOR_NONE,		/* Off */
+    XmINDICATOR_FILL,		/* On */
+    XmINDICATOR_NONE,		/* 0 */
+    XmINDICATOR_FILL,		/* 1 */
+    XmINDICATOR_NONE,		/* no */
+    XmINDICATOR_FILL,		/* yes */
 };
 
 #endif
 
 void
-XmRegisterConverters()
+XmRegisterConverters(void)
 {
     /* don't really know the difference between the two, so
        we'll just call the other from here for now. */
@@ -560,7 +604,7 @@ XmRegisterConverters()
 }
 
 void
-_XmRegisterConverters()
+_XmRegisterConverters(void)
 {
     static Boolean first = True;
     static XtConvertArgRec immediate_arg = {XtImmediate, 0, sizeof(XtPointer)};
@@ -598,7 +642,7 @@ _XmRegisterConverters()
 	}};
 
 #ifdef NONSTANDARD_CONVERTERS
-    extern void _XmRegisterNSEConverters();
+    extern void _XmRegisterNSEConverters(void);
 #endif
 
     if (first)
@@ -630,6 +674,10 @@ _XmRegisterConverters()
     REPTYPE_REGISTER(XmRMultiClick, multi_click);
     REPTYPE_REGISTER(XmRPacking, packing_styles);
     REPTYPE_REGISTER(XmRKeyboardFocusPolicy, focus_policies);
+    REPTYPE_REGISTER(XmRVerticalAlignment, vertical_alignments);
+    REPTYPE_REGISTER(XmRChildVerticalAlignment, child_vertical_alignments);
+    REPTYPE_REGISTER(XmRAlignment, horizontal_alignments);
+    REPTYPE_REGISTER(XmRChildHorizontalAlignment, horizontal_alignments);
     REPTYPE_REGISTER(XmRArrowDirection, arrow_directions);
     REPTYPE_REGISTER(XmRAttachment, attachments);
     REPTYPE_REGISTER(XmRAudibleWarning, audible_warnings);
@@ -660,12 +708,6 @@ _XmRegisterConverters()
     REPTYPE_REGISTER(XmRListSizePolicy, list_size_policies);
 
     REPTYPE_REGISTER_WITH_VALUES(XmRLabelType, label_types, label_type_values);
-    REPTYPE_REGISTER_WITH_VALUES(XmRAlignment, horizontal_alignments,
-				 horizontal_alignment_values);
-    REPTYPE_REGISTER_WITH_VALUES(XmRChildHorizontalAlignment, horizontal_alignments,
-				 horizontal_alignment_values);
-    REPTYPE_REGISTER_WITH_VALUES(XmRChildVerticalAlignment, vertical_alignments,
-				 vertical_alignment_values);
     REPTYPE_REGISTER_WITH_VALUES(XmRShadowType, shadow_types,
 				 shadow_type_values);
     REPTYPE_REGISTER_WITH_VALUES(XmRRowColumnType, row_column_types,
@@ -683,12 +725,15 @@ _XmRegisterConverters()
 
 #if XmVERSION > 1
 /* Motif 2.0 */
+    REPTYPE_REGISTER(XmRToggleMode, toggle_mode);
     REPTYPE_REGISTER_WITH_VALUES(XmRArrowLayout,
                 spb_arrow_layouts, spb_arrow_layout_values);
     REPTYPE_REGISTER_WITH_VALUES(XmRSpinBoxChildType,
                 spb_child_types, spb_child_type_values);
     REPTYPE_REGISTER_WITH_VALUES(XmRIndicatorOn,
                 indicator_on, indicator_on_values);
+    REPTYPE_REGISTER_WITH_VALUES(XmRNotebookChildType,
+                nb_child_types, nb_child_type_values);
 #endif
 
 #undef REPTYPE_REGISTER
@@ -919,6 +964,23 @@ _XmRegisterConverters()
 		       0,	/* number of args to converter routine */
 		       XtCacheNone,	/* caching instructions */
 		       NULL);	/* destructor function */
+#if XmVERSION > 1
+    XtSetTypeConverter(XtRString,	/* source type */
+		       XmRTabList,	/* target type */
+		       _XmCvtStringToTabList,	/* converter routine */
+		       NULL,	/* args for converter routine */
+		       0,	/* number of args to converter routine */
+		       XtCacheNone,	/* caching instructions */
+		       NULL);	/* destructor function */
+
+    XtSetTypeConverter(XtRString,	/* source type */
+		       XmRRenderTable,	/* target type */
+		       _XmCvtStringToRenderTable,	/* converter routine */
+		       NULL,	/* args for converter routine */
+		       0,	/* number of args to converter routine */
+		       XtCacheNone,	/* caching instructions */
+		       NULL);	/* destructor function */
+#endif
 
 #ifdef NONSTANDARD_CONVERTERS
     _XmRegisterNSEConverters();
@@ -929,24 +991,12 @@ _XmRegisterConverters()
  * error convenience
  */
 void
-#ifdef __STDC__
 _XmError(Widget w, char *fmt,...)
 {
     va_list arg_list;
     char buf[256];
 
     va_start(arg_list, fmt);
-#else
-_XmError(w, fmt, va_alist)
-     Widget w;
-     char *fmt;
-     va_dcl
-{
-    char buf[256];
-    va_list arg_list;
-
-    va_start(arg_list);
-#endif
     if (w)
     {
 	vsprintf(buf, fmt, arg_list);
@@ -971,23 +1021,12 @@ _XmError(w, fmt, va_alist)
  * bugs of cource :-(
  */
 void
-#ifdef __STDC__
 _XmWarning(Widget w, char *fmt,...)
 {
     va_list arg_list;
-    char buf[256];
+    char buf[1024];
 
     va_start(arg_list, fmt);
-#else
-_XmWarning(Widget w, fmt, va_list)
-     Widget w;
-     char *fmt;
-     va_dcl
-{
-    char buf[256];
-
-    va_start(arg_list);
-#endif
     if (w)
     {
 	sprintf(buf, "\n    Name: %s\n    Class: %s\n    ",
@@ -1287,7 +1326,7 @@ _XmGetDefaultFontList(Widget w,
 		ve = (XmVendorShellExtObject)_LtFindVendorExt((Widget)par);
 		if (ve && VSEP_ButtonFontList(ve) != NULL)
 		{
-		    fontlist = VSEP_LabelFontList(ve);
+		    fontlist = VSEP_ButtonFontList(ve);
 		    break;
 		}
 	    }
@@ -2107,6 +2146,7 @@ _XmCvtStringToBooleanDimension( Display *display,
 				XtPointer *converter_data)
 {
     static Dimension	value;
+    int			preval;
     String		str;
 
     str = (String)from->addr;
@@ -2120,11 +2160,14 @@ _XmCvtStringToBooleanDimension( Display *display,
 
     if (str != NULL) {
 	if (isdigit(str[0])) {
+	    preval = atoi(from->addr);
 	    value = (Dimension)_XmConvertUnits(*(Screen **)args[0].addr,
 					       (int)*(XtPointer *)args[1].addr,
 					       *(unsigned char *)args[2].addr,
-					       atoi(from->addr),
+					       preval,
 					       XmPIXELS);
+	    if (!value && preval)
+		value = 1;
 	} else {
 	    if ((strcasecmp(str, "true") == 0) || (strcasecmp(str, "yes") == 0)
 			|| (strcasecmp(str, "on") == 0)) {
@@ -2402,9 +2445,13 @@ _XmCvtStringToSomeTable( Display *display,
 	{
 	    Atom *atoms = (Atom *)XtMalloc((nitems + 1) * sizeof(Atom));
 
-
-	    XInternAtoms(display, (char **)table, nitems, False, atoms);
 	    atoms[nitems] = None;
+#if XlibSpecificationRelease > 5
+	    XInternAtoms(display, (char **)table, nitems, False, atoms);
+#else
+	    while (nitems--)
+		XmInternAtom(display, ((char **)table)[nitems], False);
+#endif
 	    XtFree((char *)table);
 	    table = (XtPointer *)atoms;
 	}
@@ -2488,7 +2535,67 @@ get_unit_type(Widget w, Cardinal *size, XrmValue *val)
 {
     static unsigned char ut;
 
-    val->addr = &ut;
+    val->addr = (XPointer)&ut;
     val->size = sizeof ut;
     ut = _XmGetUnitType(w);
 }
+
+#if	XmVERSION > 1
+/*
+ * XmConvertStringToUnits converts a string specification value and returns
+ * the converted value as the return value from the function. This function
+ * uses the specified screen's resolution to compute the number of units for
+ * the string specification.
+ */
+int
+XmConvertStringToUnits(Screen *screen, String spec, int orientation,
+	int to_type, XtEnum *parse_error)
+{
+	/* FIX ME not implemented yet */
+
+	if (screen == NULL)
+		return 0;
+
+	if (orientation != XmHORIZONTAL && orientation != XmVERTICAL)
+		return 0;
+
+	switch (to_type) {
+	case XmPIXELS:
+	case XmMILLIMETERS:
+	case Xm100TH_MILLIMETERS:
+	case XmCENTIMETERS:
+	case XmINCHES:
+	case Xm1000TH_INCHES:
+	case XmPOINTS:
+	case Xm100TH_POINTS:
+	case XmFONT_UNITS:
+	case Xm100TH_FONT_UNITS:
+		break;
+	default:
+		return 0;
+	}
+    return 0;
+}
+
+static Boolean
+_XmCvtStringToTabList(Display *display,
+		       XrmValue *args,
+		       Cardinal *num_args,
+		       XrmValue *from,
+		       XrmValue *to,
+		       XtPointer *converter_data)
+{
+	return False;
+}
+
+static Boolean
+_XmCvtStringToRenderTable(Display *display,
+		       XrmValue *args,
+		       Cardinal *num_args,
+		       XrmValue *from,
+		       XrmValue *to,
+		       XtPointer *converter_data)
+{
+	return False;
+}
+#endif
