@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Scale.c,v 1.30 1998/10/24 00:38:14 rwscott Exp $
+ * $Id: Scale.c,v 1.38 2000/01/21 16:04:22 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,9 +22,10 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Scale.c,v 1.30 1998/10/24 00:38:14 rwscott Exp $";
+static const char rcsid[] = "$Id: Scale.c,v 1.38 2000/01/21 16:04:22 amai Exp $";
 
 #include <LTconfig.h>
+#include <stdlib.h>
 #include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
@@ -67,10 +68,15 @@ static const char rcsid[] = "$Id: Scale.c,v 1.30 1998/10/24 00:38:14 rwscott Exp
 #define SB_TRANSVERSAL_SIZE	15
 #define SB_LONGITUDAL_SIZE	100
 
+/* T. Straumann: special value to indicate that the value rectangle position
+ * is invalid and hence must not be erased.
+ */
+#define SHOW_VALUE_X_INVALID -65536
+
 #define _XmMax(a,b) ((a)>(b)?(a):(b))
 #define Scale_ShadowThickness(x) MGR_ShadowThickness(x)
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -153,7 +159,7 @@ static XtResource resources[] =
     {
 	XmNshadowThickness, XmCShadowThickness, XmRHorizontalDimension,
 	sizeof(Dimension), XtOffsetOf(XmScaleRec, manager.shadow_thickness),
-	XmRCallProc, (XtPointer)_XmPrimitiveShadowThicknessDefault
+	XmRImmediate, (XtPointer)2
     },
     {
 	XmNvalue, XmCValue, XmRInt,
@@ -213,7 +219,7 @@ static XtResource resources[] =
     {
 	XmNhighlightThickness, XmCHighlightThickness, XmRHorizontalDimension,
 	sizeof(Dimension), Offset(highlight_thickness),
-	XmRCallProc, (XtPointer)_XmPrimitiveHighlightThicknessDefault
+	XmRImmediate, (XtPointer)2
     },
     {
 	XmNhighlightOnEnter, XmCHighlightOnEnter, XmRBoolean,
@@ -387,7 +393,7 @@ XmScaleClassRec xmScaleClassRec =
 WidgetClass xmScaleWidgetClass = (WidgetClass)&xmScaleClassRec;
 
 static void
-class_initialize()
+class_initialize(void)
 {
     _XmScaleCoreClassExtRec.record_type = XmQmotif;
 }
@@ -467,6 +473,20 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(new_w), XtHeight(new_w)));
     DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
 
+    if (Scale_ScaleWidth(new_w) == 0)
+    {
+    	/* rws 22 Dec 1999
+    	   scale/test18 scale/test16
+    	 */
+	if (Scale_Orientation(new_w) == XmHORIZONTAL)
+	{
+	    Scale_ScaleWidth(new_w) = XtWidth(request);
+    	}
+    	else
+	{
+	    Scale_ScaleWidth(new_w) = XtHeight(request);
+    	}
+    }
     if (Scale_Value(new_w) == INT_MAX)
     {
 	if (Scale_Minimum(new_w) >= 0)
@@ -487,6 +507,9 @@ initialize(Widget request, Widget new_w,
 
     Scale_LastValue(new_w) = Scale_Value(new_w);
 
+	/* T. Straumann: there is no old value to be erased in showValue() */
+	Scale_ShowValueX(new_w) = SHOW_VALUE_X_INVALID;
+
     if (Scale_FontList(new_w) == NULL)
     {
 	Scale_FontList(new_w) = _XmGetDefaultFontList(new_w, XmLABEL_FONTLIST);
@@ -529,6 +552,8 @@ initialize(Widget request, Widget new_w,
 	     Scale_HighlightThickness(new_w)); argc++;
     XtSetArg(argl[argc], XmNhighlightOnEnter,
 	     Scale_HighlightOnEnter(new_w)); argc++;
+	/* T. Straumann:  M*TIF enforces 0 borderWidth for scrollBar */
+	XtSetArg(argl[argc], XmNborderWidth, 0); argc++;
 
     sb = XmCreateScrollBar(new_w, "Scrollbar", argl, argc);
 
@@ -712,9 +737,28 @@ set_values(Widget old, Widget request, Widget new_w,
 	refresh = True;
     }
 
+	/* T. Straumann: if Scale_ShowValue changed we have to
+	 * do some more things
+	 */
+	if (Scale_ShowValue(new_w) != Scale_ShowValue(old))
+	{
+		refresh = True;
+		if ( ! Scale_ShowValue(new_w) )
+		{
+			/* undraw the old value */
+			if (XtIsRealized(old) && SHOW_VALUE_X_INVALID != Scale_ShowValueX(old))
+			{
+				XFillRectangle(XtDisplay(old), XtWindow(old), MGR_BackgroundGC(old),
+		       		Scale_ShowValueX(old), Scale_ShowValueY(old),
+		       		Scale_ShowValueWidth(old), Scale_ShowValueHeight(old));
+			}
+		}
+		/* Mark Scale_ShowValueX as invalid */
+		Scale_ShowValueX(new_w) = SHOW_VALUE_X_INVALID;
+	}
+
     if (Scale_DecimalPoints(new_w) != Scale_DecimalPoints(old) ||
-	Scale_FontList(new_w) != Scale_FontList(old) ||
-	Scale_ShowValue(new_w) != Scale_ShowValue(old))
+	Scale_FontList(new_w) != Scale_FontList(old))
     {
 	refresh = True;
 
@@ -1653,8 +1697,9 @@ _XmScaleLayout(Widget scale, XmKidGeometry boxes,
 	       Widget child, XtWidgetGeometry *childgeom,
 	       Dimension curw, Dimension curh)
 {
-    int i;
+    int i, delta;
 
+	int num_children = MGR_NumChildren(scale);
     /*
      * do the layout
      */
@@ -1702,14 +1747,55 @@ _XmScaleLayout(Widget scale, XmKidGeometry boxes,
 	}
 #endif
 
-	if (XtHeight(scale) > curh)
+	if ((delta = XtHeight(scale) - curh))
 	{
+	    if (delta < 0 && XtIsManaged(boxes[CHILD_LAB].kid))
+	    {
+		delta += boxes[CHILD_LAB].box.height +
+		    (MGR_ShadowThickness(scale) << 1);
+		if (delta > 0)
+		    delta = 0;
+	    }
 	    for (i = 0; i < 2 && boxes[i].kid; i++)
 	    {
-		boxes[i].box.y = XtHeight(scale) - (curh - boxes[i].box.y);
+		boxes[i].box.y += delta;
 	    }
 	}
 
+	/* T. Straumann: adjust width / tickmark positions if our geometry
+	 * is different from the one used in _XmScalePreferredSize().
+	 *
+	 * They do this only if Scale_ScaleWidth() == 0 or delta < 0
+	 * as can be confirmed by test14. Try also to resize 
+	 *
+	 *   test13 -xrm "*scaleHeight:200"
+	 *
+	 * using the WM
+	 */
+	delta = XtWidth(scale) - curw;
+	if ( delta < 0 || (delta > 0 && 0 == Scale_ScaleWidth(scale) ) )
+	{
+		/* different layout whether one or more tick marks are present */
+		int denom = (num_children == 3 ? 2 : num_children -3);
+		for (i = 0; boxes[i].kid; i++)
+		{
+			switch (i)
+			{
+				/* adjust the width of SB and LAB */
+				case CHILD_LAB:
+				case CHILD_SB:
+					boxes[i].box.width = XtWidth(scale);
+				break;
+
+				/* adjust the tick marks */
+
+				default:
+					boxes[i].box.x += delta * (i - 2) / denom;
+				break;
+			}
+		}
+	}
+
     }
     else if (Scale_Orientation(scale) == XmVERTICAL)
     {
@@ -1753,13 +1839,53 @@ _XmScaleLayout(Widget scale, XmKidGeometry boxes,
 	}
 #endif
 
-	if (XtWidth(scale) > curw)
+	if ((delta = XtWidth(scale) - curw))
 	{
+	    if (delta < 0 && XtIsManaged(boxes[CHILD_LAB].kid))
+	    {
+		delta += boxes[CHILD_LAB].box.width +
+		    (MGR_ShadowThickness(scale) << 1);
+		if (delta > 0)
+		    delta = 0;
+	    }
 	    for (i = 0; boxes[i].kid; i++)
 	    {
-		boxes[i].box.x = XtWidth(scale) - (curw - boxes[i].box.x);
+		boxes[i].box.x += delta;
 	    }
 	}
+
+	/* T. Straumann: adjust height / tickmark positions if our geometry
+	 * is different from the one used in _XmScalePreferredSize()
+	 *
+	 * They do this only if Scale_ScaleWidth() == 0 or delta < 0
+	 * as can be confirmed by test14. Try also to resize 
+	 *
+	 *   test13 -xrm "*scaleHeight:200"
+	 *
+	 * using the WM
+	 */
+	delta = XtHeight(scale) - curh;
+	if ( delta < 0 || (delta > 0 && 0 == Scale_ScaleHeight(scale) ) )
+	{
+		/* different layout whether one or more tick marks are present */
+		int denom = (num_children == 3 ? 2 : num_children -3);
+		for (i = 0; boxes[i].kid; i++)
+		{
+			switch (i)
+			{
+				/* adjust the height of SB and LAB */
+				case CHILD_LAB:
+				case CHILD_SB:
+					boxes[i].box.height = XtHeight(scale);
+				break;
+
+				/* adjust the tick marks */
+				default:
+					boxes[i].box.y += delta * (i - 2) / denom;
+				break;
+			}
+		}
+	}
     }
 }
 
@@ -1838,7 +1964,8 @@ showValue(Widget w, int scb_value, int scale_value)
 
 #if 1
 /* Fix from John Richardson <jrichard@zko.dec.com> 1/7/1996 */
-    if (XtIsRealized(w))
+	/* T. Straumann: only erase if Scale_ShowValueX() is valid */
+    if (XtIsRealized(w) && SHOW_VALUE_X_INVALID != Scale_ShowValueX(w))
     {
 	XFillRectangle(XtDisplay(w), XtWindow(w), MGR_BackgroundGC(w),
 		       Scale_ShowValueX(w), Scale_ShowValueY(w),
@@ -1856,16 +1983,14 @@ showValue(Widget w, int scb_value, int scale_value)
 
     if (Scale_Orientation(w) == XmHORIZONTAL)
     {
-	Scale_ShowValueX(w) = _XmScrollBarValueToPos(sb, scb_value) +
-	    (_XmScrollBarSliderPixSize(sb) / 2) -
+	Scale_ShowValueX(w) = SCB_SliderX(sb) + (SCB_SliderWidth(sb) >> 1) -
 	    (Scale_ShowValueWidth(w) / 2) + XtX(sb);
 
 	Scale_ShowValueY(w) = XtY(sb) - Scale_ShowValueHeight(w);
     }
     else if (Scale_Orientation(w) == XmVERTICAL)
     {
-	Scale_ShowValueY(w) = _XmScrollBarValueToPos(sb, scb_value) +
-	    (_XmScrollBarSliderPixSize(sb) / 2) -
+	Scale_ShowValueY(w) = SCB_SliderY(sb) + (SCB_SliderHeight(sb) >> 1) -
 	    (Scale_ShowValueHeight(w) / 2) + XtY(sb);
 
 	Scale_ShowValueX(w) = XtX(sb) - Scale_ShowValueWidth(w);
