@@ -1,6 +1,6 @@
 /**
  *
- * $Id: TearOff.c,v 1.27 1998/09/12 15:05:44 rwscott Exp $
+ * $Id: TearOff.c,v 1.42 1999/09/29 23:49:03 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: TearOff.c,v 1.27 1998/09/12 15:05:44 rwscott Exp $";
+static const char rcsid[] = "$Id: TearOff.c,v 1.42 1999/09/29 23:49:03 rwscott Exp $";
 
 #include <stdio.h>
 
@@ -48,17 +48,39 @@ XmExcludedParentPaneRec _XmExcludedParentPane =
     0
 };
 
-void
+static void
 _XmTearOffFocusChangeEventHandler(Widget reportingWidget, XtPointer data,
-			          XEvent *event, Boolean *cont)
+				  XEvent *event, Boolean *cont)
 {
+	DEBUGOUT(XdbDebug("TEAROFF", reportingWidget,
+		      "_XmTearOffFocusChangeEventHandler() - %s\n",
+		      XdbEventType2String(event->type)));
+	DEBUGOUT(XdbDebug(__FILE__, reportingWidget,
+		      "_XmTearOffFocusChangeEventHandler() - %s\n",
+		      XdbEventType2String(event->type)));
 	switch(event->type)
 	{
 	case FocusIn:
-	    _XmMenuArmItem(MGR_Children(reportingWidget)[1]);
+	    /*
+	    if (MGR_NumChildren(reportingWidget) > 1)
+	    {
+		_XmMenuArmItem(MGR_Children(reportingWidget)[1]);
+	    }
+	    */
 	    break;
 	case FocusOut:
+	    /*
 	    _XmMenuDisarmItem(MGR_ActiveChild(reportingWidget));
+	    */
+	    {
+	    int i;
+	    Widget w = reportingWidget;
+
+		for (i = 0; i < MGR_NumChildren(w); i++)
+		{
+		    _XmMenuDisarmItem(MGR_Children(w)[i]);
+		}
+	    }
 	    break;
 	default:
 		_XmWarning(reportingWidget, "%s(%d) _XmTearOffFocusChangeEventHandler unknown event type",__FILE__, __LINE__);
@@ -103,6 +125,7 @@ Widget menu;
 			GrabModeSync, GrabModeSync,
 			CurrentTime);
 
+#if 0
 	/* get the window that previously had the focus
 	 * before popping up the menu so we can revert
 	 * to it after we're done. */
@@ -111,6 +134,7 @@ Widget menu;
 	DEBUGOUT(XdbDebug(__FILE__, w, "RC FOCUS SET: %s\n", XtName(w)));
 
 	_XmMenuFocus(w, XmMENU_FOCUS_SET, CurrentTime);
+#endif
 
 	XAllowEvents(dpy, AsyncKeyboard, CurrentTime);
 
@@ -196,6 +220,7 @@ _XmDismissTearOff(Widget shell, XtPointer closure, XtPointer call_data)
 {
     Widget menu;
     ShellWidget sw = (ShellWidget)shell;
+    XEvent *event = call_data;
 
     DEBUGOUT(XdbDebug(__FILE__, shell, "_XmDismissTearOff()\n"));
 
@@ -209,18 +234,21 @@ _XmDismissTearOff(Widget shell, XtPointer closure, XtPointer call_data)
 	menu = sw->composite.children[1]; /* FIX ME - 0 is the vendor ext */
     }
 
-    _XmRestoreTearOffToMenuShell(menu, NULL);
+    _XmRestoreTearOffToMenuShell(menu, event);
 
     XtPopdown(shell);
     RC_SetTearOffActive(menu, 0);
+    /*
+    _XmCallRowColumnUnmapCallback(menu, event);
     if (RC_TearOffDeactivate_cb(menu))
     {
     XmAnyCallbackStruct cbs;
 
 	cbs.reason = XmCR_TEAR_OFF_DEACTIVATE;
-	cbs.event = ((XmAnyCallbackStruct *)call_data)->event;
+	cbs.event = event;
 	XtCallCallbackList(menu, RC_TearOffDeactivate_cb(menu), (XtPointer)&cbs);
     }
+    */
 }
 
 void
@@ -246,6 +274,27 @@ _XmTearOffInitiate(Widget w, XEvent *event)
 
     /* FIX ME: PopdownEveryone */
     {
+#if 1
+    Widget tmp;
+    Boolean poppedUp;
+
+    tmp = RC_CascadeBtn(menu);
+    RC_MenuButtonPopdown(w, event, &poppedUp);
+    RC_CascadeBtn(menu) = tmp;
+    /*
+    if (RC_CascadeBtn(menu))
+    {
+	tmp = XtParent(RC_CascadeBtn(menu));
+	Lab_MenuDisarm(RC_CascadeBtn(menu));
+	RC_PopupPosted(tmp) = NULL;
+	RCClass_MenuProcs(XtClass(tmp))(XmMENU_DISARM, tmp, NULL);
+    }
+    */
+    RC_TearOffLastSelectToplevel(menu) = RC_LastSelectToplevel(menu);
+    /*
+    RC_LastSelectToplevel(menu) = menu;
+    */
+#else
     Widget cb;
     Widget posted = NULL;
 
@@ -253,17 +302,18 @@ _XmTearOffInitiate(Widget w, XEvent *event)
 	{
 	    posted = RC_PopupPosted(XtParent(RC_CascadeBtn(menu)));
 	}
-    	cb = RC_CascadeBtn(menu);
-    	{
-    	Boolean poppedUp;
+	cb = RC_CascadeBtn(menu);
+	{
+	Boolean poppedUp;
 
 	    RC_MenuButtonPopdown(w, event, &poppedUp);
-    	}
-    	RC_CascadeBtn(menu) = cb;
+	}
+	RC_CascadeBtn(menu) = cb;
 	if (RC_CascadeBtn(menu))
 	{
 	    RC_PopupPosted(XtParent(RC_CascadeBtn(menu))) = posted;
 	}
+#endif
     }
 
     if (RC_ParentShell(menu))
@@ -276,6 +326,41 @@ _XmTearOffInitiate(Widget w, XEvent *event)
     }
 
     _XmRestoreTearOffToToplevelShell(menu, event);
+#if 1
+    RC_CascadeBtn(menu) = NULL;
+#if 0
+    if (MGR_NumChildren(menu) > 0)
+    {
+	_XmMenuFocus(menu, XmMENU_FOCUS_SAVE, CurrentTime);
+	if (XmGetTearOffControl(menu) != MGR_Children(menu)[0])
+	{
+	    RC_TearOffFocusItem(menu) = MGR_Children(menu)[0];
+	}
+	else if (MGR_NumChildren(menu) > 1)
+	{
+	  /* FIXME: this is just a hack to avoid core dumping when tearing
+	   off menus */
+	  Widget w=MGR_Children(menu)[1];
+	  if (w &&  /* XtSensitive(w)? FIXME */
+	      (
+	       (XmIsLabel(w) &&
+		!((XmLabelClassRec *)(XtClass(w)))->label_class.menuProcs) ||
+	       (XmIsLabelGadget(w) &&
+		!((XmLabelGadgetClassRec *)(XtClass(w)))->label_class.menuProcs)))
+	  {
+	    /* it's a label[g] w/o a menuProc */
+	    RC_TearOffFocusItem(menu) = NULL;
+	  } else {
+	    RC_TearOffFocusItem(menu) = w;
+	  }
+	}
+	if (RC_TearOffFocusItem(menu))
+	{
+		Lab_MenuArm(RC_TearOffFocusItem(menu));
+	}
+    }
+#endif
+#endif
     RC_SetTearOffActive(menu, 1);
     if (RC_TearOffActivate_cb(menu))
     {
@@ -285,6 +370,9 @@ _XmTearOffInitiate(Widget w, XEvent *event)
 	cbs.event = event;
 	XtCallCallbackList(menu, RC_TearOffActivate_cb(menu), (XtPointer)&cbs);
     }
+    _XmCallRowColumnMapCallback(menu, event);
+    _XmMenuFocus(menu, XmMENU_FOCUS_SAVE, CurrentTime);
+    RCClass_MenuTraverse(menu, XmTRAVERSE_HOME);
 }
 
 void
@@ -371,7 +459,7 @@ CreateTopLevelShell(Widget w)
 		XrmValue to;
 		from.addr = (char *)string;
 
-		if (XmCvtXmStringToText(XtDisplay(w), NULL, &zero, 
+		if (XmCvtXmStringToText(XtDisplay(w), NULL, &zero,
 					&from, &to, NULL))
 		{
 		    to_shell_title = to.addr;
@@ -379,8 +467,11 @@ CreateTopLevelShell(Widget w)
 		else
 		{
 		    /* converter failed - use simple fallback */
-		    XmStringGetLtoR(string, XmFONTLIST_DEFAULT_TAG, 
-				    &to_shell_title);
+		    if ( ! XmStringGetLtoR(string, XmFONTLIST_DEFAULT_TAG,
+				    &to_shell_title) )
+			{
+			to_shell_title = XtNewString("");
+			}
 		}
 		XmStringFree(string);
 	    }
@@ -388,14 +479,14 @@ CreateTopLevelShell(Widget w)
 	    {
 		to_shell_title = XtNewString("");
 	    }
-    	}
-    	else
-    	{
-    		to_shell_title = XtNewString("");
-    	}
+	}
+	else
+	{
+		to_shell_title = XtNewString("");
+	}
     }
 
-    /* find the toplevel shell in this heirarchy and make 
+    /* find the toplevel shell in this heirarchy and make
      * that the parent of the transient shell */
     for (shell = w;
 	     !XtIsSubclass(shell, vendorShellWidgetClass) && XtParent(shell);
@@ -409,10 +500,10 @@ CreateTopLevelShell(Widget w)
     XtSetArg(args[nargs], XmNtransientFor, shell); nargs++;
     XtSetArg(args[nargs], XmNtitle, to_shell_title); nargs++;
     XtSetArg(args[nargs], XmNmwmFunctions,
-    	 MWM_FUNC_ALL & ~(MWM_FUNC_RESIZE | MWM_FUNC_MINIMIZE |
+	 MWM_FUNC_ALL & ~(MWM_FUNC_RESIZE | MWM_FUNC_MINIMIZE |
 				  MWM_FUNC_MAXIMIZE)); nargs++;
     XtSetArg(args[nargs], XmNmwmDecorations,
-    	 MWM_DECOR_BORDER | MWM_DECOR_MENU | MWM_DECOR_TITLE); nargs++;
+	 MWM_DECOR_BORDER | MWM_DECOR_MENU | MWM_DECOR_TITLE); nargs++;
 
     parent_shell = XtCreatePopupShell(to_shell_name, transientShellWidgetClass,
 					  shell, args, nargs);
@@ -458,21 +549,15 @@ _XmRestoreTearOffToToplevelShell(Widget w, XEvent *event)
 	RC_ParentShell(w) = CreateTopLevelShell(w);
     }
 
-    XSync(XtDisplay(XtParent(w)), False);
+    XtUnmanageChild(w);
+
+    XSync(XtDisplay(w), False);
 
     XGrabServer(XtDisplay(w));
 
-    /* Tear off buttons should not appear in torn off shells */
-    if (RC_TearOffControl(w))
-    {
-	XtUnmanageChild(RC_TearOffControl(w));
-    }
+    XSync(XtDisplay(w), False);
 
-    XSync(XtDisplay(XtParent(w)), False);
-
-    XtUnmanageChild(w);
-
-    (*((CompositeWidgetClass)XtClass(XtParent(w)))
+    (*((CompositeWidgetClass)XtClass(menu_shell))
      ->composite_class.delete_child) (w);
 
     /* set our parent to the created transient shell */
@@ -499,25 +584,24 @@ _XmRestoreTearOffToToplevelShell(Widget w, XEvent *event)
 #if 0
     if (XtIsRealized(XtParent(w)) & XtIsRealized(w))
 #endif
-	XReparentWindow(XtDisplay(XtParent(w)),
+	XReparentWindow(XtDisplay(w),
 			XtWindow(w),
 			XtWindow(XtParent(w)),
 			0, 0);
 
-    XtManageChild(w);
+    XUngrabServer(XtDisplay(w));
 
+    XtManageChild(w);
 
     XtPopup(XtParent(w), XtGrabNone);
 
-    XUngrabServer(XtDisplay(w));
-
-    XFlush(XtDisplay(XtParent(w)));
+    XFlush(XtDisplay(w));
 
     RC_SetTornOff(w, 1);
     RC_SetFromInit(w, 0);
     RC_ParentShell(w) = menu_shell;	/* save off the menu shell */
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmRestoreTearOffToToplevelShell() new parent %s\n",
-    		XtName(XtParent(w))));
+		XtName(XtParent(w))));
     _XmAddTearOffEventHandlers(w);
 }
 
@@ -566,7 +650,6 @@ _XmRestoreTearOffToMenuShell(Widget w,
 		XtWidth(transient),
 		XtHeight(transient)
 		));
-	RCClass_MenuProcs(XtClass(w))(XmMENU_DISARM, w, NULL);
 	if (RC_TearOffDirty(w))
 	{
 		XtVaGetValues(transient,
@@ -585,6 +668,14 @@ _XmRestoreTearOffToMenuShell(Widget w,
 			    XtWidth(transient),
 			    XtHeight(transient),
 			    transient->core.depth);
+	{
+	int i;
+
+	    for (i = 0; i < MGR_NumChildren(w); i++)
+	    {
+	    	_XmMenuDisarmItem(MGR_Children(w)[i]);
+	    }
+	}
 	XmUpdateDisplay(transient);
 	XCopyArea(XtDisplay(transient), XtWindow(transient), MenuImage, gc,
 		    0, 0,
@@ -606,10 +697,16 @@ _XmRestoreTearOffToMenuShell(Widget w,
 		XmNbackgroundPixmap, MenuImage,
 		NULL);
 	RC_SetTearOffDirty(w, True);
-    }
+	_XmCallRowColumnUnmapCallback(w, event);
+	if (RC_TearOffDeactivate_cb(w))
+	{
+	XmAnyCallbackStruct cbs;
 
-    XSync(XtDisplay(XtParent(w)), False);
-    XGrabServer(XtDisplay(w));
+	    cbs.reason = XmCR_TEAR_OFF_DEACTIVATE;
+	    cbs.event = event;
+	    XtCallCallbackList(w, RC_TearOffDeactivate_cb(w), (XtPointer)&cbs);
+	}
+    }
 
     menu_shell = RC_ParentShell(w);
 
@@ -618,6 +715,9 @@ _XmRestoreTearOffToMenuShell(Widget w,
 	XtUnmanageChild(w);
     }
 
+    XSync(XtDisplay(w), False);
+    XGrabServer(XtDisplay(w));
+
     (*((CompositeWidgetClass)XtClass(transient))
      ->composite_class.delete_child) (w);
 
@@ -637,27 +737,17 @@ _XmRestoreTearOffToMenuShell(Widget w,
 #if 0
     if (XtIsRealized(XtParent(w)) & XtIsRealized(w))
 #endif
-	XReparentWindow(XtDisplay(XtParent(w)),
+	XReparentWindow(XtDisplay(w),
 			XtWindow(w),
 			XtWindow(XtParent(w)),
 			0, 0);
 
+    XUngrabServer(XtDisplay(w));
 
     XtManageChild(RC_TearOffControl(w));
 
-    XUngrabServer(XtDisplay(w));
-
     XFlush(XtDisplay(XtParent(w)));
 
     RC_SetTornOff(w, 0);
     RC_ParentShell(w) = transient;	/* save off the transient */
 }
-
-
-void
-_XmTearOffArm(Widget menu)
-{
-    DEBUGOUT(XdbDebug(__FILE__, menu, "_XmTearOffArm()\n"));
-
-    RCClass_MenuProcs(xmRowColumnWidgetClass)(XmMENU_ARM, menu, NULL);
-}
