@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/Text.c,v 1.72 1998/10/24 03:18:42 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/Text.c,v 1.120 2000/03/11 13:44:26 rwscott Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Text.c,v 1.72 1998/10/24 03:18:42 rwscott Exp $";
+static const char rcsid[] = "$Id: Text.c,v 1.120 2000/03/11 13:44:26 rwscott Exp $";
 
 #undef VERBOSE
 
@@ -31,6 +31,7 @@ static const char rcsid[] = "$Id: Text.c,v 1.72 1998/10/24 03:18:42 rwscott Exp
 #include <Xm/TextP.h>
 #include <Xm/TextOutP.h>
 #include <Xm/TextF.h>
+#include <Xm/VendorSEP.h>   /* for _XmGetAudibleWarning() only */
 
 #include <Xm/CutPaste.h>
 
@@ -58,7 +59,7 @@ static const char rcsid[] = "$Id: Text.c,v 1.72 1998/10/24 03:18:42 rwscott Exp
 
 /* Forward Declarations */
 
-static void ClassInitialize();
+static void ClassInitialize(void);
 
 static void ClassPartInitialize(WidgetClass class);
 
@@ -99,11 +100,7 @@ void _XmTextInvalidate(XmTextWidget w, XmTextPosition position,
 
 void _XmTextExportValue(Widget w, int offset, XtArgVal *value);
 
-void TextGetValues(Widget w, ArgList args, Cardinal *num_args);
-
-void _XmChangeVSB(XmTextWidget w, XmTextPosition pos);
-
-void _XmRedisplayHBar(XmTextWidget w, int offset);
+static void TextGetValues(Widget w, ArgList args, Cardinal *num_args);
 
 #if XmVERSION > 1
 #include <Xm/TraitP.h>
@@ -241,16 +238,6 @@ static XtResource resources[] =
 	sizeof(Boolean), Offset(verify_bell),
 	XtRImmediate, (XtPointer)((unsigned char)XmUNSPECIFIED)
     },
-    {
-	XmNshadowThickness, XmCShadowThickness, XmRHorizontalDimension,
-	sizeof(Dimension), XtOffsetOf(XmTextRec, primitive.shadow_thickness),
-	XmRCallProc, (XtPointer)_XmPrimitiveShadowThicknessDefault
-    },
-    {
-	XmNhighlightThickness, XmCHighlightThickness, XmRHorizontalDimension,
-	sizeof(Dimension), XtOffsetOf(XmTextRec, primitive.highlight_thickness),
-	XmRCallProc, (XtPointer)_XmPrimitiveHighlightThicknessDefault
-    },
     {
 	XmNnavigationType, XmCNavigationType, XmRNavigationType,
     sizeof(XmNavigationType), XtOffsetOf(XmTextRec, primitive.navigation_type),
@@ -489,7 +476,7 @@ WidgetClass xmTextInnerObjectClass = (WidgetClass)&xmTextInnerClassRec;
 extern XmFontList _XmFontListCreateDefault(Display *);
 
 static void
-ClassInitialize()
+ClassInitialize(void)
 {
     int len1 = strlen(_XmTextIn_XmTextEventBindings1);
     int len2 = strlen(_XmTextIn_XmTextEventBindings2);
@@ -535,6 +522,7 @@ Initialize(Widget request, Widget new,
 	   ArgList args, Cardinal *num_args)
 {
     XmTextWidget w = (XmTextWidget)new;
+    Boolean passedSourceAsArg;
 
     DEBUGOUT(XdbDebug(__FILE__, new,
 		      "initialize: %i args\n"
@@ -576,34 +564,113 @@ Initialize(Widget request, Widget new,
     Text_Line(w) = NULL;
     Text_LineMax(w) = Text_LineCount(w) = 0;
     Text_DisableDepth(w) = 0;
-    Text_FirstPos(w) = Text_LastPos(w) = Text_TopPos(w) = Text_CursorPos(w) = 0;
+    Text_FirstPos(w) = Text_LastPos(w) = Text_TopPos(w) = 0;
+    Text_DestPosition(w) = Text_CursorPos(w);
 
     (*Text_OutputCreate(w)) ((Widget)w, args, *num_args);
 
+    Text_CursorPositionX(w) = Text_OutputData(w)->leftmargin;
+    passedSourceAsArg = ( Text_Source(w) != 0 );
+
     (*Text_InputCreate(w)) ((Widget)w, args, *num_args);
 
     InitializeLineTable(w);
+    if ( Text_CursorPos(w) > GetSrc(w)->data->length )
+	Text_CursorPos(w) = GetSrc(w)->data->length;
+
+    Text_InRedisplay(w) = False;
+    Text_InRefigureLines(w) = False;
+    Text_InExpose(w) = False;
+    Text_InResize(w) = False;
+    Text_InSetValues(w) = False;
+    Text_NeedsRedisplay(w) = True;
+    Text_NeedsRefigureLines(w) = False;
+
+	/* The following seem to be the same as M*tif initializes */
+    Text_AddMode(w) = False; 
+    Text_PendingOff(w) = True; 
+    Text_ForgetPast(w) = PASTENDPOS;
+    Text_TotalLines(w) = 1;
+    Text_ForceDisplay(w) = -1;
+    Text_Traversed(w) = False;
+    Text_OnOrOff(w) = False;
+    Text_NewTop(w) = 0;
+    Text_LastTopChar(w) = 0;
+    Text_TopLine(w) = 0;
+    Text_VSBarScrolling(w) = 0;
+    Text_TableIndex(w) = 0;
 
     /* Highlight array */
     Text_Highlight(w).list = (_XmHighlightRec *)XtCalloc(32,
-						      sizeof(_XmHighlightRec));
+				      sizeof(_XmHighlightRec));
     Text_Highlight(w).maximum = 32;
     Text_Highlight(w).number = 0;
+
+
+    Text_Highlight(w).list[ 0 ].position = 0;
+    Text_Highlight(w).list[ 0 ].mode = XmHIGHLIGHT_NORMAL;
+
+    Text_OldHighlight(w).maximum = 1;
+    Text_OldHighlight(w).number = 0;
+    Text_OldHighlight(w).list = XtNew(_XmHighlightRec);
+
+    Text_Repaint(w).number = 1;
+    Text_Repaint(w).maximum = 1;
+    Text_Repaint(w).range = XtNew(RangeRec);
+    Text_Repaint(w).range[0].from=PASTENDPOS;
+    Text_Repaint(w).range[0].to = 0;
+    Text_PendingScroll(w) = 0;
+    Text_HighlightChanged(w) = False;
+	/* FIXME : when/how is char_size more than 1 */
+    Text_CharSize(w) = 1;
+
+    if ( Text_VerifyBell(w) == XmUNSPECIFIED )
+	{
+	Widget vendor = XtParent(w);
+	while ( vendor && !XmIsVendorShell( vendor ) )
+	    vendor = XtParent( vendor );
+	if ( vendor )
+	    Text_VerifyBell(w) = _XmGetAudibleWarning(vendor);
+	}
+    if ( strlen ( Text_Value(w) ) != 0 || passedSourceAsArg )
+	{
+	_XmTextUpdateLineTable( (Widget)w, 0, strlen( Text_Value(w) ),
+		       NULL, True /* it is unused */ );
+	Text_BottomPos(w) = Text_LineTable(w)[Text_LineCount(w)].start_pos;
+	Text_Repaint(w).range[0].to = Text_BottomPos(w);
+
+	/*CP: but the value attribute is a hangover of M*tif 1.1
+		and seems to be unused in 1.2. SO free it. */
+	if (strlen(Text_Value(w)) == 0)
+		XtFree ( (char*) Text_Value(w) );
+
+	Text_Value(w) = NULL;
+	}
 }
 
 static void
 Destroy(Widget w)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "Destroy\n"));
+
     /* rws 2 Oct 1998
-       This is odd, I thought Xt should be taking care of this
+     * This is odd, I thought Xt should be taking care of this
+     *
+     * Danny 30 Nov 1998 : only if the parent widget (XmText) is a composite.
+     *   Otherwise (like here) Xt doesn't have the information.
      */
-    XtDestroyWidget(((XmTextWidget)w)->text.inner_widget);
-    (*Text_Output(w)->destroy) (w);
-
     (*Text_Input(w)->destroy) (w);
+    (*Text_Output(w)->destroy) (w);
+    XtDestroyWidget(((XmTextWidget)w)->text.inner_widget);
 
+#if 1
+    /* rws 1 Apr 1999
+     */
+    XtFree((XtPointer)Text_LineTable(w));
+#endif
     XtFree((XtPointer)Text_Highlight(w).list);
+    XtFree((XtPointer)Text_OldHighlight(w).list);
+    XtFree((XtPointer)Text_Repaint(w).range);
 }
 
 static Boolean
@@ -628,6 +695,8 @@ SetValues(Widget old, Widget request, Widget new,
 		      XtWidth(new), XtHeight(new)));
     DEBUGOUT(XdbPrintArgList(__FILE__, new, args, *num_args, False));
 
+    Text_InSetValues(nw) = True;
+
     if (Text_Value(nw) != Text_Value(ow))
     {
 	XmTextSetString(new, Text_Value(new));
@@ -710,6 +779,10 @@ SetValues(Widget old, Widget request, Widget new,
      * unsigned int table_index;
      */
 
+    Text_InSetValues(nw) = False;
+    if ( refresh_needed ) /*CP: then we can "force" a complete RefigureLines call by: */
+	Text_ForgetPast(nw) = 0;
+
     return refresh_needed;
 }
 
@@ -718,8 +791,8 @@ Realize(Widget aw, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 {
     XmTextWidget w = (XmTextWidget)aw;
 
-    DEBUGOUT(XdbDebug(__FILE__, aw, "%s:Realize(%d) - %dx%d\n",
-    		__FILE__, __LINE__,
+    DEBUGOUT(XdbDebug(__FILE__, aw, "%s:Realize(%d) - valuemask 0x%X - %dx%d\n",
+    		__FILE__, __LINE__, *value_mask,
     		XtWidth(aw), XtHeight(aw)));
 
 #define superclass (&xmPrimitiveClassRec)
@@ -741,6 +814,15 @@ DoExpose(Widget aw, XEvent *event, Region region)
 {
     XmTextWidget w = (XmTextWidget)aw;
 
+    if (XtWindow(aw) == 0)
+	return;
+
+    if (event == NULL)
+    {
+        XClearArea(XtDisplay(aw),XtWindow(aw),0,0,0,0,True);
+	return;
+    }
+
     DEBUGOUT(XdbDebug(__FILE__, aw, "DoExpose\n"));
     if (Text_DisableDepth(w) != 0)
 	return;		/* Should we be doing this ? FIX ME */
@@ -755,7 +837,13 @@ DoExpose(Widget aw, XEvent *event, Region region)
 	RefigureLines(w);
     }
 
+    if ( Text_InExpose(w) )
+	return;
+
+    Text_InExpose(w)= True;
+
     (*Text_Output(w)->expose) (aw, event, region);
+    Text_InExpose(w)= False;
 }
 
 static void
@@ -776,8 +864,16 @@ Resize(Widget aw)
 	return;
     }
 
+    if ( Text_InResize( w ) )
+    	return;
+
+    Text_ForgetPast(w) = 0; /*CP: so that we recalculate all lines again */
+    Text_InResize( w ) = True;
+
     (*Text_Output(w)->resize) (aw, True);
 
+    Text_InResize( w ) = False;
+
     RefigureLines(w);
 }
 
@@ -858,6 +954,7 @@ static void
 Redisplay(XmTextWidget w)
 {
     DEBUGOUT(XdbDebug(__FILE__, (Widget)w, "Redisplay\n"));
+
     if (Text_DisableDepth(w) != 0)
 	return;
 
@@ -866,6 +963,11 @@ Redisplay(XmTextWidget w)
 	return;
     }
 
+    if ( Text_InRedisplay(w) )
+	return;
+
+    Text_InRedisplay(w) = True;
+
     Text_NeedsRedisplay(w) = False;
 
     if (Text_NeedsRefigureLines(w))
@@ -881,6 +983,7 @@ Redisplay(XmTextWidget w)
     }
 
     (*Text_Output(w)->expose) ((Widget)w, NULL, NULL);
+    Text_InRedisplay(w) = False;
 }
 
 static
@@ -946,8 +1049,10 @@ RefigureLines(XmTextWidget w)
  *
  * Not too sure about this. FIX ME.
  */
-    if (Text_DisableDepth(w) != 0)
+    if (Text_DisableDepth(w) != 0) {
+	Text_NeedsRefigureLines(w) = True;
 	return;
+    }
 /* End unsure stuff */
 
     Text_NeedsRefigureLines(w) = False;
@@ -1181,6 +1286,75 @@ _XmTextUpdateLineTable(Widget w,
 		      start, end, XdbBoolean2String(update),
 		      wrap ? "wrap" : ""));
 
+/* rws 10 Feb 1999
+   text/test14, grok phone database, note field.
+   I suspect that this should be done in TextOut.c not here, and somehow
+   this thing should be able to shrink to some sort of minimum size as the
+   text gets smaller.
+ */
+/*CP:19 May 1999: The difference with GROK and other scrollable
+   XmText widgets is that the scrollable window is created first and then
+   the XmText widget is created (i.e. it does not use the convenience 
+   function XmCreateScrolledText ). When this happens then XmText
+   does not have the XmScrolledWindow as it's parent and therefor
+   all lines in the XmText must be drawn hence the need to increase
+   the lines and column count of the output window.
+   This way of creating scrollable XmText windows is not efficient!
+   As Rick above states, I am also not sure that this should not be in
+   TextOut.c.
+*/
+    if(!XmIsScrolledWindow(XtParent(w)))
+        {
+        int max_lines = 1;
+        int max_cols = 0;
+        int col = 0;
+        int index;
+        String value = XmTextGetString(w);
+        OutputData od = Text_OutputData(w);
+        Arg args[2];
+        int n,len;
+
+        len=Text_LastPos(w);
+    	for (index = 0; index < len; index++)
+    	{
+    		switch (value[index])
+    		{
+    		case '\n':
+    			max_lines++;
+    			col = 0;
+    			break;
+    		default:
+    			col++;
+    			break;
+    		}
+		if (col > max_cols)
+		{
+		    max_cols = col;
+		}
+    	}
+	max_cols = max_cols < Out_ColumnsSet(od) ? Out_ColumnsSet(od) : max_cols;
+	max_lines = max_lines < Out_RowsSet(od) ? Out_RowsSet(od) : max_lines;
+
+	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
+		      "_XmTextUpdateLineTable - %d %d %d %d\n",
+		      max_lines, max_cols, Out_Rows(od), Out_Columns(od)));
+    	XtFree(value);
+    	n = 0;
+    	if (!wrap && !Out_ScrollHorizontal(od) && max_cols != Out_Columns(od))
+    	{
+	    XtSetArg(args[n], XmNcolumns, max_cols); n++;
+    	}
+    	if (!Out_ScrollVertical(od) && max_lines != Out_Rows(od))
+    	{
+	    XtSetArg(args[n], XmNrows, max_lines); n++;
+    	}
+    	if (n > 0)
+    	{
+	    XtSetValues(w, args, n);
+    	}
+    }
+
+
     if (wrap)
     {
 	/*
@@ -1274,7 +1448,7 @@ _XmTextUpdateLineTable(Widget w,
 
 	    /* Be careful not to do PASTENDPOS + 1 */
 	    start = (*Text_Source(w)->Scan) (Text_Source(w), start,
-					  XmSELECT_LINE, XmsdRight, -1, False);
+					  XmSELECT_LINE, XmsdRight, 1, False);
 
 	    if (start == PASTENDPOS)
 	    {
@@ -1366,10 +1540,22 @@ _XmTextMarkRedraw(XmTextWidget w, XmTextPosition left,
 LineNum
 _XmTextPosToLine(XmTextWidget w, XmTextPosition position)
 {
-    return _XmTextGetTableIndex(w, position) + 1;
+    if (position < 1)
+	return 0;
+    else if (position > Text_LastPos(w))
+	return NOLINE;
+    else
+	return _XmTextGetTableIndex(w, position) + 1;
 }
 
-void
+/*
+ * As far as I can see, this function needs to :
+ *	- insert text
+ *	- update the cursor position (done inside TextStrSo->Replace)
+ *	- call XmNvalueChangedCallback (done inside TextStrSo->Replace)
+ *	- call XmNmodifyVerifyCallback (and Wcs)	( ?? )
+ */
+static void
 _XmTextInsert(Widget w, XmTextPosition position, char *string, XEvent *evp)
 {
     XmTextStatus st;
@@ -1457,7 +1643,14 @@ _XmTextSetCursorPosition(Widget w, XmTextPosition position)
 
 	(*Text_Output(w)->DrawInsertionPoint) (tw, Text_CursorPos(w), off);
 
+#if 0
+	/* rws 10 Mar 2000
+	   This sucks, but text/test16 sub-test 5 proves this
+	 */
 	Text_CursorPos(w) = cbs.newInsert;
+#else
+	Text_CursorPos(w) = position;
+#endif
 
 #if VERBOSE
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
@@ -1522,12 +1715,15 @@ _XmTextDisableRedisplay(XmTextWidget w, Boolean losesbackingstore)
 void
 _XmTextEnableRedisplay(XmTextWidget w)
 {
-    Text_DisableDepth(w)--;
-    if (Text_DisableDepth(w) == 0)
+    if (Text_DisableDepth(w) > 0)
     {
-	if (Text_NeedsRedisplay(w))
+	Text_DisableDepth(w)--;
+	if (Text_DisableDepth(w) == 0)
 	{
-	    Redisplay(w);
+	    if (Text_NeedsRedisplay(w))
+	    {
+		Redisplay(w);
+	    }
 	}
     }
 }
@@ -1726,7 +1922,8 @@ XmTextCut(Widget w, Time time)
         return False;
 
     sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
-    if (!sel)
+
+    if (!sel || !Text_Editable(w))
 	return False;
 
     /* Now also remove the string from the buffer. */
@@ -1739,52 +1936,76 @@ Boolean
 XmTextFindString(Widget w, XmTextPosition start, char *string,
 		 XmTextDirection direction, XmTextPosition *position)
 {
-    char *buf;
-    char *str;
+    char *buf, *str;
+    int len, i;
 
-    if ((start < 0) || (direction != XmTEXT_FORWARD))
+    len = XmTextGetLastPosition(w);
+    if ((start < 0) || (start > len))
     {
 	return False;
     }
 
     buf = XmTextGetString(w);
 
-    if (start > strlen(buf))
-    {
-	str = (char *)NULL;
-    }
-    else
-    {
+    if (direction == XmTEXT_FORWARD) {
 	str = strstr(&buf[start], string);
+    } else if (direction == XmTEXT_BACKWARD) {
+	i = start;
+	str = NULL;
+	len = strlen(string);
+	while (i) {
+		if (! strncmp(&buf[i], string, len)) {
+			str = &buf[i];
+			break;
+		}
+		i--;
+	}
+    } else {
+	str = NULL;
     }
 
     if (str == (char *)NULL)
     {
 	XtFree(buf);
-
 	return False;
     }
 
     *str = 0;
-    *position = strlen(buf);
+    *position = (XmTextPosition)(str-buf);
     XtFree(buf);
     return True;
 }
 
+/*
+ * Returns an integer value that indicates the y position of the first baseline
+ * in the Text widget. The calculation takes into account the margin height,
+ * shadow thickness, highlight thickness, and font ascent of the first font in the
+ * fontlist. In this calculation the x position of the top of the widget is 0. 
+ */
 int
 XmTextGetBaseline(Widget w)
 {
+    /*
+     * TextField does this :
+     *	Prim_ShadowThickness(w) + Prim_HighlightThickness(w) +
+     *  TextF_MarginHeight(w) + TextF_FontAscent(w)
+     */
+
     if (XtIsSubclass(w, xmTextWidgetClass))
     {
+	return Prim_ShadowThickness(w) + Prim_HighlightThickness(w) +
+		Text_MarginHeight(w) + Out_FontAscent(Text_OutputData(w));
+#if 0
         _XmWarning(w, "XmTextGetBaseline: not implemented");
 	return 0;
+#endif
     }
     else if (XtIsSubclass(w, xmTextFieldWidgetClass))
     {
 	return XmTextFieldGetBaseline(w);
     }
 
-    _XmWarning(w, "XmTextGetCursorPosition: widget has invalid class");
+    _XmWarning(w, "XmTextGetBaseline: widget has invalid class");
 
     return 0;
 }
@@ -2019,7 +2240,7 @@ XmTextGetSubstring(Widget w, XmTextPosition start, int num_chars,
 	retval = XmCOPY_TRUNCATED;
     }
 
-    (*Text_Source(w)->ReadSource) (Text_Source(w), start, start+len-1, &block);
+    (*Text_Source(w)->ReadSource) (Text_Source(w), start, start+len, &block);
 
     memcpy(buffer, block.ptr, len);
     XtFree(block.ptr);
@@ -2110,7 +2331,8 @@ XmTextPaste(Widget w)
 	Window		win = XtWindow(w);
 	int		r;
 	char		*buf;
-	unsigned long	private_id, len, nbytes;
+	unsigned long	len, nbytes;
+	long		private_id;
 	XmTextPosition	pos;
 
 	if (win == None || !Text_Editable(w))
@@ -2195,10 +2417,12 @@ XmTextRemove(Widget w)
 	/* Copied part of the code of TextIn.c:DeleteForwardChar */
 	sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
 
-	if (sel)
+	if (!sel || !Text_Editable(w))
 	{
-	    _XmTextDelete((XmTextWidget)w, NULL, left, right);
-	}
+	    return False;
+        }
+
+	_XmTextDelete((XmTextWidget)w, NULL, left, right);
 
 	return True;
     }
@@ -2245,7 +2469,7 @@ XmTextReplace(Widget w, XmTextPosition from_pos, XmTextPosition to_pos, char *va
     endret = to_pos;
 
     st = (*Text_Source(w)->Replace) (tw, NULL, &startret, &endret,
-				     &block, False);
+				     &block, True);
     /* FIX ME ?? */
 
     RefigureLines(tw);
@@ -2353,16 +2577,115 @@ XmTextSetHighlight(Widget w, XmTextPosition left, XmTextPosition right, XmHighli
 
     if (XtIsSubclass(w, xmTextWidgetClass))
     {
+	int		i, j, k, l;
+	XmHighlightMode m;
+
+#if 0
+	fprintf(stderr,
+			  "XmTextSetHighlight(%d,%d,%s)\n",
+			  left, right, XdbHighlightMode2String(mode));
+#endif
 	DEBUGOUT(XdbDebug(__FILE__, w,
 			  "XmTextSetHighlight(%d,%d,%s)\n",
 			  left, right, XdbHighlightMode2String(mode)));
 
-	/* FIX ME - simplistic approach for now */
-	Text_Highlight(w).number = 2;
-	Text_Highlight(w).list[0].position = left;
-	Text_Highlight(w).list[0].mode = mode;
-	Text_Highlight(w).list[1].position = right;
-	Text_Highlight(w).list[1].mode = XmHIGHLIGHT_NORMAL;
+	/* Provide for initial entries if there's none */
+	if (Text_Highlight(w).number == 0) 
+	{
+	    Text_Highlight(w).list[0].position = 0;
+	    Text_Highlight(w).list[0].mode = XmHIGHLIGHT_NORMAL;
+	    Text_Highlight(w).list[1].position = INT_MAX;
+	    Text_Highlight(w).list[1].mode = XmHIGHLIGHT_NORMAL;
+	    Text_Highlight(w).number = 2;
+	}
+
+	/* Find the place from where to change the highlight */
+	for (i=0; i < Text_Highlight(w).number &&
+			Text_Highlight(w).list[i].position < left; i++)
+		;
+	if (i == 0)
+	{
+	    m = XmHIGHLIGHT_NORMAL;
+	}
+	else
+	{
+	    m = Text_Highlight(w).list[i-1].mode;
+	}
+	if (Text_Highlight(w).list[i].position == left) 
+	{
+	    Text_Highlight(w).list[i].mode = mode;
+	} 
+	else if (Text_Highlight(w).list[i].position < right) 
+	{
+	    Text_Highlight(w).list[i].mode = mode;
+	    Text_Highlight(w).list[i].position = left;
+	} 
+	else 
+	{
+	    /* Insert an entry for the left */
+	    if (Text_Highlight(w).list[i].mode != mode) 
+	    {
+		/* Need one more entry */
+		/* Make sure there's an empty space */
+		if (Text_Highlight(w).number == Text_Highlight(w).maximum) 
+		{
+		    Text_Highlight(w).maximum += 32;
+		    Text_Highlight(w).list = (_XmHighlightRec *)XtRealloc(
+				(XtPointer)Text_Highlight(w).list,
+				Text_Highlight(w).maximum * sizeof(_XmHighlightRec));
+		}
+
+		/* Go ahead. Do we need to insert ? */
+		for (j=Text_Highlight(w).number; j>i; j--) 
+		{
+		    Text_Highlight(w).list[j] = Text_Highlight(w).list[j-1];
+		}
+		Text_Highlight(w).number++;
+		Text_Highlight(w).list[i].mode = mode;
+		Text_Highlight(w).list[i].position = left;
+	    }
+	}
+
+	/* Remove entries */
+	/* FIX ME this can be more efficient */
+	for (i=0; i<Text_Highlight(w).number && Text_Highlight(w).list[i].position < left; i++) ;
+		/* i points to the entry just beyond "left" */
+	for (j=i; j<Text_Highlight(w).number && Text_Highlight(w).list[j].position <= right; j++) ;
+	if (i < j-1) 
+	{
+	    for (k=i+1, l=j; l<Text_Highlight(w).number; k++,l++) 
+	    {
+		Text_Highlight(w).list[k] = Text_Highlight(w).list[l];
+	    }
+	    Text_Highlight(w).number -= j-i-1;
+	}
+	/* Now this is hilarious - insert one again */
+	i++;
+	if (Text_Highlight(w).list[i].mode != mode) 
+	{
+	    if (Text_Highlight(w).number == Text_Highlight(w).maximum) 
+	    {
+		Text_Highlight(w).maximum += 32;
+		Text_Highlight(w).list = (_XmHighlightRec *)XtRealloc(
+			    (XtPointer)Text_Highlight(w).list,
+			    Text_Highlight(w).maximum * sizeof(_XmHighlightRec));
+	    }
+	    for (k=Text_Highlight(w).number; k>i; k--) 
+	    {
+		Text_Highlight(w).list[k] = Text_Highlight(w).list[k-1];
+	    }
+	    Text_Highlight(w).number++;
+	    Text_Highlight(w).list[i].mode = m;
+	    Text_Highlight(w).list[i].position = right;
+	}
+	/* End Highlighting */
+#if 0
+	for (i=0; i<Text_Highlight(w).number; i++)
+		fprintf(stderr,
+			  "\tXmTextSetHighlight -> [%d] %d %s\n",
+			  i, Text_Highlight(w).list[i].position,
+			  XdbHighlightMode2String(Text_Highlight(w).list[i].mode));
+#endif
 
 	if (XtIsRealized(w))
 	{
@@ -2389,6 +2712,7 @@ XmTextSetInsertionPosition(Widget w, XmTextPosition position)
     else if (XtIsSubclass(w, xmTextWidgetClass))
     {
 	_XmTextSetCursorPosition(w, position);
+	Text_DestPosition(w)= Text_CursorPos(w);
     }
     else
     {
@@ -2446,7 +2770,22 @@ XmTextSetString(Widget w, char *value)
     {
 	_XmStringSourceSetValue(tw, value);
 
+	Text_TopPos(w) = 0;
+#if 0
+	/* rws 10 Mar 2000
+	   This cannot happen here. If the modifyVerify callback came back
+	   with doit False, nothing changed!!!
+	 */
 	Text_LastPos(w) = (value ? strlen(value) : 0);
+#endif
+
+#if 0
+        // SG 01/11/1998 Reset the insertion point to the beginning
+        // however this will only effect this widget using the source,
+        // should the source take care of doing this for all its attached
+        // widgets.
+#endif
+	Text_CursorPos(w) = 0;
 
 	DEBUGOUT(XdbDebug(__FILE__, w, "XmTextSetString(%s), lastpos %d\n",
 		value, Text_LastPos(w)));
@@ -2489,19 +2828,13 @@ XmTextSetStringWcs(Widget w, wchar_t *wcstring)
 void
 XmTextDisableRedisplay(Widget w)
 {
-    Text_DisableDepth(w)--;
-#if 0
-    _XmWarning(w, "XmTextDisableRedisplay is not implemented");
-#endif
+    Text_DisableDepth(w)++;
 }
 
 void
 XmTextEnableRedisplay(Widget w)
 {
-    Text_DisableDepth(w)++;
-#if 0
-    _XmWarning(w, "XmTextEnableRedisplay is not implemented");
-#endif
+    _XmTextEnableRedisplay((XmTextWidget)w);
 }
 
 
@@ -2568,6 +2901,7 @@ XmTextScroll(Widget aw, int n)
     }
     DEBUGOUT(XdbDebug(__FILE__, aw, "XmTextScroll index=%d n=%d\n", index, n));
 
+    Text_TableIndex(w) = index;
     XmTextSetTopCharacter(aw, Text_LineTable(w)[index].start_pos);
 }
 
@@ -2590,7 +2924,16 @@ XmTextSetTopCharacter(Widget w, XmTextPosition top_character)
     }
     else
     {
-	Text_TopPos(w) = top_character;
+	/*
+	 * If w is in MULTI_LINE_EDIT then set Text_TopPos(w)
+	 * to the position of the first character of the line that
+	 * top_character points at.
+	 */
+
+	XmTextLineTable	lte;
+
+	lte = Text_LineTable(w) + _XmTextGetTableIndex((XmTextWidget)w,top_character);
+	Text_TopPos(w) = lte->start_pos;
 
 	RefigureLines((XmTextWidget)w);
 
@@ -2697,7 +3040,7 @@ _XmTextExportValue(Widget w, int offset, XtArgVal *value)
  *      the XmTextSource, has no resources that you can access this way. It's
  *      not an Xt Object.
  */
-void
+static void
 TextGetValues(Widget w, ArgList args, Cardinal *num_args)
 {
     /* Call the GetValues method in our subparts. */
@@ -2716,6 +3059,47 @@ TextGetValues(Widget w, ArgList args, Cardinal *num_args)
     DEBUGOUT(XdbPrintArgList(__FILE__, w, args, *num_args, True));
 }
 
+int
+_XmTextGetTotalLines(Widget w)
+{
+	return Text_TotalLines(w);
+}
+
+void
+_XmTextSetTopCharacter (  Widget w,
+                      XmTextPosition top_character)
+{
+    if (Text_EditMode(w) == XmSINGLE_LINE_EDIT)
+    {
+	/* Scroll horizontally until top_character is leftmost character */
+    }
+    else
+    {
+	int index = _XmTextGetTableIndex( (XmTextWidget)w, top_character);
+	if ( Text_TopPos(w) != top_character )
+	    {
+	    Text_NeedsRedisplay(w) = True;
+	    Text_NeedsRefigureLines(w) = True;
+	    }
+	Text_NewTop(w) = Text_LineTable(w)[index].start_pos;
+	Text_TableIndex(w)=index;
+	if ( Text_NeedsRefigureLines(w) ) 
+	    RefigureLines((XmTextWidget)w);
+
+	if (XtIsRealized(w))
+	{
+	    Redisplay((XmTextWidget)w);
+	}
+    }
+}
+
+
+LineNum _XmTextNumLines(
+                        XmTextWidget widget)
+{
+	return Text_LineCount(widget);
+}
+
 #if XmVERSION > 1
 /*
  * Not sure whether I need to do all this copying.
