@@ -1,7 +1,7 @@
 /*                   -*-Mode: c; c-file-style: "GNU"-*-              */
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.46 2000/01/09 22:21:21 hennessy Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.56 2000/04/08 14:54:51 hennessy Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.46 2000/01/09 22:21:21 hennessy Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextIn.c,v 1.56 2000/04/08 14:54:51 hennessy Exp $";
 
 #include <LTconfig.h>
 #include <Xm/XmP.h>
@@ -588,18 +588,51 @@ _XmTextDelete(XmTextWidget w, XEvent *ev, XmTextPosition start, XmTextPosition e
     XmTextVerifyCallbackStruct cbs;
     XmTextBlockRec blockrec;
 
+/*    printf("_XmTextDelete - %li %li\n", start, end); */
     if (end <= 0)
     {
 	end = 0;
     }
 
+    if ( start > end )
+    {
+	XmTextPosition tmp = start;
+	start = end;
+	end = tmp;
+    }
     blockrec.ptr = NULL;
     blockrec.length = 0;
     blockrec.format = XmFMT_8_BIT;
 
     cbs.reason = XmCR_MODIFYING_TEXT_VALUE;
     cbs.event = ev;
-    cbs.startPos = cbs.newInsert = start;
+/*CP: when doing a delete-previous-char these are not the same. Is this true for other actions ? */
+/*    cbs.startPos = cbs.newInsert = start; */
+    cbs.startPos = start;
+    if (Text_CursorPos(w) <= start) {
+	cbs.newInsert = Text_CursorPos(w);
+    } else if ( Text_CursorPos(w) + ( end - start ) > Text_LastPos(w) )
+    {
+	/*CP: WE do not want to new insertion position to be past the end. */
+	cbs.newInsert = Text_CursorPos(w) - ( end - start ) ;
+    }
+    else
+    {
+	/* rws 4 Mar 2000
+	   This is wrong in the case of delete-previous-character. Think about
+	   it, if you are at position 1 and do a delete-previous, you should
+	   end up at position 0. Is the other half of this if not always
+	   correct??
+	*/
+	/*
+	cbs.newInsert = Text_CursorPos(w);
+	*/
+	cbs.newInsert = Text_CursorPos(w) - ( end - start ) ;
+
+	/* Danny 17 mar 2000 don't go below position 0 */
+	if (cbs.newInsert < 0)
+		cbs.newInsert = 0;
+    }
     cbs.endPos = end;
     cbs.currInsert = Text_CursorPos(w);
     cbs.text = &blockrec;
@@ -622,7 +655,7 @@ _XmTextDelete(XmTextWidget w, XEvent *ev, XmTextPosition start, XmTextPosition e
 	start = cbs.startPos;
 	end = cbs.endPos;
 	status = (*Text_Source(w)->Replace) (w, ev, &start, &end,
-					     &blockrec, True);
+					     &blockrec, False);
 
 	if (status == EditDone)
 	{
@@ -653,7 +686,7 @@ DoInsert(XmTextWidget w, XEvent *ev, char *buf, int len)
     XmTextVerifyCallbackStruct cbs;
     XmTextBlockRec blockrec;
     Boolean sel;
-    XmTextPosition left, right;
+    XmTextPosition left, right, startInsert;
 
     if (len <= 0)
     {
@@ -694,12 +727,14 @@ DoInsert(XmTextWidget w, XEvent *ev, char *buf, int len)
         cbs.currInsert = Text_CursorPos(w);
         cbs.startPos = left;
         cbs.endPos = right;
-        cbs.newInsert = left + blockrec.length;
+        startInsert = left ;
+        cbs.newInsert = cbs.currInsert;
     }
     else
     {
         cbs.currInsert = cbs.startPos = cbs.endPos = Text_CursorPos(w);
-        cbs.newInsert = cbs.currInsert + blockrec.length;
+        startInsert = cbs.currInsert ;
+        cbs.newInsert = cbs.currInsert;
     }
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
@@ -730,13 +765,15 @@ DoInsert(XmTextWidget w, XEvent *ev, char *buf, int len)
 			  "DoInsert: start %08x:%08x end %08x:%08x\n",
 			  start, cbs.startPos, end, cbs.endPos));
 	status = (*Text_Source(w)->Replace) (w, ev, &start,
-					     &end, &blockrec, True);
+					     &end, &blockrec, False);
 
-	if (status == EditDone)
+	if (status == EditDone && blockrec.length > 0 )
 	{
-	    _XmTextSetCursorPosition(aw, cbs.newInsert);
+	    _XmTextSetCursorPosition(aw, startInsert + blockrec.length );
 	}
     }
+    Text_PendingOff(w) = False; 
+    (*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), on );
 
     XtFree(blockrec.ptr);
 }
@@ -868,8 +905,6 @@ MoveBackwardChar(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     if (Text_CursorPos(w) > 0)
     {
-	(*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					       Text_CursorPos(w), off);
 	_XmTextSetCursorPosition((Widget)w, Text_CursorPos(w) - 1);
     }
 }
@@ -890,9 +925,6 @@ MoveBackwardParagraph(Widget w, XEvent *ev,
 	pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
 				       XmSELECT_PARAGRAPH, XmsdLeft, 1, False);
 
-	(*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					       Text_CursorPos(w), off);
-
 	_XmTextSetCursorPosition((Widget)w, pos);
     }
 }
@@ -912,9 +944,6 @@ MoveBackwardWord(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
 				       XmSELECT_WORD, XmsdLeft, 1, False);
 
-	(*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					       Text_CursorPos(w), off);
-
 	_XmTextSetCursorPosition((Widget)w, pos);
     }
 }
@@ -933,8 +962,6 @@ MoveBeginningOfFile(Widget w, XEvent *ev, String *params,
 
     if (Text_CursorPos(w) > 0)
     {
-	(*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					       Text_CursorPos(w), off);
 	_XmTextSetCursorPosition(w, 0);
     }
 }
@@ -950,9 +977,6 @@ MoveToLineStart(Widget w, XEvent *ev, String *params,
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
 				   XmSELECT_LINE, XmsdLeft, 1, False);
 
-    (*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					   Text_CursorPos(w), off);
-
     _XmTextSetCursorPosition(w, pos);
 }
 
@@ -1042,6 +1066,7 @@ ProcessCopy(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     	nev = XtNew(XEvent);
     	*nev = *ev;
+    	Text_PendingOff(w)= True;
 	XtGetSelectionValue(w, XA_SECONDARY, XA_STRING, _XmTextGetSelection,
 			(XtPointer)nev, ev->xbutton.time);
     }
@@ -1051,6 +1076,14 @@ static void
 ProcessCancel(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "ProcessCancel\n"));
+    /* rws 15 Jan 2000
+       There is more to it than this. It needs to cancel some operations that
+       may be in progress, otherwise do this. Look at the man page.
+     */
+    if (XmIsManager(XtParent(w)))
+    {
+    	XtCallActionProc(XtParent(w), "ManagerParentCancel", ev, params, *num_params);
+    }
 }
 
 static void
@@ -1130,6 +1163,8 @@ DeleteBackwardChar(Widget w, XEvent *ev, String *params,
     if(sel && In_PendingDelete(Text_Input(w)->data) &&
        Text_CursorPos(w)>=left && Text_CursorPos(w)<=right)
     {
+	/*CP: we have to reset the selection */
+      (*Text_Source(w)->SetSelection) (Text_Source(w), right, right, ev->xkey.time);
       _XmTextDelete((XmTextWidget)w,ev,left,right);
     }
     else if (Text_CursorPos(w) > 0)
@@ -1266,7 +1301,7 @@ static void
 DeselectAll(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "DeselectAll\n"));
-    (*Text_Source(w)->SetSelection)(Text_Source(w),-1,-1,ev->xkey.time);
+    (*Text_Source(w)->SetSelection)(Text_Source(w),1,0,ev->xkey.time);
 }
 
 
@@ -1279,8 +1314,6 @@ MoveToLineEnd(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     pos = (*Text_Source(w)->Scan) (Text_Source(w), Text_CursorPos(w),
 				   XmSELECT_LINE, XmsdRight, 1, False);
-    (*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					   Text_CursorPos(w), off);
     _XmTextSetCursorPosition(w, pos);
 }
 
@@ -1293,9 +1326,6 @@ MoveEndOfFile(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     pos = Text_LastPos(w);
 
-    (*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					   Text_CursorPos(w), off);
-
     _XmTextSetCursorPosition(w, pos);
 }
 
@@ -1342,11 +1372,14 @@ ExtendEnd(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 static void
 ExtendSelection(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
-    XmTextPosition pos, left, right;
+    XmTextPosition pos, left, right, cursor_pos;
     XmTextScanType st;
     InputData i = Text_InputData(w);
     Boolean sel;
 
+    Text_InputData(w) -> select_pos_y = ev->xbutton.y ;
+    Text_InputData(w) -> select_pos_x = ev->xbutton.x ;
+
     pos = (*Text_Output(w)->XYToPos) ((XmTextWidget)w,
 				      ev->xbutton.x, ev->xbutton.y);
 
@@ -1357,6 +1390,10 @@ ExtendSelection(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 	DEBUGOUT(XdbDebug(__FILE__, w,
 		 "ExtendSelection (pos %d, origleft %d) existing sel. %d-%d\n",
 			  pos, In_HighlightPivot(i), left, right));
+	if ( pos >= left && pos <= right )
+	     /*CP: no need to do any more processing since it was only a mouse move
+		which does not change the selection positions */
+	    return ;
     }
     else
     {
@@ -1385,7 +1422,7 @@ ExtendSelection(Widget w, XEvent *ev, String *params, Cardinal *num_params)
                                             XmsdRight, 1, False);
             left = In_HighlightPivot(i);
         }
-        XmTextSetCursorPosition(w,right);
+        cursor_pos = right;
     }
     else
     {
@@ -1401,10 +1438,11 @@ ExtendSelection(Widget w, XEvent *ev, String *params, Cardinal *num_params)
             right = (*Text_Source(w)->Scan)(Text_Source(w),In_HighlightPivot(i),
                                             st, XmsdRight, 1, False);
         }
-        XmTextSetCursorPosition(w,left);
+        cursor_pos = left;
     }
 
     XmTextSetSelection(w,left,right,ev->xbutton.time);
+    XmTextSetCursorPosition(w, cursor_pos);
 
     i->extending = True;
 }
@@ -1421,8 +1459,6 @@ MoveForwardChar(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     if (Text_CursorPos(w) < Text_LastPos(w))
     {
-	(*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					       Text_CursorPos(w), off);
         _XmTextSetCursorPosition(w, Text_CursorPos(w) + 1);
     }
 }
@@ -1441,9 +1477,6 @@ MoveForwardParagraph(Widget w, XEvent *ev, String *params,
     pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
 				   XmSELECT_WHITESPACE, XmsdRight, 1, False);
 
-    (*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					   Text_CursorPos(w), off);
-
     _XmTextSetCursorPosition(w, pos);
 }
 
@@ -1462,9 +1495,6 @@ MoveForwardWord(Widget w, XEvent *ev, String *params,
     pos = (*Text_Source(w)->Scan) (Text_Source(w), pos,
 				   XmSELECT_WHITESPACE, XmsdRight, 1, False);
 
-    (*Text_Output(w)->DrawInsertionPoint) ((XmTextWidget)w,
-					   Text_CursorPos(w), off);
-
     _XmTextSetCursorPosition(w, pos);
 }
 
@@ -1513,10 +1543,16 @@ _KeySelection(Widget w, XEvent *ev, int flag, String *params, Cardinal *np)
 
     sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
 
+    if ( ! sel )
+    {
+	In_HighlightPivot( Text_InputData(w) ) = Text_CursorPos(w) ;
+    }
+
+    old = Text_CursorPos(w);
+
     switch (flag)
     {
     case 1:			/* left */
-	old = Text_CursorPos(w);
 	MoveBackwardChar(w, ev, params, np);
 
 	DEBUGOUT(XdbDebug(__FILE__, w,
@@ -1551,7 +1587,6 @@ _KeySelection(Widget w, XEvent *ev, int flag, String *params, Cardinal *np)
 	break;
 
     case 2:			/* right */
-	old = Text_CursorPos(w);
 	MoveForwardChar(w, ev, params, np);
 
 	DEBUGOUT(XdbDebug(__FILE__, w,
@@ -1585,7 +1620,6 @@ _KeySelection(Widget w, XEvent *ev, int flag, String *params, Cardinal *np)
 	break;
 
     case 3:			/* up */
-	old = Text_CursorPos(w);
 	MovePreviousLine(w, ev, params, np);
 
 	DEBUGOUT(XdbDebug(__FILE__, w,
@@ -1619,7 +1653,6 @@ _KeySelection(Widget w, XEvent *ev, int flag, String *params, Cardinal *np)
 	break;
 
     case 4:			/* down */
-	old = Text_CursorPos(w);
 	MoveNextLine(w, ev, params, np);
 
 	DEBUGOUT(XdbDebug(__FILE__, w,
@@ -1835,7 +1868,6 @@ MoveNextLine(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
 
     if (pos != Text_CursorPos(w))
     {
-	(*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), off);
 	_XmTextSetCursorPosition((Widget)w, pos);
     }
 }
@@ -1880,7 +1912,6 @@ MoveNextPage(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
 
     if (pos != Text_CursorPos(w))
     {
-	(*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), off);
 	_XmTextSetCursorPosition((Widget)w, pos);
     }
 }
@@ -2001,8 +2032,6 @@ MovePreviousLine(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
     }
     if (pos != Text_CursorPos(w))
     {
-	(*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), off);
-
 	_XmTextSetCursorPosition((Widget)w, pos);
     }
 }
@@ -2039,8 +2068,6 @@ MovePreviousPage(Widget aw, XEvent *ev, String *params, Cardinal *num_params)
     }
     if (pos != Text_CursorPos(w))
     {
-	(*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), off);
-
 	_XmTextSetCursorPosition((Widget)w, pos);
     }
 }
@@ -2278,8 +2305,10 @@ TraverseUp(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 static void
 TextFocusIn(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
+#if EXTRA_FOCUS
     XmTextVerifyCallbackStruct cbs;
     OutputData o = Text_OutputData(w);
+#endif
 
     DEBUGOUT(XdbDebug(__FILE__, w, "TextFocusIn"));
 
@@ -2287,7 +2316,7 @@ TextFocusIn(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     {
 	XmImSetFocusValues(w, NULL, 0);
     }
-
+#if EXTRA_FOCUS
     if (!Out_HasFocus(o))
     {
 	if (Text_FocusCallback(w))
@@ -2303,7 +2332,7 @@ TextFocusIn(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
 	Out_HasFocus(o) = True;
     }
-
+#endif
     /*
      * 980827 - pgw@hungry.com: Fix for highlight defect.
      */
@@ -2315,8 +2344,10 @@ TextFocusIn(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 static void
 TextFocusOut(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
+#if EXTRA_FOCUS
     XmTextVerifyCallbackStruct cbs;
     OutputData o = Text_OutputData(w);
+#endif
 
     DEBUGOUT(XdbDebug(__FILE__, w, "TextFocusOut\n"));
 
@@ -2324,7 +2355,7 @@ TextFocusOut(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     {
 	XmImUnsetFocus(w);
     }
-
+#if EXTRA_FOCUS
     if (Out_HasFocus(o))
     {
 	if (Text_LosingFocusCallback(w))
@@ -2340,6 +2371,7 @@ TextFocusOut(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
 	Out_HasFocus(o) = False;
     }
+#endif
     /*
      * 980827 - pgw@hungry.com: Fix for highlight defect.
      */
