@@ -1,7 +1,7 @@
 /*                   -*-Mode: c; c-file-style: "GNU"-*-              */
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.48 1998/10/24 03:18:44 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.81 2000/04/08 14:54:51 hennessy Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.48 1998/10/24 03:18:44 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/TextOut.c,v 1.81 2000/04/08 14:54:51 hennessy Exp $";
 
 #include <LTconfig.h>
 #include <Xm/XmP.h>
@@ -222,6 +222,7 @@ FontInit(XmTextWidget w)
 
 	Out_Font(o) = fs;
 	Out_FontHeight(o) = Out_FontAscent(o) + Out_FontDescent(o);
+	Out_CursorHeight(o) = Out_FontHeight(o);
 	Out_FontAverageWidth(o) =
 	    (fs->max_bounds.width + fs->min_bounds.width) / 2;
 
@@ -257,6 +258,8 @@ FontInit(XmTextWidget w)
 	Out_TabWidth(o) = 8 * Out_FontAverageWidth(o);
     }
 
+    Out_Font_Ascent(o) = Out_FontAscent(o);
+    Out_Font_Descent(o) = Out_FontDescent(o);
     DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 		      "font: ascent=%d descent=%d height=%d avewidth=%d\n",
 		      Out_FontAscent(o), Out_FontDescent(o),
@@ -350,8 +353,8 @@ GCPixmap(XmTextWidget tw)
     values.background = 0;
 
     display = XtDisplay(w);
-    width = 5;
-    height = Out_FontHeight(o);
+    width = Out_CursorWidth(o) ;
+    height = Out_CursorHeight(o) = Out_FontHeight(o);
     width_erase = 2 * Out_Font(o)->max_bounds.width;
 
     if (Out_CursorIBeam(o))
@@ -399,18 +402,18 @@ GCPixmap(XmTextWidget tw)
 
 	XSetForeground(display, gc, 1);
 
-	XDrawLine(display, Out_CursorIBeam(o), gc, 2, 0, 2, height - 2);
+	XDrawLine(display, Out_CursorIBeam(o), gc, 2, 1, 2, height - 2);
 	XDrawLine(display, Out_CursorIBeam(o), gc, 0, 0, 4, 0);
-	XDrawLine(display, Out_CursorIBeam(o), gc, 0, height - 2,
-		  4, height - 2);
+	XDrawLine(display, Out_CursorIBeam(o), gc, 0, height - 1,
+		  4, height - 1);
 
 	XSetLineAttributes(display, gc, 0, LineOnOffDash, CapRound, JoinRound);
 	XSetDashes(display, gc, 0, &dots[1], (int)dots[0]);
 
-	XDrawLine(display, Out_CursorStipple(o), gc, 2, 0, 2, height - 2);
+	XDrawLine(display, Out_CursorStipple(o), gc, 2, 1, 2, height - 2);
 	XDrawLine(display, Out_CursorStipple(o), gc, 1, 0, 3, 0);
-	XDrawLine(display, Out_CursorStipple(o), gc, 1, height - 2,
-		  3, height - 2);
+	XDrawLine(display, Out_CursorStipple(o), gc, 1, height - 1,
+		  3, height - 1);
 
 	XFreeGC(display, gc);
     }
@@ -424,16 +427,12 @@ GCClip(XmTextWidget w)
 
     clip.x = 0;
     clip.y = 0;
-    clip.width = XtWidth(w) -
-	2 * Prim_ShadowThickness(w) - 2 * Prim_HighlightThickness(w) -
-	2 * Text_MarginWidth(w);
-    clip.height = XtHeight(w) -
-	2 * Prim_ShadowThickness(w) - 2 * Prim_HighlightThickness(w) -
-	2 * Text_MarginHeight(w);
+    clip.width = XtWidth(w) - ( Out_RightMargin(o) + Out_LeftMargin(o) );
+    clip.height = XtHeight(w) - ( Out_TopMargin(o) + Out_BottomMargin(o) );
 
     /* These calls change the GCs */
     XSetClipRectangles(XtDisplay((Widget)w), Out_DrawGC(o),
-	   Out_XDraw(o), Out_YDraw(o) - Out_FontAscent(o), &clip, 1, Unsorted);
+	   Out_LeftMargin(o), Out_TopMargin(o), &clip, 1, Unsorted);
 
     GCPixmap(w);
 }
@@ -443,19 +442,17 @@ static void
 SizeRecalc(XmTextWidget w)
 {
     OutputData o = Text_OutputData(w);
+    int previousColumns = Out_Columns(o);
 
-    Out_XDraw(o) = Prim_ShadowThickness(w) + Prim_HighlightThickness(w) +
-	Text_MarginWidth(w);
+    Out_XDraw(o) = Out_LeftMargin(o);
 
     /* Y offset for a row R is calculated as Out_YDraw(o) - R*fontHeight,
      * and I add FontAscent here because y positions for fonts are
      * measured from the baseline of the font. */
-    Out_YDraw(o) = Prim_ShadowThickness(w) + Prim_HighlightThickness(w) +
-	Text_MarginHeight(w) + Out_FontAscent(o);
+    Out_YDraw(o) = Out_TopMargin(o) + Out_FontAscent(o);
 
-    Out_Columns(o) = (XtWidth(w)
-		      - 2 * Prim_ShadowThickness(w)
-		      - 2 * Text_MarginWidth(w)) / Out_FontAverageWidth(o);
+    Out_Columns(o) = (XtWidth(w) - ( Out_LeftMargin(o) + Out_RightMargin(o) ) )
+			/ Out_FontAverageWidth(o);
 
     if (Text_EditMode(w) == XmSINGLE_LINE_EDIT)
     {
@@ -464,9 +461,24 @@ SizeRecalc(XmTextWidget w)
     else
     {
 	Out_Rows(o) = (XtHeight(w)
-		       - 2 * Prim_ShadowThickness(w)
-		       - 2 * Text_MarginHeight(w)) / Out_FontHeight(o);
+		- ( Out_TopMargin(o) + Out_BottomMargin(o) ) )
+		/ Out_FontHeight(o);
+	if ( Out_Rows(o) < 1 )
+	{
+		Out_Rows(o) = 1;
+	}
     }
+    if ( XtHeight(w) < ( Out_TopMargin(o) + Out_BottomMargin(o) + Out_FontHeight(o) * Out_Rows(o) ) )
+    {
+	XtHeight(w) = Out_TopMargin(o) + Out_BottomMargin(o) + Out_FontHeight(o) * Out_Rows(o) ;
+    }
+
+    Out_NumberLines(o) = Out_Rows(o);
+    if ( Out_Columns(o) != previousColumns && Out_WordWrap(o) )
+	{
+        _XmTextUpdateLineTable((Widget)w, 0, 0, NULL, False);
+	}
+
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 		      "SizeRecalc wid %d ht %d -> rows %d cols %d\n",
@@ -497,15 +509,15 @@ SizeRecalc(XmTextWidget w)
 static void
 DrawAll(XmTextWidget w)
 {
-    int i, hi, mh, resty;
-    int highlight = XmHIGHLIGHT_NORMAL;		/* initial value */
-    XmTextPosition np, first;
+    int			i, resty, mh, hi;
+    int			highlight = XmHIGHLIGHT_NORMAL;		/* initial value */
+    XmTextPosition	np, first;
 
     OutputData o = Text_OutputData(w);
 
     /*
      * Without this, the area under the cursor is saved before it
-     * is actuall drawn; I can't figure out how that can happen but
+     * is actually drawn; I can't figure out how that can happen but
      * it does. Registering when you redraw that the saved image is
      * invalid helps because we no longer restore invalid images...
      */
@@ -547,7 +559,7 @@ DrawAll(XmTextWidget w)
        /* Cleaning part of widget */
     XClearArea(XtDisplay(w), XtWindow(w),
               Prim_HighlightThickness(w) + Prim_ShadowThickness(w),
-               Out_YDraw(o) + i * Out_FontHeight(o) - Out_FontAscent(o) + 1,
+               Out_TopMargin(o) + i * Out_FontHeight(o),
               XtWidth(w) -
               2 * (Prim_HighlightThickness(w) + Prim_ShadowThickness(w)),               
                Out_FontHeight(o),
@@ -585,26 +597,40 @@ DrawAll(XmTextWidget w)
 
 	    while (np != -1 && np <= next->start - 1)
 	    {
-		Draw(w, i, first, np, highlight);
+	    	XmTextPosition drawTo = np;
+#if 0
+		if ( np < next->start - 1 )
+		    {
+		    	/*CP:16 May 1999: basically do not include the return char 
+		    	** if the next highlight position is not past the end of the line. */
+		    drawTo ++;
+		    }
+#endif
+		(*Text_Output(w)->Draw)(w, i, first, drawTo, highlight);
 		first = np;
 
 		/* Find the next change */
 		highlight = Text_Highlight(w).list[hi].mode;
 		hi++;
-		if (hi < mh)
-		{
-		    np = Text_Highlight(w).list[hi].position;
-		}
-		else
-		{
+		if (hi == mh - 1) {
 		    np = -1;
+		} else if (hi < mh) {
+		    np = Text_Highlight(w).list[hi].position;
 		}
 	    }
 
 	    /* Draw the last part */
 	    if (np != next->start - 1)
 	    {
-		Draw(w, i, first, next->start - 1, highlight);
+		XmTextPosition endPos = next->start - 1;
+		if ( np > endPos )
+		     {
+			/*CP:16 May 1999: basically include the return char 
+			** if the next highlight position is past the end of the line. */
+		     endPos ++;
+		     }
+		(*Text_Output(w)->Draw)(w, i, first, endPos,
+			highlight);
 	    }
 	}
 	else
@@ -612,13 +638,22 @@ DrawAll(XmTextWidget w)
 	    /* Sanity checks */
 	    if (next->start > 0 && line->start < next->start - 1)
 	    {
-		Draw(w, i, line->start, next->start - 1, highlight);
+			/*CP:16 May 1999: this point is reached only for lines which
+			** are inside a selection area, so also include the newline. */
+		(*Text_Output(w)->Draw)(w, i, line->start, next->start,
+			highlight);
+	    }
+	    else if ( (  line->start == next->start - 1 ) &&
+	    		highlight != XmHIGHLIGHT_NORMAL )
+	    {
+		(*Text_Output(w)->Draw)(w, i, line->start, next->start,
+			highlight);
 	    }
 	}
     }
 
        /* Cleaning rest of widget */
-    resty = Out_YDraw(o) + i * Out_FontHeight(o) - Out_FontAscent(o) + 1;
+    resty = Out_TopMargin(o) + i * Out_FontHeight(o) ;
     XClearArea(XtDisplay(w), XtWindow(w),
               Prim_HighlightThickness(w) + Prim_ShadowThickness(w),
                resty,
@@ -642,7 +677,7 @@ DrawAll(XmTextWidget w)
  * FIX ME: Must start from col 0 (or a tab boundary) for tab position to be
  *        accurate.
  */
-int
+static int
 _XmTextNextX(XmTextWidget w, int x, char *ptr, int len)
 {
     OutputData o = Text_OutputData(w);
@@ -657,6 +692,10 @@ _XmTextNextX(XmTextWidget w, int x, char *ptr, int len)
 	{
 	    x = Out_NextTabStop(o, x);
 	}
+	else if (c == '\0')
+	{
+	    x = 0;
+	}
 	else
 	{
 	    if (c < fs->min_char_or_byte2 || c > fs->max_char_or_byte2)
@@ -760,7 +799,7 @@ XToPos(XmTextWidget w, LineNum row, int x)
     }
 
     /* x is now the width from the start of the line to the pixel pos */
-    x -= Out_XDraw(o) - Out_XOffset(o);
+    x -= Out_LeftMargin(o) - Out_XOffset(o);
 
     (*Text_Source(w)->ReadSource) (Text_Source(w), start, end, &block);
 
@@ -777,9 +816,9 @@ XToPos(XmTextWidget w, LineNum row, int x)
     /* must be in between somewhere... */
     else
     {
-	int tot, count, i;
+	int prevTot, tot, count, i;
 
-	tot = 0;
+	prevTot = tot = 0;
 	count = end - start;
 	pos = -1;
 	for (i = 0; i < count; i++)
@@ -788,8 +827,16 @@ XToPos(XmTextWidget w, LineNum row, int x)
 	    if (x < tot)
 	    {
 		pos = start + i;
+
+		/*CP :Saturday 08 May 1999:
+		the following makes sure that if the cursor is past the
+		half-way point between the begining of one char and
+		the next then the position returned is for the "next" char. */
+		if ( ( tot - ( ( tot - prevTot ) / 2 ) ) < x )
+			pos ++;
 		break;
 	    }
+	    prevTot = tot;
 	}
 
 	if (pos < 0)
@@ -816,13 +863,12 @@ YToLineNum(XmTextWidget w, Position y)
     OutputData o = Text_OutputData(w);
     LineNum line, l;
 
-    l = line = (LineNum)((y - Prim_ShadowThickness(w)
-		- Prim_HighlightThickness(w) - Text_MarginHeight(w))
+    l = line = (LineNum)((y - Out_TopMargin(o))
 		/ Out_FontHeight(o));
 
     if (line >= Text_LineCount(w))
     {
-	line = Text_LineCount(w) - 1;
+	line = Text_LineCount(w);
     }
 
     DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
@@ -845,7 +891,7 @@ DrawText(XmTextWidget w, int x, int y, char *ptr, int len,
 		      x, y, (ptr == 0) ? "(NULL)" : ptr, len, highlight));
 #endif
 
-    xoff = Out_XDraw(o) - Out_XOffset(o);
+    xoff = Out_LeftMargin(o) - Out_XOffset(o);
 
     /* Only change the GC if we need to */
     if (highlight == XmHIGHLIGHT_SELECTED && !Out_DrawGCInverted(o))
@@ -861,7 +907,7 @@ DrawText(XmTextWidget w, int x, int y, char *ptr, int len,
 	Out_DrawGCInverted(o) = False;
     }
 
-    while (len > 0)
+    while (len > 0 && *ptr)
     {
 	char *start;
 	Dimension piecewidth;
@@ -869,7 +915,7 @@ DrawText(XmTextWidget w, int x, int y, char *ptr, int len,
 
 	start = ptr;
 	count = 0;
-	while (len > 0 && *ptr && *ptr != '\t')
+	while (len > 0 && *ptr && *ptr != '\t' && *ptr != '\n' )
 	{
 	    ptr++, count++, len--;
 	}
@@ -901,7 +947,34 @@ DrawText(XmTextWidget w, int x, int y, char *ptr, int len,
 
 	if (len > 0 && *ptr == '\t')
 	{
+	    int left = x;
 	    x = Out_NextTabStop(o, x);
+	    if ( highlight == XmHIGHLIGHT_SELECTED )
+		{
+		XSetForeground(XtDisplay(w), Out_DrawGC(o), Prim_Foreground(w));
+		XFillRectangle(XtDisplay(w), XtWindow(w), Out_DrawGC(o),
+			left + xoff, y - Out_FontAscent(o),
+			x - left, Out_FontHeight(o));
+		XSetForeground(XtDisplay(w), Out_DrawGC(o), XtBackground(w));
+		}
+
+	    ptr++, len--;
+	}
+	if (len > 0 && *ptr == '\n')
+	{
+	    int left = x;
+
+	    x = Out_NextTabStop(o, x);
+
+	    if (highlight == XmHIGHLIGHT_SELECTED)
+	    {
+		XSetForeground(XtDisplay(w), Out_DrawGC(o), Prim_Foreground(w));
+		XFillRectangle(XtDisplay(w), XtWindow(w), Out_DrawGC(o),
+			left + xoff, y - Out_FontAscent(o),
+			Out_ScrollWidth(o) - ( left + xoff ),
+			Out_FontHeight(o));
+		XSetForeground(XtDisplay(w), Out_DrawGC(o), XtBackground(w));
+	    }
 
 	    ptr++, len--;
 	}
@@ -945,6 +1018,7 @@ _XmRedisplayHBar(XmTextWidget w, int offset)
     int newValue;
     int i;
 
+    Out_ScrollWidth(o) = XtWidth(w) - ( Out_RightMargin(o) + Out_LeftMargin(o) );
     if (Out_Hbar(o) == NULL)
     {
 	return;
@@ -966,8 +1040,7 @@ _XmRedisplayHBar(XmTextWidget w, int offset)
 	}
     }
 
-    displayWidth = XtWidth(w) - 2 * (Prim_HighlightThickness(w) +
-				Prim_ShadowThickness(w) + Text_MarginWidth(w));
+    displayWidth = XtWidth(w) - ( Out_RightMargin(o) + Out_LeftMargin(o) );
 
     if (maxWidth < displayWidth)
     {
@@ -1046,8 +1119,16 @@ _XmChangeVSB(XmTextWidget w, XmTextPosition pos)
 
     /* Setting XmNminimum == XmNmaximum is not a good idea */
 
+#if 0
+    /* rws 16 Jan 1999
+       What does this do????  The scrollbar value should be the index of the
+       first visible line, right??
+     */
     newValue = pos_index > (newMax - Out_Rows(o))
 	? (newMax - Out_Rows(o)) : pos_index;
+#else
+    newValue = top_index;
+#endif
 
     if (Out_Vbar(o) && (oldMax != newMax || oldMin != 0 ||
 			oldSize != Out_Rows(o) || oldValue != newValue))
@@ -1133,12 +1214,12 @@ CursorSaveUnderIBeam(XmTextWidget w, Position x, Position y)
 
     /* save the area under the cursor */
     XFillRectangle(XtDisplay((Widget)w), Out_CursorSave(o), Out_CopyGC(o),
-		   0, 0, 5, Out_FontHeight(o));
+		   0, 0, Out_CursorWidth(o), Out_CursorHeight(o) );
 
     XCopyArea(XtDisplay((Widget)w), XtWindow((Widget)w), Out_CursorSave(o),
 	      Out_CursorGC(o),
 	      x - 2, y,
-	      5, Out_FontHeight(o), 0, 0);
+	      Out_CursorWidth(o), Out_CursorHeight(o), 0, 0);
 
     Out_CursorSaveValid(o) = True;
 }
@@ -1187,7 +1268,7 @@ CursorDrawIBeam(XmTextWidget w, int x, int y)
 		 x - 2, y);
 
     XFillRectangle(XtDisplay((Widget)w), XtWindow((Widget)w), Out_CursorGC(o),
-		   x - 2, y, 5, Out_FontHeight(o));
+		   x - 2, y, Out_CursorWidth(o), Out_CursorHeight(o));
 }
 
 static void
@@ -1197,7 +1278,8 @@ CursorDraw(XmTextWidget w)
 
     if (Out_CursorPositionVisible(o))
     {
-	CursorDrawIBeam(w, Out_CursorX(o), Out_CursorY(o));
+	CursorDrawIBeam(w, Out_CursorX(o), 
+		( Out_CursorY(o) - Out_CursorHeight(o) ) + Out_FontDescent(o) );
 	Out_BlinkOn(o) = True;
     }
 }
@@ -1209,7 +1291,8 @@ CursorErase(XmTextWidget w)
 
     if (Out_CursorSaveValid(o))
     {
-	CursorRestoreUnderIBeam(w, Out_CursorX(o), Out_CursorY(o));
+	CursorRestoreUnderIBeam(w, Out_CursorX(o), 
+		( Out_CursorY(o) - Out_CursorHeight(o) ) + Out_FontDescent(o) );
     }
 
     Out_BlinkOn(o) = False;
@@ -1222,10 +1305,11 @@ HandleTimer(XtPointer client_data, XtIntervalId *timer)
 {
     XmTextWidget w = (XmTextWidget)client_data;
     OutputData o = Text_OutputData(w);
-    Out_BlinkState(o) = Out_BlinkState(o) == on ? off : on;
-    (*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w),
-					   Out_BlinkState(o));
 
+    OnOrOff nextBlinkState = Out_BlinkState(o) == on ? off : on;
+    (*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w),
+					   nextBlinkState );
+    Out_BlinkState(o) = nextBlinkState;
     Out_TimerId(o) = XtAppAddTimeOut(XtWidgetToApplicationContext((Widget)w),
 				     Out_BlinkRate(o), HandleTimer,
 				     (XtPointer)w);
@@ -1261,13 +1345,16 @@ HandleHBar(Widget sw, XtPointer client_data, XtPointer call_data)
 	DEBUGOUT(XdbDebug(__FILE__, (Widget)w,
 		   "HandleHbar reason=XmCR_INCREMENT value=%d\n", cbs->value));
 
-	displayWidth = XtWidth(w) - 2 * (Prim_HighlightThickness(w) +
-				Prim_ShadowThickness(w) + Text_MarginWidth(w));
+	displayWidth = XtWidth(w) - ( Out_RightMargin(o) + Out_LeftMargin(o) );
 
-	if (Out_XOffset(o) < Out_ScrollWidth(o) - displayWidth)
+	if (Out_XOffset(o) < Out_ScrollWidth(o) - displayWidth - Out_FontAverageWidth(o))
 	{
 	    newOffset = Out_XOffset(o) + Out_FontAverageWidth(o);
 	}
+	else
+	{
+	    newOffset = Out_ScrollWidth(o) - displayWidth;
+	}
 	break;
 
     case XmCR_PAGE_DECREMENT:
@@ -1275,9 +1362,8 @@ HandleHBar(Widget sw, XtPointer client_data, XtPointer call_data)
 			  "HandleHbar reason=XmCR_PAGE_DECREMENT value=%d\n",
 			  cbs->value));
 
-	/* FIX ME How mutch does we have to move on page_decrement ? */
-	newOffset = Out_XOffset(o) > Out_FontAverageWidth(o) ?
-	    Out_XOffset(o) - Out_FontAverageWidth(o) : 0;
+	newOffset = Out_XOffset(o) > Out_Columns(o) * Out_FontAverageWidth(o) ?
+	    Out_XOffset(o) - Out_Columns(o) * Out_FontAverageWidth(o) : 0;
 	break;
 
     case XmCR_PAGE_INCREMENT:
@@ -1285,14 +1371,17 @@ HandleHBar(Widget sw, XtPointer client_data, XtPointer call_data)
 			  "HandleHbar reason=XmCR_PAGE_INCREMENT value=%d\n",
 			  cbs->value));
 
-	/* FIX ME How mutch does we have to move on page_increment ? */
 	displayWidth = XtWidth(w) - 2 * (Prim_HighlightThickness(w) +
 				Prim_ShadowThickness(w) + Text_MarginWidth(w));
 
 	if (Out_XOffset(o) < Out_ScrollWidth(o) - displayWidth -
-	    Out_FontAverageWidth(o))
+	    Out_Columns(o) * Out_FontAverageWidth(o))
 	{
-	    newOffset = Out_XOffset(o) + Out_FontAverageWidth(o);
+	    newOffset = Out_XOffset(o) + Out_Columns(o) * Out_FontAverageWidth(o);
+	}
+	else
+	{
+	    newOffset = Out_ScrollWidth(o) - displayWidth;
 	}
 
 	break;
@@ -1333,7 +1422,7 @@ HandleHBar(Widget sw, XtPointer client_data, XtPointer call_data)
     XtVaSetValues(Out_Hbar(o), XmNvalue, Out_XOffset(o), NULL);
 }
 
-void
+static void
 HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 {
     XmTextWidget w = (XmTextWidget)client_data;
@@ -1357,6 +1446,7 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 
 	index = cbs->value;
 	Text_TopPos(w) = Text_LineTable(w)[index].start_pos;
+	Text_NeedsRefigureLines(w) = True;
 	break;
 
     case XmCR_INCREMENT:
@@ -1385,6 +1475,7 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 
 	index = Text_TotalLines(w) - 1 - Out_Rows(o);
 	Text_TopPos(w) = Text_LineTable(w)[index].start_pos;
+	Text_NeedsRefigureLines(w) = True;
 	break;
 
     case XmCR_TO_TOP:
@@ -1392,6 +1483,7 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 			  "HandleVbar reason=XmCR_TO_TOP\n"));
 
 	Text_TopPos(w) = Text_FirstPos(w);
+	Text_NeedsRefigureLines(w) = True;
 	break;
 
     case XmCR_VALUE_CHANGED:
@@ -1400,6 +1492,7 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 
 	index = cbs->value;
 	Text_TopPos(w) = Text_LineTable(w)[index].start_pos;
+	Text_NeedsRefigureLines(w) = True;
 	break;
 
     default:
@@ -1410,51 +1503,74 @@ HandleVBar(Widget sw, XtPointer client_data, XtPointer call_data)
 
     _XmRedisplayHBar(w, Out_XOffset(o));
 
+/*CP : I do not think this is needed here
     Text_NeedsRefigureLines(w) = True;
-
+*/
     (*w->core.widget_class->core_class.expose) ((Widget)w, NULL, (Region)NULL);
 
 }
 
 
 static void
-HandleFocusEvents(Widget aw, XtPointer client_data, XtPointer call_data)
+HandleFocusEvents(Widget aw, XtPointer client_data, XEvent* event, Boolean* continueToDispatch )
 {
-    XmAnyCallbackStruct *call_struct = (XmAnyCallbackStruct *)call_data;
     XmTextWidget w = (XmTextWidget)aw;
     OutputData o = Text_OutputData(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)w, "HandleFocusEvent reason=%s\n",
-		      call_struct->reason == XmCR_FOCUS ? "XmCR_FOCUS" :
-		      "XmCR_LOSING_FOCUS"));
+    DEBUGOUT(XdbDebug(__FILE__, (Widget)w, "HandleFocusEvent event=%s\n",
+		      event->type == FocusIn ? "FocusIn" : "FocusOut"));
 
-    if (call_struct->reason == XmCR_FOCUS)
+    if ( event->type == FocusIn && event->xany.send_event && !Out_HasFocus(o) )
     {
+	if (Text_FocusCallback(w) )
+	{
+	    XmAnyCallbackStruct cbs;
+	    cbs.reason = XmCR_FOCUS;
+	    cbs.event = event;
+
+	    XtCallCallbackList((Widget)w, Text_FocusCallback(w), &cbs);
+	}
+
 	Out_HasFocus(o) = True;
-	Out_BlinkState(o) = on;
 
 	XSetStipple(XtDisplay(aw), Out_CursorGC(o), Out_CursorIBeam(o));
 
 	if (Out_BlinkRate(o) > 0)
 	{
+/*CP:Handle timer will do all this
 	    Out_TimerId(o) = XtAppAddTimeOut(XtWidgetToApplicationContext(aw),
 					     Out_BlinkRate(o), HandleTimer,
 					     (XtPointer)w);
+*/
+	    HandleTimer(w, 0);
 	}
+	(*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), on);
     }
-    else
+    else if ( event->type == FocusOut && Out_HasFocus(o) )
     {
+	if (Text_LosingFocusCallback(w))
+	{
+	    XmTextVerifyCallbackStruct cbs;
+	    cbs.reason = XmCR_LOSING_FOCUS;
+	    cbs.event = event;
+	    cbs.currInsert = cbs.newInsert = Text_CursorPos(w);
+	    cbs.startPos = cbs.endPos = 0;
+	    cbs.text = NULL;
+
+	    XtCallCallbackList((Widget)w, Text_LosingFocusCallback(w), &cbs);
+	}
 	Out_HasFocus(o) = False;
 
 	XSetStipple(XtDisplay(aw), Out_CursorGC(o), Out_CursorStipple(o));
+	(*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), on);
 
 	if (Out_TimerId(o))
 	{
 	    XtRemoveTimeOut(Out_TimerId(o));
+	    Out_TimerId(o) = 0;
 	}
     }
 
-    (*Text_Output(w)->DrawInsertionPoint) (w, Text_CursorPos(w), on);
 }
 
 
@@ -1465,7 +1581,8 @@ static XmTextPosition
 XYToPos(XmTextWidget w, Position x, Position y)
 {
     LineNum row = YToLineNum(w, y);
-
+    if ( row == Text_LineCount(w))
+	return Text_LastPos(w);
     return XToPos(w, row, x);
 }
 
@@ -1548,9 +1665,7 @@ MakePositionVisible(XmTextWidget w, XmTextPosition pos)
 
     start = Text_LineTable(w)[line].start_pos;
     xpos = FontTextWidth(w, start, pos);
-    last_xpos = XtWidth(w) - 2 * (Prim_HighlightThickness(w) +
-				  Prim_ShadowThickness(w) +
-				  Text_MarginWidth(w));
+    last_xpos = XtWidth(w) - ( Out_RightMargin(o) + Out_LeftMargin(o) );
 
     if (xpos - Out_XOffset(o) > last_xpos)
     {
@@ -1613,7 +1728,7 @@ MeasureLine(XmTextWidget w, LineNum num, XmTextPosition pos,
     if (start < Text_LastPos(w))
     {
 	end = (*Text_Source(w)->Scan) (Text_Source(w), pos,
-				       XmSELECT_LINE, XmsdRight, -1, False);
+				       XmSELECT_LINE, XmsdRight, 1, False);
     }
     else
     {
@@ -1698,6 +1813,42 @@ DrawInsertionPoint(XmTextWidget w, XmTextPosition pos, OnOrOff state)
     OutputData o = Text_OutputData(w);
     Position x, y;
 
+#if 1
+/* rws 1 Apr 2000
+WRT the disappearing cursor....
+
+HandleTimer seems to be getting called when it should. This takes care of
+setting Out_BlinkState, then calls DrawInsertionPoint, which is also
+messing with Out_BlinkState. I'm not sure what Out_BlinkOn is keeping track
+of, but I would expect the two if clauses near the start of
+DrawInsertionPoint to be a little more symetrical wrt Out_BlinkOn and
+Out_BlinkState. Commenting out the two lines messing with the BlinkState
+seems to keep my cursor from disappearing. I've bracketted the section in
+question with #if 1, it needs a little more investigation.....
+*/
+    if ( state == off )
+    {
+	if ( Out_BlinkOn(o) == 0 )
+	{
+	    Out_BlinkState(o) = off;
+	}
+	Out_BlinkOn(o) -- ;
+    }
+    if ( state == on )
+    {
+	if ( Out_BlinkOn(o) < 0 )
+	{
+	    Out_BlinkOn(o)++ ;
+	}
+	Out_BlinkState(o) = on;
+    }
+#endif
+
+    if ( Out_BlinkOn(o) != 0 )
+    {
+	return;
+    }
+
     if (pos == PASTENDPOS)	/* No No No : don't want this */
     {
 	pos = Text_LastPos(w);	/* ??? Danny 6/7/97 */
@@ -1741,7 +1892,7 @@ DrawInsertionPoint(XmTextWidget w, XmTextPosition pos, OnOrOff state)
 	if ((*Text_Output(w)->PosToXY) (w, pos, &x, &y))
 	{
 	    Out_CursorX(o) = x;
-	    Out_CursorY(o) = y - Out_FontAscent(o) + 1;
+	    Out_CursorY(o) = y;
 
 #if 0
 	/*
@@ -1782,6 +1933,10 @@ Draw(XmTextWidget w, LineNum num, XmTextPosition start, XmTextPosition end,
     XmTextBlockRec block;
     Line line = &Text_Line(w)[num];
 
+    if (start > Text_LastPos(w)) {
+	return;
+    }
+
     if (end > Text_LastPos(w))
     {
 	end = Text_LastPos(w);
@@ -1833,13 +1988,11 @@ GetPreferredSize(Widget aw, Dimension *width, Dimension *height)
     XmTextWidget w = (XmTextWidget)aw;
     OutputData o = Text_OutputData(w);
 
-    *width = (2 * Prim_ShadowThickness(w) + 2 * Prim_HighlightThickness(w) +
-	      2 * Text_MarginWidth(w) +
-	      Out_Columns(o) * Out_FontAverageWidth(o));
+    *width = Out_LeftMargin(o) + Out_RightMargin(o) +
+	      Out_Columns(o) * Out_FontAverageWidth(o) ;
 
-    *height = (2 * Prim_ShadowThickness(w) + 2 * Prim_HighlightThickness(w) +
-	       2 * Text_MarginHeight(w) +
-	       Out_Rows(o) * Out_FontHeight(o));
+    *height = Out_TopMargin(o) + Out_BottomMargin(o) +
+	       Out_Rows(o) * Out_FontHeight(o);
 }
 
 static void
@@ -1927,6 +2080,17 @@ OutputSetValues(Widget old, Widget request, Widget new,
     XtSetSubvalues(iw, output_resources, XtNumber(output_resources),
 		   args, *num_args);
 
+    Out_RightMargin(nd) = Out_LeftMargin(nd) = Prim_ShadowThickness(new) + 
+		Prim_HighlightThickness(new) + Text_MarginWidth(new) ;
+    Out_TopMargin(nd) = Out_BottomMargin(nd) = Prim_ShadowThickness(new) + 
+		Prim_HighlightThickness(new) + Text_MarginHeight(new) ;
+
+    if (Out_RightMargin(od) != Out_RightMargin(nd) ||  Out_LeftMargin(nd) !=  Out_LeftMargin(od) 
+	|| Out_TopMargin(nd) != Out_TopMargin(od) || Out_BottomMargin(nd) != Out_BottomMargin(od))
+    {
+	(*Text_Output(old)->resize) (new, True);          
+	redraw = True;
+    }
     /* Check if we need to act */
 
     if (Out_FontList(od) != Out_FontList(nd))
@@ -1934,12 +2098,43 @@ OutputSetValues(Widget old, Widget request, Widget new,
 	DEBUGOUT(XdbDebug(__FILE__, new,
 			  "OutputSetValues: FontList change.\n"));
 
+	/* T. Straumann: make a copy of the font list and free the old one */
+	Out_FontList(nd) = XmFontListCopy(Out_FontList(nd));
+	/* rws 17 Jul 1999
+	   But only free it if we created it....
+	   xmgrace Data->Results
+	*/
+	if (Out_FontListCreated(od) && Out_FontList(od))
+	{
+	    XmFontListFree(Out_FontList(od));
+	}
+
 	FontInit((XmTextWidget)new);
 	/* From OutputCreate : */
 	XtHeight(new) = Out_FontHeight(nd)
 	    + 2 * (Prim_ShadowThickness(new) + Text_MarginHeight(new));
 	XtWidth(new) = Out_Columns(nd) * Out_FontAverageWidth(nd)
-	    + 2 * (Prim_ShadowThickness(new) + Text_MarginWidth(new));
+		+ Out_LeftMargin(nd) + Out_RightMargin(nd) ;
+
+	if (Out_DrawGC(nd)) {
+	    /*
+	     * We need to change the font id in the DrawGC,
+	     * not the other GC's.
+	     *
+	     * Note we can only do this because the mask used when
+	     * we got this GC specifies that we can change the font.
+	     */
+	    XGCValues	values;
+
+	    values.font = Out_Font(nd)->fid;
+	    XChangeGC(XtDisplay(new), Out_DrawGC(nd), GCFont, &values);
+
+	    /* We may still need to do things here.
+	     * CP Hennessy suggest calling
+		GCClip((XmTextWidget)new);
+	     */
+	}
+
 	redraw = True;
     }
 
@@ -1947,8 +2142,8 @@ OutputSetValues(Widget old, Widget request, Widget new,
     {
 	DEBUGOUT(XdbDebug(__FILE__, new,
 			  "OutputSetValues: WordWrap change.\n"));
-
-	/* FIX ME */
+		/*CP:15 May 1999:rebuild the line table */
+	_XmTextUpdateLineTable(new, 0, 0, NULL, False); 
 	redraw = True;
     }
 
@@ -1969,7 +2164,7 @@ OutputSetValues(Widget old, Widget request, Widget new,
 
 	/* FIX ME ? */
 	XtHeight(new) = Out_Rows(nd) * Out_FontHeight(nd)
-	    + 2 * (Prim_ShadowThickness(new) + Text_MarginHeight(new));
+	    + Out_TopMargin(nd) + Out_BottomMargin(nd) ;
 
 	redraw = True;
     }
@@ -1980,11 +2175,31 @@ OutputSetValues(Widget old, Widget request, Widget new,
 
 	/* FIX ME ? */
 	XtWidth(new) = Out_Columns(nd) * Out_FontAverageWidth(nd)
-	    + 2 * (Prim_ShadowThickness(new) + Text_MarginWidth(new));
+		+ Out_LeftMargin(nd) + Out_RightMargin(nd);
 
 	redraw = True;
     }
 
+    if ( XtBackground(old) != XtBackground(new) )
+    {
+	if (Out_DrawGC(nd)) {
+	    XGCValues	values;
+
+	    values.background =  XtBackground(new);
+	    XChangeGC(XtDisplay(new), Out_DrawGC(nd), GCBackground, &values );
+	}
+    }
+
+    if ( Prim_Foreground(old) != Prim_Foreground(new) )
+    {
+	if (Out_DrawGC(nd)) {
+	    XGCValues	values;
+
+	    values.foreground =  Prim_Foreground(new);
+	    XChangeGC(XtDisplay(new), Out_DrawGC(nd), GCForeground, &values );
+	}
+    }
+
     if (Out_BlinkRate(od) != Out_BlinkRate(nd))
     {
 	/* FIX ME */
@@ -2038,6 +2253,7 @@ destroy(Widget aw)
     if (o && Out_TimerId(o))
     {
 	XtRemoveTimeOut(Out_TimerId(o));
+	Out_TimerId(o) = 0;
     }
 
     XtReleaseGC((Widget)w, Out_DrawGC(o));
@@ -2057,6 +2273,11 @@ destroy(Widget aw)
 	XFreePixmap(XtDisplay((Widget)w), Out_CursorSave(o));
     }
 
+	if (Out_FontListCreated(o) && Out_FontList(o))
+	{
+	XmFontListFree(Out_FontList(o));
+	}
+
     XtFree((char *)Text_Output(w));
 }
 
@@ -2086,6 +2307,15 @@ OutputExpose(Widget aw, XEvent *event, Region region)
 
     DrawAll(w);
 
+    if (Prim_Highlighted(aw))
+    {
+	(*PrimC_BorderHighlight(XtClass(aw))) (aw);
+    }
+    else
+    {
+	(*PrimC_BorderUnhighlight(XtClass(aw))) (aw);
+    }
+
     _XmDrawShadows(XtDisplay(w),
 		   XtWindow(w),
 		   Prim_TopShadowGC(w),
@@ -2191,30 +2421,39 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
 
     Out_XOffset(o) = 0;
 
-    XtAddCallback(aw, XmNfocusCallback,
-		  HandleFocusEvents,
-		  (XtPointer)w);
-    XtAddCallback(aw, XmNlosingFocusCallback,
-		  HandleFocusEvents,
-		  (XtPointer)w);
+    XtAddEventHandler(aw, FocusChangeMask, False, HandleFocusEvents, NULL);
 
     CursorInit(w);
 
     /* Use DrawGC as a flag to see if the GCs have been created */
     Out_DrawGC(o) = NULL;
+    Out_RightMargin(o) = Out_LeftMargin(o) = Prim_ShadowThickness(w) + Prim_HighlightThickness(w)
+		+ Text_MarginWidth(w) ;
+    Out_TopMargin(o) = Out_BottomMargin(o) = Prim_ShadowThickness(w) + Prim_HighlightThickness(w)
+		+ Text_MarginHeight(w) ;
 
 
+    Out_CursorWidth(o) = 5; /*CP: M*tif seems to use 5, but why ?? */
+
     /* Get the font information */
     if (Out_FontList(o) == (XmFontList)XmUNSPECIFIED || !Out_FontList(o))
     {
 	Out_FontList(o) = _XmGetDefaultFontList((Widget)w, XmTEXT_FONTLIST);
-	Out_FontListCreated(o) = False;
+	/* T. Straumann: changed this to 'true'; XmGetDefaultFontList() seems
+	 *				 to make a copy.
+	 */
+	Out_FontListCreated(o) = True;
 	/* we didn't create this, so don't free it */
     }
     else
     {
 	Out_FontListCreated(o) = True;
+	Out_FontListCreated(o) = False;
 	/* we created this, so free it when the time comes */
+	/* rws 12 Apr 1999
+	   If we created this, where???? A True value here is causing ml to
+	   segfault in FontListFree whenever the message window is destroyed.
+	 */
     }
 
     FontInit(w);
@@ -2234,9 +2473,8 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
     else
     {
 	/* Reverse of this thing also copied into OutputSetValues */
-	Out_Columns(o) = (XtWidth(w)
-			  - 2 * Prim_ShadowThickness(w)
-			  - 2 * Text_MarginWidth(w)) / Out_FontAverageWidth(o);
+	Out_Columns(o) = (XtWidth(w) - ( Out_LeftMargin(o) + Out_RightMargin(o) )
+		) / Out_FontAverageWidth(o);
     }
 
     if (XtHeight(w) == (Dimension)0)
@@ -2248,9 +2486,8 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
     }
     else
     {
-	Out_Rows(o) = (XtHeight(w)
-		       - 2 * Prim_ShadowThickness(w)
-		       - 2 * Text_MarginHeight(w)) / Out_FontHeight(o);
+	Out_Rows(o) = (XtHeight(w) - ( Out_TopMargin(o) + Out_BottomMargin(o) )
+		) / Out_FontHeight(o);
     }
 
     if (Text_EditMode(w) == XmSINGLE_LINE_EDIT)
@@ -2258,17 +2495,19 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
 	Out_Rows(o) = 1;
 	if (XtHeight(w) != 0 &&
 	    (XtHeight(w) < Out_FontHeight(o)
-	     + 2 * (Prim_ShadowThickness(w) + Text_MarginHeight(w))))
+		+ Out_TopMargin(o) + Out_BottomMargin(o) ) )
 	{
 	    /* Statement below also copied into OutputSetValues */
 	    XtHeight(w) = Out_FontHeight(o)
-		+ 2 * (Prim_ShadowThickness(w) + Text_MarginHeight(w));
+		+ Out_TopMargin(o) + Out_BottomMargin(o);
 	}
     }
 
     DEBUGOUT(XdbDebug(__FILE__, aw,
 		      "OutputCreate: rows %d cols %d\n",
 		      Out_Rows(o), Out_Columns(o)));
+    Out_ColumnsSet(o) = Out_Columns(o);
+    Out_RowsSet(o) = Out_Rows(o);
 
     if (XtWidth(w) == 0 || XtHeight(w) == 0)
     {
@@ -2339,6 +2578,15 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
 			  HandleHBar,
 			  (XtPointer)w);
 	    XtManageChild(Out_Hbar(o));
+
+	    /*CP:Friday 14 May 1999: Set HBar resources to initially sane values */
+	    XtVaSetValues(Out_Hbar(o),
+			  XmNincrement, 1,
+			  XmNpageIncrement, 1,
+			  XmNsliderSize, 1,
+			  XmNmaximum, 1, 
+			  XmNminimum, 0,
+			  NULL);
 	}
 	else
 	{
@@ -2390,6 +2638,7 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
 			  XmNincrement, 1,
 			  XmNpageIncrement, Out_Rows(o),
 			  XmNsliderSize, Out_Rows(o),
+			  XmNmaximum, Out_Rows(o), /*CP:Friday 14 May 1999 a reasonable initial value */
 			  NULL);
 	}
 	else
@@ -2464,7 +2713,7 @@ _XmTextShouldWordWrap(XmTextWidget w)
 {
     OutputData o = Text_OutputData(w);
 
-    if (!o->wordwrap)
+    if (! Out_WordWrap(o) )
     {
 	return False;
     }
@@ -2472,12 +2721,12 @@ _XmTextShouldWordWrap(XmTextWidget w)
     {
 	return False;
     }
-    if (o->scrollhorizontal &&
+    if ( Out_ScrollHorizontal(o) &&
 	XtClass(XtParent(w)) == xmScrolledWindowWidgetClass)
     {
 	return False;
     }
-    if (o->resizewidth)
+    if ( Out_ResizeWidth(o) )
     {
 	return False;
     }
@@ -2546,13 +2795,13 @@ _XmTextFindLineEnd(XmTextWidget w, XmTextPosition pos, LineTableExtra *extra)
 {
     Dimension avail, width, oldwidth;
     XmTextPosition wordend, last, end;
+    OutputData o = Text_OutputData(w);
 
-    avail = XtWidth(w) - 2 * (Prim_ShadowThickness(w) + Text_MarginWidth(w)
-			      + Prim_HighlightThickness(w));
+    avail = XtWidth(w) - (Out_RightMargin(o) + Out_LeftMargin(o));
 
     /* What's our line ? */
     end = (*Text_Source(w)->Scan) (Text_Source(w), pos,
-				   XmSELECT_LINE, XmsdRight, -1, False);
+				   XmSELECT_LINE, XmsdRight, 1, False);
 
     /* Do we need to spend time on this; the line may be short enough */
     width = FontTextWidth(w, pos, end);
@@ -2572,7 +2821,7 @@ _XmTextFindLineEnd(XmTextWidget w, XmTextPosition pos, LineTableExtra *extra)
 	last = wordend;
 	wordend = (*Text_Source(w)->Scan) (Text_Source(w), last + 1,
 					   XmSELECT_WORD,
-					   XmsdRight, -1, False);
+					   XmsdRight, 1, False);
 	oldwidth = width;
 	width = FontTextWidth(w, pos, wordend);
     }
@@ -2612,7 +2861,7 @@ void
 _XmTextMovingCursorPosition(XmTextWidget tw, XmTextPosition position)
 {
     OutputData o = Text_OutputData(tw);
-    (*Text_Output(tw)->DrawInsertionPoint) (tw, position, Out_BlinkState(o));
+/*CP    (*Text_Output(tw)->DrawInsertionPoint) (tw, position, Out_BlinkState(o)); */
 }
 
 Boolean
