@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextStrSo.c,v 1.38 2000/01/09 22:34:54 hennessy Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/TextStrSo.c,v 1.46 2000/04/02 14:08:23 rwscott Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: TextStrSo.c,v 1.38 2000/01/09 22:34:54 hennessy Exp $";
+static const char rcsid[] = "$Id: TextStrSo.c,v 1.46 2000/04/02 14:08:23 rwscott Exp $";
 
 /*
  * Empirical evidence (fooling around with the structures found in Motif
@@ -44,6 +44,7 @@ static const char rcsid[] = "$Id: TextStrSo.c,v 1.38 2000/01/09 22:34:54 henness
 #include <LTconfig.h>
 #include <Xm/XmP.h>
 #include <Xm/TextP.h>
+#include <Xm/VendorSEP.h>
 #include <X11/Xfuncs.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -323,74 +324,134 @@ Replace(XmTextWidget w,
 {
     XmSourceData d;
     int i;
-    XmTextPosition start, end;
+    XmTextPosition start, end, lastPos = 0;
     XmTextVerifyCallbackStruct cbs;
 
-    d = Text_Source(w)->data;
-
-    start = *startret;
-    end = *endret;
-
-    if (start > end)
+    if (!call_callback || _XmTextModifyVerify(w, ev, startret, endret, &Text_CursorPos(w), block, NULL, False))
     {
-	int swap;
+	d = Text_Source(w)->data;
 
-	swap = start;
-	start = end;
-	end = swap;
-    }
+	start = *startret;
+	end = *endret;
 
-    if (end > d->length)
-    {
-	end = d->length;
-    }
-
-    if (start > d->length)
-    {
-	return EditError;
-    }
-
-    if ( ( start >= end ) &&
-               d->length + (end - start) + block->length > d->maxallowed)
-    {
-	return EditError;
-    }
-
-
-    for (i = 0; i < d->numwidgets; i++)
-    {
-	_XmTextDisableRedisplay(d->widgets[i], True);
-    }
-
-    if (end > start)
-    {				/* we need to delete some stuff */
-	char *dest, *source, *last;
-
-	dest = d->ptr + start;
-	source = d->ptr + end;
-	last = d->ptr + d->length;
-	while (source < last)
+	if (start > end)
 	{
-	    *dest++ = *source++;
+	    int swap;
+
+	    swap = start;
+	    start = end;
+	    end = swap;
 	}
 
-	d->length -= end - start;
-    }
+	if (end > d->length)
+	{
+	    end = d->length;
+	}
 
-    if (block && block->length > 0)
-    {
-	CheckSize(d, block->length);
-	Insert(d, start, block->ptr, block->length);
-    }
+	if (start > d->length)
+	{
+	    return EditError;
+	}
 
-    for (i = 0; i < d->numwidgets; i++)
-    {
-	Text_LastPos(d->widgets[i]) = d->length;
-    }
+	if ( ( start >= end ) &&
+		   d->length + (end - start) + block->length > d->maxallowed)
+	{
+	    return EditError;
+	}
 
-    /* Callback : only do this for the widget activating this ! */
-    if (call_callback)
-    {
+	for (i = 0; i < d->numwidgets; i++)
+	{
+	    _XmTextDisableRedisplay(d->widgets[i], True);
+	}
+
+	if (end > start)
+	{				/* we need to delete some stuff */
+	    char *dest, *source, *last;
+
+	    dest = d->ptr + start;
+	    source = d->ptr + end;
+	    last = d->ptr + d->length;
+	    while (source < last)
+	    {
+		*dest++ = *source++;
+	    }
+
+	    d->length -= end - start;
+	}
+
+	if (block && block->length > 0)
+	{
+	    CheckSize(d, block->length);
+	    Insert(d, start, block->ptr, block->length);
+	}
+
+	for (i = 0; i < d->numwidgets; i++)
+	{
+	    XmTextPosition startPos = (*Text_Source(w)->Scan)(Text_Source(w), 0, XmSELECT_ALL, XmsdLeft, 1, False ) ;
+	    lastPos = (*Text_Source(w)->Scan)(Text_Source(w), startPos, XmSELECT_ALL, XmsdRight, 1, False ) ;
+	}
+
+       if ( d -> hasselection )
+	    {
+	    /* rws 15 Jan 2000
+	       What if only part of the selection is being replaced???
+	    if ( start == d->left && end == d-> right )
+	    */
+	    if ( start <= d->right && end >= d->left )
+		{
+		    /* CP:15 May 1999: If the text we are replacing is the
+		    ** selected text then unselect it. */
+		d->hasselection = False;
+		for (i = 0; i < d->numwidgets; i++)
+		  {
+#if 0
+		    _XmHighlightRecNode* p;
+		    p = Text_Highlight(d->widgets[i]).list;
+		    while (p) {
+		      _XmHighlightRecNode* k=p;
+		      p = p->next;
+		      XtFree((XtPointer)k);
+		    }
+		    Text_Highlight(d->widgets[i]).list = NULL;
+#else
+		    Text_Highlight(d->widgets[i]).number = 0;
+#endif
+		  }
+		}
+	    /*CP:15 May 1999:
+		    at this point we do not want to overwrite the selection
+		    but just to move its coordinates if we are inserting
+		    text before the selection. */
+
+	    else if ( start <  d-> left )
+		    {
+		    d-> left += block -> length;
+		    d-> right += block -> length;
+		    for (i = 0; i < d->numwidgets; i++)
+			    {
+			    XmTextSetHighlight((Widget)d->widgets[i],
+				    d-> left, d->right, XmHIGHLIGHT_SELECTED);
+			    }
+		    }
+	    }
+	/* End selection */
+
+	if (Text_CursorPos(w) == Text_LastPos(w))
+	{
+	    if (call_callback)
+	    {
+		Text_LastPos(w) = lastPos;
+	    }
+	    if (lastPos != Text_CursorPos(w))
+	    {
+		_XmTextSetCursorPosition((Widget)w, lastPos);
+	    }
+	}
+	Text_LastPos(w) = lastPos;
+	if (Text_CursorPos(w) > Text_LastPos(w))
+	{
+	    Text_CursorPos(w) = Text_LastPos(w);
+	}
 	if (Text_ValueChangedCallback(w))
 	{
 	    cbs.reason = XmCR_VALUE_CHANGED;
@@ -400,79 +461,38 @@ Replace(XmTextWidget w,
 
 	    XtCallCallbacks((Widget)w, XmNvalueChangedCallback, &cbs);
 	}
-    }
 
-   if ( d -> hasselection )
+	for (i = 0; i < d->numwidgets; i++)
 	{
-	if ( start == d->left && end == d-> right )
-	    {
-		/* CP:15 May 1999: If the text we are replacing is the
-		** selected text then unselect it. */
-	    d->hasselection = False;
-	    for (i = 0; i < d->numwidgets; i++)
-	      {
+	    _XmTextUpdateLineTable((Widget)d->widgets[i], start, end, block, True);
+	}
+
+	for (i = 0; i < d->numwidgets; i++)
+	{
+	    _XmTextInvalidate(d->widgets[i], start, end,
+			      block->length - end + start);
+	}
+
 #if 0
-		_XmHighlightRecNode* p;
-		p = Text_Highlight(d->widgets[i]).list;
-		while (p) {
-		  _XmHighlightRecNode* k=p;
-		  p = p->next;
-		  XtFree((XtPointer)k);
-		}
-		Text_Highlight(d->widgets[i]).list = NULL;
-#else
-		Text_Highlight(d->widgets[i]).number = 0;
-#endif
-	      }
+	/* Update the insertion position */
+	for (i = 0; i < d->numwidgets; i++)
+	{
+	    XmTextPosition	p = Text_CursorPos(d->widgets[i]);
+
+	    if (p < *startret) {
+		    /* no action */
+	    } else if (p < *endret) {
+		    _XmTextSetCursorPosition(d->widgets[i], *startret);
+	    } else {
+		    _XmTextSetCursorPosition(d->widgets[i], p + block->length);
 	    }
-	/*CP:15 May 1999:
-		at this point we do not want to overwrite the selection
-		but just to move its coordinates if we are inserting
-		text before the selection. */
-
-	else if ( start <  d-> left )
-		{
-		d-> left += block -> length;
-		d-> right += block -> length;
-		for (i = 0; i < d->numwidgets; i++)
-			{
-			XmTextSetHighlight((Widget)d->widgets[i],
-				d-> left, d->right, XmHIGHLIGHT_SELECTED);
-			}
-		}
 	}
-    /* End selection */
-
-    for (i = 0; i < d->numwidgets; i++)
-    {
-	_XmTextUpdateLineTable((Widget)d->widgets[i], start, end, block, True);
-    }
-
-    for (i = 0; i < d->numwidgets; i++)
-    {
-	_XmTextInvalidate(d->widgets[i], start, end,
-			  block->length - end + start);
-    }
-
-#if 0
-    /* Update the insertion position */
-    for (i = 0; i < d->numwidgets; i++)
-    {
-	XmTextPosition	p = Text_CursorPos(d->widgets[i]);
-
-	if (p < *startret) {
-		/* no action */
-	} else if (p < *endret) {
-		_XmTextSetCursorPosition(d->widgets[i], *startret);
-	} else {
-		_XmTextSetCursorPosition(d->widgets[i], p + block->length);
-	}
-    }
 #endif
 
-    for (i = 0; i < d->numwidgets; i++)
-    {
-	_XmTextEnableRedisplay(d->widgets[i]);
+	for (i = 0; i < d->numwidgets; i++)
+	{
+	    _XmTextEnableRedisplay(d->widgets[i]);
+	}
     }
 
     return EditDone;
@@ -505,7 +525,10 @@ Scan(XmTextSource source, XmTextPosition pos, XmTextScanType type,
 
     if (pos > d->length)
     {
+#if 0
+	/* This is reported to be a bad idea. */
 	pos = d->length;
+#endif
 	return(pos);
     }
 
@@ -951,7 +974,10 @@ _XmStringSourceGetValue(XmTextSource source, Boolean want_wchar)
 	DEBUGOUT(XdbDebug(__FILE__, NULL,
 	   "_XmStringSourceGetValue: wide characters not implemented yet.\n"));
 
-	return NULL;
+	/* Returning NULL seems to do strange things */
+	txt = XtMalloc(1);
+	txt[0] = '\0';
+	return txt;
     }
 
     txt = XtMalloc(d->length + 1);
@@ -972,7 +998,8 @@ _XmStringSourceSetValue(XmTextWidget w, char *value)
     XmTextBlockRec tb;
     int len;
 
-    SetSelection(Text_Source(w),-1,-1,CurrentTime);
+    (Text_Source(w)->SetSelection)(Text_Source(w),1,0,
+		XtLastTimestampProcessed ( XtDisplay ( w ) ));
 
     len = value ? strlen(value) : 0;
 
@@ -980,9 +1007,9 @@ _XmStringSourceSetValue(XmTextWidget w, char *value)
     cbs.doit = True;
     cbs.event = NULL;
     cbs.startPos = 0;
-    cbs.endPos = Text_Source(w)->data->length;
+    cbs.endPos = Text_LastPos(w);
     cbs.currInsert = Text_CursorPos(w);
-    cbs.newInsert = len;
+    cbs.newInsert = Text_CursorPos(w);
 
     if (value)
     {
@@ -1017,7 +1044,20 @@ _XmStringSourceSetValue(XmTextWidget w, char *value)
 #endif
     if (cbs.doit)
     {
-	Replace(w, NULL, &cbs.startPos, &cbs.endPos, &tb, True);
+	XmTextStatus status ;
+	Text_BottomPos(w) = 0; /*CP: probably not the right value or place for initialization !*/
+	status = (Text_Source(w)->Replace)(w, NULL, &cbs.startPos, &cbs.endPos, &tb, False);
+        /* rws 2 Apr 2000
+           Done in Replace
+        if ( (status == EditDone) && ( Text_ValueChangedCallback(w)) )
+        {
+            XmAnyCallbackStruct any;
+            any.reason = XmCR_VALUE_CHANGED;
+            any.event = NULL;
+
+            XtCallCallbacks((Widget)w, XmNvalueChangedCallback, &any );
+        }
+        */
     }
 #if 0
     else {
@@ -1150,6 +1190,19 @@ _XmTextModifyVerify(XmTextWidget initiator,
 		    XmTextBlock newblock,
 		    Boolean *freeBlock)
 {
-    _XmWarning((Widget)initiator, "_XmTextModifyVerify is not implemented yet.");
-    return False;		/* FIX ME */
+XmTextVerifyCallbackStruct cbs;
+
+    cbs.reason = XmCR_MODIFYING_TEXT_VALUE;
+    cbs.event = event;
+    cbs.doit = True;
+    cbs.currInsert = *cursorPos;
+    cbs.newInsert = *cursorPos;
+    cbs.startPos = *start;
+    cbs.endPos = *end;
+    cbs.text = block;
+    if (Text_ModifyVerifyCallback(initiator))
+    {
+        XtCallCallbacks((Widget)initiator, XmNmodifyVerifyCallback, &cbs);
+    }
+    return(cbs.doit);
 }
