@@ -1,6 +1,6 @@
 /**
  *
- * $Id: TravAct.c,v 1.8 1998/09/12 15:05:48 rwscott Exp $
+ * $Id: TravAct.c,v 1.12 1999/11/19 20:20:59 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: TravAct.c,v 1.8 1998/09/12 15:05:48 rwscott Exp $";
+static const char rcsid[] = "$Id: TravAct.c,v 1.12 1999/11/19 20:20:59 amai Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -38,7 +38,6 @@ static const char rcsid[] = "$Id: TravAct.c,v 1.8 1998/09/12 15:05:48 rwscott Ex
 
 #include <XmI/DebugUtil.h>
 
-
 typedef struct _XmFocusFlag
 {
     struct _XDisplay *display;
@@ -46,8 +45,14 @@ typedef struct _XmFocusFlag
 }
 XmFocusFlagRec, *XmFocusFlag;
 
-XmFocusFlag flag_list = NULL;
-int flag_list_size = 0;
+static XmFocusFlag flag_list = NULL;
+static int flag_list_size = 0;
+
+/* For the prototype police */
+extern void _XmSetFocusFlag(Widget w, unsigned mask, Boolean value);
+extern unsigned short _XmGetFocusFlag(Widget w, unsigned int mask);
+extern void _XmPrimitiveFocusInInternal(Widget w, XEvent *event,
+					String *params, Cardinal *num_params);
 
 /*
  * apparently a flag is kept on a per- display basis.  Why, I don't
@@ -137,6 +142,19 @@ _XmTrackShellFocus(Widget widget,
 
     if (event->type == FocusIn || event->type == FocusOut)
     {
+	DEBUGOUT(XdbDebug("RWS", widget,"%s:_XmTrackShellFocus(%d) - %s\n",
+	    __FILE__, __LINE__,
+	    ve ? XtName(ve) : "NULL"
+	    ));
+	DEBUGOUT(XdbDebug("RWS", widget,
+		   "FOCUS EVENT: Shell: 0x%08x %s\n", widget, XtName(widget)));
+	DEBUGOUT(XdbDebug("RWS", widget,
+			  " type %s mode %s detail %s window %p(%s)\n",
+			  XdbEventType2String(fev->type), 
+			  XdbFocusMode2String(fev->mode), 
+			  XdbFocusDetail2String(fev->detail),
+			  fev->window,
+			  XtName(XtWindowToWidget(XtDisplay(widget), fev->window))));
 	DEBUGOUT(XdbDebug(__FILE__, widget,
 		   "FOCUS EVENT: Shell: 0x%08x %s\n", widget, XtName(widget)));
 	DEBUGOUT(XdbDebug(__FILE__, widget,
@@ -270,6 +288,13 @@ _XmTrackShellFocus(Widget widget,
 	}
     }
 
+    DEBUGOUT(XdbDebug("RWS", widget,"%s:_XmTrackShellFocus(%d) - %d %d %d %d %d %s %s\n",
+	    __FILE__, __LINE__,
+	    fd->focus_policy, XmEXPLICIT,
+	    focal_point, fd->focal_point, XmNO_RELATION,
+	    fd->focus_item ? XtName(fd->focus_item) : "NULL",
+	    fd->focus_item ? (CoreBeingDestroyed(fd->focus_item) ? "des" : "not-des") : ""
+	    ));
     if (fd->focus_policy == XmEXPLICIT && focal_point != fd->focal_point &&
 	fd->focus_item != NULL)
     {
@@ -277,15 +302,37 @@ _XmTrackShellFocus(Widget widget,
 	{
 	    if (fd->focal_point == XmNO_RELATION)
 	    {
+		DEBUGOUT(XdbDebug("RWS", widget,
+			"_XmTrackShellFocus: _XmCallFocusMoved(NULL, _)\n"));
 		DEBUGOUT(XdbDebug(__FILE__, widget,
 			"_XmTrackShellFocus: _XmCallFocusMoved(NULL, _)\n"));
-		_XmCallFocusMoved(NULL, fd->focus_item, event);
+		if (!CoreBeingDestroyed(fd->focus_item))
+		{
+		    _XmCallFocusMoved(NULL, fd->focus_item, event);
+		}
+		else
+		{
+		    _XmWarning(widget, "%s:_XmTrackShellFocus(%d) - bad news!!!!",
+		    	__FILE__, __LINE__);
+		    fd->focus_item = NULL;
+		}
 	    }
 	    else
 	    {
+		DEBUGOUT(XdbDebug("RWS", widget,
+			"_XmTrackShellFocus: _XmCallFocusMoved(_, NULL)\n"));
 		DEBUGOUT(XdbDebug(__FILE__, widget,
 			"_XmTrackShellFocus: _XmCallFocusMoved(_, NULL)\n"));
-		_XmCallFocusMoved(fd->focus_item, NULL, event);
+		if (!CoreBeingDestroyed(fd->focus_item))
+		{
+		    _XmCallFocusMoved(fd->focus_item, NULL, event);
+		}
+		else
+		{
+		    _XmWarning(widget, "%s:_XmTrackShellFocus(%d) - bad news!!!!",
+		    	__FILE__, __LINE__);
+		    fd->focus_item = NULL;
+		}
 	    }
 	}
     }
@@ -293,7 +340,7 @@ _XmTrackShellFocus(Widget widget,
     fd->focal_point = focal_point;
 }
 
-Boolean
+static Boolean
 set_pointer_item(Widget w, XEvent *event)
 {
     XCrossingEvent *ev = (XCrossingEvent *)event;
@@ -323,7 +370,7 @@ set_pointer_item(Widget w, XEvent *event)
     return True;
 }
 
-void
+static void
 flush_pointer_item(Widget w, XEvent *event)
 {
     XmFocusData fd = _XmGetFocusData(w);
@@ -356,10 +403,12 @@ _XmPrimitiveEnter(Widget w, XEvent *event,
 {
     XCrossingEvent *ev = (XCrossingEvent *)event;
 
+    _XmObjectLock(w);
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmPrimitiveEnter()\n"));
 
     if (_XmGetFocusPolicy(w) != XmPOINTER)
     {
+	_XmObjectUnlock(w);
 	return;
     }
 
@@ -370,6 +419,7 @@ _XmPrimitiveEnter(Widget w, XEvent *event,
     }
 
     set_pointer_item(w, event);
+    _XmObjectUnlock(w);
 }
 
 void
@@ -378,10 +428,12 @@ _XmPrimitiveLeave(Widget w, XEvent *event,
 {
     XCrossingEvent *ev = (XCrossingEvent *)event;
 
+    _XmObjectLock(w);
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmPrimitiveLeave()\n"));
 
     if (_XmGetFocusPolicy(w) != XmPOINTER)
     {
+	_XmObjectUnlock(w);
 	return;
     }
 
@@ -390,6 +442,7 @@ _XmPrimitiveLeave(Widget w, XEvent *event,
 	_XmCallFocusMoved(w, XtParent(w), event);
 	_XmWidgetFocusChange(w, XmLEAVE);
     }
+    _XmObjectUnlock(w);
 }
 
 void
@@ -398,12 +451,14 @@ _XmPrimitiveFocusInInternal(Widget w, XEvent *event,
 {
     XFocusChangeEvent *ev = (XFocusChangeEvent *)event;
 
+    _XmObjectLock(w);
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmPrimitiveFocusInInternal()\n"));
 
     if (!ev->send_event)
     {
 	DEBUGOUT(XdbDebug(__FILE__, w,
 			  "_XmPrimitiveFocusInInternal: !send_event\n"));
+	_XmObjectUnlock(w);
 	return;
     }
 
@@ -411,6 +466,7 @@ _XmPrimitiveFocusInInternal(Widget w, XEvent *event,
     {
 	DEBUGOUT(XdbDebug(__FILE__, w,
 			  "_XmPrimitiveFocusInInternal: FOCUS_IGNORE\n"));
+	_XmObjectUnlock(w);
 	return;
     }
 
@@ -422,11 +478,13 @@ _XmPrimitiveFocusInInternal(Widget w, XEvent *event,
 
 	if (!XtIsShell(XtParent(w)))
 	{
+	    _XmObjectUnlock(w);
 	    return;
 	}
 
 	flush_pointer_item(w, event);
 
+	_XmObjectUnlock(w);
 	return;
     }
 
@@ -438,10 +496,12 @@ _XmPrimitiveFocusInInternal(Widget w, XEvent *event,
 
 	_XmMgrTraversal(_XmFindTopMostShell(w), XmTRAVERSE_NEXT_TAB_GROUP);
 
+	_XmObjectUnlock(w);
 	return;
     }
 
     _XmWidgetFocusChange(w, XmFOCUS_IN);
+    _XmObjectUnlock(w);
 }
 
 void
@@ -450,35 +510,42 @@ _XmPrimitiveFocusOut(Widget w, XEvent *event,
 {
     XFocusChangeEvent *ev = (XFocusChangeEvent *)event;
 
+    _XmObjectLock(w);
     DEBUGOUT(XdbDebug(__FILE__, w,
 		      "_XmPrimitiveFocusOut(): send_event: %d\n",
 		      ev->send_event));
 
     if (!ev->send_event)
     {
+	_XmObjectUnlock(w);
 	return;
     }
 
     if (CoreBeingDestroyed(w))
     {
+	_XmObjectUnlock(w);
 	return;
     }
 
     if (_XmGetFocusPolicy(w) != XmEXPLICIT)
     {
+	_XmObjectUnlock(w);
 	return;
     }
 
     _XmWidgetFocusChange(w, XmFOCUS_OUT);
+    _XmObjectUnlock(w);
 }
 
 void
 _XmPrimitiveFocusIn(Widget w, XEvent *event,
 		    String *params, Cardinal *num_params)
 {
+    _XmObjectLock(w);
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmPrimitiveFocusIn()\n"));
 
     _XmPrimitiveFocusInInternal(w, event, params, num_params);
+    _XmObjectUnlock(w);
 }
 
 void
@@ -487,9 +554,11 @@ _XmPrimitiveUnmap(Widget w,
 		  String *params,
 		  Cardinal *num_params)
 {
+    _XmObjectLock(w);
     DEBUGOUT(XdbDebug(__FILE__, w, "_XmPrimitiveUnmap()\n"));
 
     _XmValidateFocus(w);
+    _XmObjectUnlock(w);
 }
 
 void
