@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Vendor.c,v 1.38 1998/04/07 20:38:18 u27113 Exp $
+ * $Id: Vendor.c,v 1.51 2000/01/26 12:43:53 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Vendor.c,v 1.38 1998/04/07 20:38:18 u27113 Exp $";
+static const char rcsid[] = "$Id: Vendor.c,v 1.51 2000/01/26 12:43:53 amai Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -69,7 +69,7 @@ static const char rcsid[] = "$Id: Vendor.c,v 1.38 1998/04/07 20:38:18 u27113 Exp
  *
  ***************************************************************************/
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -163,6 +163,8 @@ static void _XmVendorExtInitialize(Widget req, Widget new_w,
 static Boolean _XmVendorExtSetValues(Widget cw, Widget rw, Widget nw,
 				     ArgList args, Cardinal *nargs);
 
+static void _XmVendorExtDestroy(Widget w);
+
 #define Offset(field) XtOffsetOf(XmVendorShellExtRec, vendor.field)
 static XtResource ext_resources[] =
 {
@@ -356,7 +358,7 @@ XmVendorShellExtClassRec xmVendorShellExtClassRec = {
 	/* pad                */    False,
 	/* pad                */    False,
 	/* pad                */    False,
-	/* destroy            */    NULL,
+	/* destroy            */    _XmVendorExtDestroy,
 	/* pad                */    NULL,
 	/* pad                */    NULL,
 	/* set_values         */    NULL,
@@ -625,7 +627,7 @@ static Display *default_display = NULL;
 
 
 static void
-class_initialize()
+class_initialize(void)
 {
     /* we only do this here so that people can override the default reptypes,
        based on information from Harald Albrecht */
@@ -965,12 +967,11 @@ initialize(Widget req, Widget new_w,
 
     if (!XmIsDisplay(new_w))
     {
-	extern void _XmNSEEditResCheckMessages(Widget w, XtPointer data,
-					       XEvent *event, Boolean *cont);
-
+#ifdef LESSTIF_EDITRES
 	/* add the handler for editres messages */
 	XtAddEventHandler(new_w, (EventMask)0, True,
 			  (XtEventHandler)_XmNSEEditResCheckMessages, NULL);
+#endif
 
 	DEBUGOUT(XdbDebug(__FILE__, new_w,
 			  "Setting up virtual key bindings\n"));
@@ -1008,6 +1009,7 @@ destroy(Widget w)
     XmWidgetExtData data;
 
     DEBUGOUT(XdbDebug(__FILE__, w, "VendorShell Destroy()\n"));
+    DEBUGOUT(XdbDebug("NEDIT", w, "VendorShell Destroy()\n"));
 
     _XmPopWidgetExtData(w, &data, XmSHELL_EXTENSION);
     if (data != NULL) {
@@ -1019,6 +1021,45 @@ destroy(Widget w)
     }
 }
 
+static void
+_XmVendorExtDestroy(Widget w)
+{
+	XmVendorShellExtObject ve = (XmVendorShellExtObject)w;
+    Atom wm_delete_window;
+	Widget shell = ExtObj_LogicalParent(w);
+
+    DEBUGOUT(XdbDebug(__FILE__, w, "VendorShellExt Destroy()\n"));
+
+	if (VSEP_DefaultFontList(ve))
+	{
+	XmFontListFree(VSEP_DefaultFontList(ve));
+	}
+	if (VSEP_ButtonFontList(ve))
+	{
+	XmFontListFree(VSEP_ButtonFontList(ve));
+	}
+	if (VSEP_TextFontList(ve))
+	{
+	XmFontListFree(VSEP_TextFontList(ve));
+	}
+	if (VSEP_LabelFontList(ve))
+	{
+	XmFontListFree(VSEP_LabelFontList(ve));
+	}
+
+    if (VSEP_MwmMenu(ve) != NULL)
+    {
+	  XtFree((char*)VSEP_MwmMenu(ve));
+    }
+
+    XtFree((char*)VSEP_FocusData(ve));
+
+    wm_delete_window = XmInternAtom(XtDisplay(shell), _XA_WM_DELETE_WINDOW, False);
+
+	/* This removes all protocols and destroys the associated extension object */
+	_XmDestroyProtocols(shell);
+}
+
 
 static void
 realize(Widget w,
@@ -1187,10 +1228,11 @@ set_values(Widget old, Widget req, Widget new_w,
     Boolean refresh = False;
 
     DEBUGOUT(XdbDebug(__FILE__, new_w,
-		      "set_values: %i args\n"
+		      "%s:set_values(%d): %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
+		      __FILE__, __LINE__,
 		      *num_args,
 		      XtX(old), XtY(old),
 		      XtWidth(old), XtHeight(old),
@@ -1293,6 +1335,9 @@ static void
 delete_child(Widget w)
 {
     DEBUGOUT(XdbDebug(__FILE__, w, "delete_child()\n"));
+    DEBUGOUT(XdbDebug2("RWS", XtParent(w), w,"%s:delete_child(%d)\n",
+    	__FILE__, __LINE__
+    	));
     /* Keep those pesky objects OUT of the child list */
     if (!XtIsRectObj(w))
     {
@@ -1362,6 +1407,18 @@ LTAddGrab(Widget wid, Boolean exclusive, Boolean spring_loaded,
     }
 
     d = XmGetXmDisplay(XtDisplayOfObject(wid));
+    {
+    int i;
+    XmModalData ps = Display_Modals(d);
+
+	for (i = Display_NumModals(d); i > 0; i--, ps++)
+	{
+	    if (ps->wid == wid)
+	    {
+		return;
+	    }
+	}
+    }
     dump_grab_list(d);
 
     /* grow the modal list, if necessary */
@@ -1474,7 +1531,7 @@ LTRemoveGrab(Widget wid, XmVendorShellExtObject ve,
 
     for (i = Display_NumModals(d); i > 0; i--, ps++)
     {
-	if ((ps->wid == wid) && remove_grab_physically)
+	if ((ps->wid == wid) && !CoreBeingDestroyed(wid) && remove_grab_physically)
 	{
 	    XtRemoveGrab(wid);
 	}
@@ -1560,7 +1617,15 @@ LTRemoveGrabCallback(Widget wid, XtPointer client_data,
     if (!XtIsSubclass(wid, vendorShellWidgetClass))
 #endif
     {
+#if 0
+	/* rws 18 Sep 1999
+	   This seems to be the problem of the nedit File->Open problem
+	   Also ddd --separate-windows
+	 */
 	LTRemoveGrab(wid, (XmVendorShellExtObject)client_data, False);
+#else
+	LTRemoveGrab(wid, (XmVendorShellExtObject)client_data, True);
+#endif
     }
 }
 
@@ -2685,6 +2750,10 @@ geometry_manager(Widget wid,
     ShellWidget shell = (ShellWidget)XtParent(wid);
     XtWidgetGeometry req;
 
+    DEBUGOUT(XdbDebug2(__FILE__, XtParent(wid), wid, "%s:geometry_manager(%d) - request (%s)\n",
+    	__FILE__, __LINE__,
+    	XdbWidgetGeometry2String(request)));
+
     if ((request->width == 0 && (request->request_mode & CWWidth)) ||
 	(request->height == 0 && (request->request_mode & CWHeight)))
     {
@@ -2695,7 +2764,7 @@ geometry_manager(Widget wid,
 	return XtGeometryNo;
     }
 
-    if (shell->shell.allow_shell_resize == False && XtIsRealized(wid))
+    if (shell->shell.allow_shell_resize == False && XtIsRealized(wid) && (request->request_mode & (CWWidth | CWHeight)))
     {
 	DEBUGOUT(XdbDebug(__FILE__, wid,
 		          "Returning XtGeometryNo because "
@@ -2960,13 +3029,15 @@ _XmVendorExtSetValues(Widget cw, Widget rw, Widget nw,
 	    VSEP_MwmMenu(nw) = XtNewString(VSEP_MwmMenu(nw));
 	}
 
-	at = XmInternAtom(XtDisplay(nw), _XA_MWM_MENU, False);
-	/* note the 8 format */
-	XChangeProperty(XtDisplay(nw), XtWindow(nw), at, at, 8,
-			PropModeReplace,
-			(unsigned char *)VSEP_MwmMenu(nw),
-			VSEP_MwmMenu(nw) == NULL
-			? 0 : strlen(VSEP_MwmMenu(nw)));
+	if (VSEP_MwmMenu(nw) != NULL) {
+	    /* Only do this if we have a reason for it.
+	     * The Motif 2.1 mwm seems to crash without this. */
+	    at = XmInternAtom(XtDisplay(nw), _XA_MWM_MENU, False);
+	    /* note the 8 format */
+	    XChangeProperty(XtDisplay(nw), XtWindow(nw), at, at, 8,
+		PropModeReplace,
+		(unsigned char *)VSEP_MwmMenu(nw), strlen(VSEP_MwmMenu(nw)));
+	}
     }
 
     return False;
@@ -3016,12 +3087,14 @@ _XmVendorExtRealize(Widget w, XtPointer closure, XtPointer call_data)
 		    (unsigned char *)prop32,
 		    PROP_MOTIF_WM_HINTS_ELEMENTS);
 
-    at = XmInternAtom(XtDisplay(par), _XA_MWM_MENU, False);
-    XChangeProperty(XtDisplay(par), XtWindow(par), at, at, 8,
+    if (VSEP_MwmMenu(w)) {
+	/* Only do this if we have a reason for it.
+	 * The Motif 2.1 mwm seems to crash without this. */
+	at = XmInternAtom(XtDisplay(par), _XA_MWM_MENU, False);
+	XChangeProperty(XtDisplay(par), XtWindow(par), at, at, 8,
 		    PropModeReplace,
-		    (unsigned char *)VSEP_MwmMenu(w),
-		    VSEP_MwmMenu(w) == NULL
-		    ? 0 : strlen(VSEP_MwmMenu(w)));
+		    (unsigned char *)VSEP_MwmMenu(w), strlen(VSEP_MwmMenu(w)));
+    }
 
     for (i = 0; i < par->core.num_popups; i++)
     {
@@ -3123,8 +3196,8 @@ _LtFindVendorExt(Widget w)
    done by the _DLL_InitTerm-function under OS/2.
  */
 #ifdef __EMX__
-void
-_XmFixupVendorShell()
+static void
+_XmFixupVendorShell(void)
 {
     transientShellWidgetClass->core_class.superclass =
 	(WidgetClass)&vendorShellClassRec;
@@ -3135,7 +3208,7 @@ _XmFixupVendorShell()
 unsigned long
 _DLL_InitTerm(unsigned long mod_handle, unsigned long flag)
 {
-    if (flag == 0)
+    if (flag == 0)  /* initialize DLL */
     {
 	_XmFixupVendorShell();
     }
