@@ -1,6 +1,6 @@
 /*
 
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.45 1999/12/13 15:59:10 rwscott Exp $
+ * $Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.46 2000/02/03 17:55:37 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  *
@@ -22,7 +22,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.45 1999/12/13 15:59:10 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.46 2000/02/03 17:55:37 amai Exp $";
 
 /*
  * ** Danny had added a flag here, but I think the behavior he added is always
@@ -4269,19 +4269,27 @@ XmStringParseText(XtPointer text, XtPointer *text_end, XmStringTag tag,
 	XmTextType type, XmParseTable parse_table, Cardinal parse_count,
 	XtPointer call_data)
 {
-	return NULL;
+	return (XmString)NULL;
 }
 
+
+/* the following calls are somehow described on
+   http://www.cs.stedwards.edu/usr/man/info/C/a_doc_lib/motif/motifpg/motifpg70.htm
+*/
+		
 Cardinal
 XmStringToXmStringTable(XmString string, XmString break_comp, XmStringTable *table)
 {
-	return 0;
+	
+	_XmWarning(NULL, "XmStringToXmStringTable() is not implemented yet!\n");
+	return (Cardinal)0;
 }
 
 XmString
 XmStringTableToXmString(XmStringTable table, Cardinal count, XmString break_component)
 {
-	return NULL;
+	_XmWarning(NULL, "XmStringTableToXmString() is not implemented yet!\n");
+	return (XmString)NULL;
 }
 
 XtPointer *
@@ -4289,7 +4297,8 @@ XmStringTableUnparse(XmStringTable table, Cardinal count, XmStringTag tag,
 	XmTextType tag_type, XmTextType output_type, XmParseTable parse,
 	Cardinal parse_count, XmParseModel parse_model)
 {
-	return NULL;
+	_XmWarning(NULL, "XmStringTableUnparse() is not implemented yet!\n");
+	return (XtPointer *)NULL;
 }
 
 XmStringTable
@@ -4297,38 +4306,46 @@ XmStringTableParseStringArray(XtPointer *strings, Cardinal count,
 	XmStringTag tag, XmTextType type, XmParseTable parse,
 	Cardinal parse_count, XtPointer call_data)
 {
-	return NULL;
+	_XmWarning(NULL, "XmStringTableParseStringArray() is not implemented yet!\n");
+	return (XmStringTable)NULL;
 }
 
 XmString
 XmStringPutRendition(XmString string, XmStringTag rendition)
 {
-	return NULL;
+	_XmWarning(NULL, "XmStringPutRendition() is not implemented yet!\n");
+	return (XmString)NULL;
 }
 
 XmParseMapping
 XmParseMappingCreate(ArgList  arg_list, Cardinal arg_count)
 {
-	return NULL;
+	_XmWarning(NULL, "XmParseMappingCreate() is not implemented yet!\n");
+	return (XmParseMapping)NULL;
 }
 
 void
 XmParseMappingSetValues(XmParseMapping parse_mapping, ArgList arg_list, Cardinal arg_count)
 {
+	_XmWarning(NULL, "XmParseMappingSetValues() is not implemented yet!\n");
 }
 
 void
 XmParseMappingGetValues(XmParseMapping parse_mapping, ArgList arg_list, Cardinal arg_count)
 {
+	_XmWarning(NULL, "XmParseMappingGetValues() is not implemented yet!\n");
 }
 
 void
 XmParseMappingFree(XmParseMapping parse_mapping)
 {
+	_XmWarning(NULL, "XmParseMappingFree() is not implemented yet!\n");
 }
 
 void
 XmParseTableFree(XmParseTable parse_table, Cardinal parse_count)
 {
+	_XmWarning(NULL, "XmParseTableFree() is not implemented yet!\n");
 }
+
 #endif
