@@ -1,3 +1,5 @@
+#include <stdio.h>
+#include <Xm/Xm.h>
 #include <Xm/DragDrop.h>
 
 extern void transfer_bitmap();
@@ -38,8 +40,12 @@ import_bitmap(w, client_data, call_data)
 			XtSetArg(al[ac], XmNnumDropTransfers, 1); ac++;
 			XtSetArg(al[ac], XmNtransferProc, transfer_bitmap); ac++;
 			drop_data->operation = XmDROP_COPY;
+
+			fprintf(stderr, "Drop COPY has succeeded\n");
 		} else {
 			XtSetArg(al[ac], XmNtransferStatus, XmTRANSFER_FAILURE); ac++;
+
+			fprintf(stderr, "Drop has failed\n");
 		}
 	}
 	XmDropTransferStart(drop_data->dragContext, al, ac);
